/*
 * Copyright (c) 2017 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */


#include <infiniband/ibdiag/ibdiag_csv_out.h>


void CSVOut::SetCommentPos()
{
    IBDIAG_ENTER;

    idx_tbl_comment_pos = (size_t)sout.tellp() + string("# INDEX_TABLE ").size();
    sout << "# INDEX_TABLE " << CommentString(0, 0) << endl;
    sout << endl << endl;
    cur_CSV_line += 3;

    IBDIAG_RETURN_VOID;
}


void CSVOut::DumpStart(const char *name)
{
    IBDIAG_ENTER;

    cur_idx.name = name;
    sout << "START_" << name << endl;

    cur_idx.offset = (size_t)sout.tellp();
    cur_idx.line = ++cur_CSV_line;

    IBDIAG_RETURN_VOID;
}


void CSVOut::WriteBuf(string buf)
{
    IBDIAG_ENTER;

    sout << buf;
    const char* ptr = buf.c_str();
    while(*ptr)
    {
        if (*ptr == '\n')
            ++cur_CSV_line;
        ++ptr;
    }

    IBDIAG_RETURN_VOID;
}


void CSVOut::DumpEnd(const char *name)
{
    IBDIAG_ENTER;

    cur_idx.size = (size_t)sout.tellp() - cur_idx.offset;
    cur_idx.rows = cur_CSV_line - cur_idx.line - 1;
    index_table.push_back(cur_idx);

    sout << "END_" << name << endl;
    sout << endl << endl;
    cur_CSV_line += 3;

    IBDIAG_RETURN_VOID;
}


void CSVOut::DumpIndexTableCSV()
{
    IBDIAG_ENTER;

    char buf[256];

    size_t cur_offset = sout.tellp();

    sout << "START_" << SECTION_INDEX_TABLE << endl;
    sout << "Name," << " Offset," << " Size," << " Line," << " Rows" << endl;

    for (list_index_line::iterator it = index_table.begin(); it != index_table.end(); ++it) {
        sprintf(buf, "%s, %11ld, %11ld, %11ld, %11ld\n", it->name.c_str(), it->offset,
                it->size, it->line, it->rows);

        sout << buf;
    }

    sout << "END_" << SECTION_INDEX_TABLE << endl;

    sout.seekp(idx_tbl_comment_pos);
    sout << CommentString(cur_offset, cur_CSV_line);

    IBDIAG_RETURN_VOID;
}
