/*
 *  Copyright (c) 2004-2013 Mellanox Technologies LTD. All rights reserved
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */


#include "infiniband/ibdiag/ibdiag.h"
#include "infiniband/ibdiag/ibdiag_clbck.h"

extern IBDiagClbck ibDiagClbck;


/****************************************************/
/******************private methods*******************/
/****************************************************/
void IBDiag::DumpSMInfo(ofstream &sout)
{
    IBDIAG_ENTER;

    list_p_sm_info_obj p_sm_not_active_list;
    list_p_sm_info_obj p_sm_discover_list;
    list_p_sm_info_obj p_sm_standby_list;
    list_p_sm_info_obj p_sm_masters_list;
    list_p_sm_info_obj p_sm_unknown_list;

    sout << "ibdiagnet fabric SM report" << endl;

    //collect info
    for (list_p_sm_info_obj::iterator it = this->fabric_extended_info.getSMPSMInfoListRef().begin();
            it != this->fabric_extended_info.getSMPSMInfoListRef().end(); ++it) {
        switch ((*it)->smp_sm_info.SmState) {
        case IBIS_IB_SM_STATE_NOT_ACTIVE: {
            p_sm_not_active_list.push_back(*it);
            break;
        }
        case IBIS_IB_SM_STATE_DISCOVER: {
            p_sm_discover_list.push_back(*it);
            break;
        }
        case IBIS_IB_SM_STATE_STANDBY: {
            p_sm_standby_list.push_back(*it);
            break;
        }
        case IBIS_IB_SM_STATE_MASTER: {
            p_sm_masters_list.push_back(*it);
            break;
        }
        default: {
            p_sm_unknown_list.push_back(*it);
            break;
        }
        }
    }

    char buffer[1024];
    if (p_sm_masters_list.size() > 0) {
        sout << endl << "  SM - master" << endl;
        for (list_p_sm_info_obj::iterator it = p_sm_masters_list.begin();
                it != p_sm_masters_list.end(); ++it) {
            memset(buffer, 0, sizeof(buffer));
            sprintf(buffer, "    Port=" U32D_FMT " lid=" U16H_FMT " guid=" U64H_FMT \
                    " dev=" U32D_FMT " priority:" U32D_FMT,
                    (*it)->p_port->num,
                    (*it)->p_port->base_lid,
                    (*it)->p_port->guid_get(),
                    (*it)->p_port->p_node->devId,
                    (*it)->smp_sm_info.Priority);
            sout << buffer << endl;
        }
    }
    if (p_sm_standby_list.size() > 0) {
        sout << endl << "  SM - standby" << endl;
        for (list_p_sm_info_obj::iterator it = p_sm_standby_list.begin();
                it != p_sm_standby_list.end(); ++it) {
            memset(buffer, 0, sizeof(buffer));
            sprintf(buffer, "    Port=" U32D_FMT " lid=" U16H_FMT " guid=" U64H_FMT \
                    " dev=" U32D_FMT " priority:" U32D_FMT,
                    (*it)->p_port->num,
                    (*it)->p_port->base_lid,
                    (*it)->p_port->guid_get(),
                    (*it)->p_port->p_node->devId,
                    (*it)->smp_sm_info.Priority);
            sout << buffer << endl;
        }
    }
    if (p_sm_discover_list.size() > 0) {
        sout << endl << "  SM - discover" << endl;
        for (list_p_sm_info_obj::iterator it = p_sm_discover_list.begin();
                it != p_sm_discover_list.end(); ++it) {
            memset(buffer, 0, sizeof(buffer));
            sprintf(buffer, "    Port=" U32D_FMT " lid=" U16H_FMT " guid=" U64H_FMT \
                    " dev=" U32D_FMT " priority:" U32D_FMT,
                    (*it)->p_port->num,
                    (*it)->p_port->base_lid,
                    (*it)->p_port->guid_get(),
                    (*it)->p_port->p_node->devId,
                    (*it)->smp_sm_info.Priority);
            sout << buffer << endl;
        }
    }
    if (p_sm_not_active_list.size() > 0) {
        sout << endl << "  SM - not active" << endl;
        for (list_p_sm_info_obj::iterator it = p_sm_not_active_list.begin();
                it != p_sm_not_active_list.end(); ++it) {
            memset(buffer, 0, sizeof(buffer));
            sprintf(buffer, "    Port=" U32D_FMT " lid=" U16H_FMT " guid=" U64H_FMT \
                    " dev=" U32D_FMT " priority:" U32D_FMT,
                    (*it)->p_port->num,
                    (*it)->p_port->base_lid,
                    (*it)->p_port->guid_get(),
                    (*it)->p_port->p_node->devId,
                    (*it)->smp_sm_info.Priority);
            sout << buffer << endl;
        }
    }
    if (p_sm_unknown_list.size() > 0) {
         sout << endl << "  SM - unknown state" << endl;
         for (list_p_sm_info_obj::iterator it = p_sm_unknown_list.begin();
                 it != p_sm_unknown_list.end(); ++it) {
             memset(buffer, 0, sizeof(buffer));
             sprintf(buffer, "    Port=" U32D_FMT " lid=" U16H_FMT " guid=" U64H_FMT \
                     " dev=" U32D_FMT " priority:" U32D_FMT,
                     (*it)->p_port->num,
                     (*it)->p_port->base_lid,
                     (*it)->p_port->guid_get(),
                     (*it)->p_port->p_node->devId,
                     (*it)->smp_sm_info.Priority);
             sout << buffer << endl;
         }
     }

    p_sm_not_active_list.clear();
    p_sm_discover_list.clear();
    p_sm_standby_list.clear();
    p_sm_masters_list.clear();
    p_sm_unknown_list.clear();
    IBDIAG_RETURN_VOID;
}


/****************************************************/
/*******************public methods*******************/
/****************************************************/
//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS / ERR_CODE_DB_ERR / ERR_CODE_NO_MEM / ERR_CODE_FABRIC_ERROR
int IBDiag::BuildSMInfoDB(list_p_fabric_general_err& ports_errors)
{
    IBDIAG_ENTER;
    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    int rc = IBDIAG_SUCCESS_CODE;
    // go over all nodes:
    struct SMP_SMInfo curr_sm_info;
    struct SMP_PortInfo *p_curr_port_info = NULL;

    ibDiagClbck.Set(this, &fabric_extended_info, &ports_errors);
    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &ibDiagClbck;
    clbck_data.m_handle_data_func = IBDiagSMPSMInfoMadGetClbck;

    for (map_str_pnode::iterator nI = this->discovered_fabric.NodeByName.begin();
            nI != this->discovered_fabric.NodeByName.end();
            ++nI) {
        IBNode *p_curr_node = (*nI).second;
        if (!p_curr_node) {
            this->SetLastError("DB error - found null node in NodeByName map for key = %s", (*nI).first.c_str());
            IBDIAG_RETURN(IBDIAG_ERR_CODE_DB_ERR);
        }

        u_int8_t start_port_idx;
        u_int8_t end_port_idx;
        if (p_curr_node->type == IB_SW_NODE) {
            start_port_idx = 0;
            end_port_idx = 0;   //only query manage port
        } else {   //CA
            start_port_idx = 1;
            end_port_idx = p_curr_node->numPorts;
        }
        // go over all ports of this node, if SW than query only manage port
        // use unsigned int index to prevent overwrap
        for (unsigned int port_num = start_port_idx; port_num <= end_port_idx; ++port_num) {
            IBPort *p_curr_port = p_curr_node->getPort((phys_port_t)port_num);
            if (!p_curr_port)                                       //port not exist
                continue;
            //Skip ports that are down or have undefined state
            //or are not in the scope of the fabric.
            if ((port_num != 0) && !p_curr_port->is_data_worthy())
                continue;

            //get the port info for this port
            p_curr_port_info = this->fabric_extended_info.getSMPPortInfo(p_curr_port->createIndex);
            if (!p_curr_port_info) {
                this->SetLastError("DB error - found connected port=%s without SMPPortInfo", p_curr_port->getName().c_str());
                IBDIAG_RETURN(IBDIAG_ERR_CODE_DB_ERR);
            }
            if (!(p_curr_port_info->CapMsk & 0x00000002))           //no sm port
                continue;

            //get direct route for this port
            direct_route_t *p_curr_direct_route = this->GetDirectRouteByPortGuid(p_curr_port->guid_get());
            if (!p_curr_direct_route) {
                this->SetLastError("DB error - can't find direct route to node=%s", p_curr_node->getName().c_str());
                IBDIAG_RETURN(IBDIAG_ERR_CODE_DB_ERR);
            }

            clbck_data.m_data1 = p_curr_port;
            this->ibis_obj.SMPSMInfoMadGetByDirect(p_curr_direct_route,
                                                   &curr_sm_info, &clbck_data);
            /*
            if (this->ibis_obj.SMPSMInfoMadGetByDirect(p_curr_direct_route, &curr_sm_info)) {
                FabricErrPortNotRespond *p_curr_fabric_port_err =
                        new FabricErrPortNotRespond(p_curr_port, "SMPSMInfoMad");
                if (!p_curr_fabric_port_err) {
                    this->SetLastError("Failed to allocate FabricErrPortNotRespond");
                    IBDIAG_RETURN(IBDIAG_ERR_CODE_NO_MEM);
                }
                ports_errors.push_back(p_curr_fabric_port_err);
                rc = IBDIAG_ERR_CODE_FABRIC_ERROR;
                continue;           //we will try to proceed
            }
            int rc2 = this->fabric_extended_info.addSMPSMInfoObj(p_curr_port, curr_sm_info);
            if (rc2) {
                this->SetLastError("Failed to add SMInfoObj for port=%s, err=%s",
                        p_curr_port->getName().c_str(), this->fabric_extended_info.GetLastError());
                IBDIAG_RETURN(rc2);
            }
            */
            if(ibDiagClbck.GetState())
                goto exit;

        }
    }
exit:
    this->ibis_obj.MadRecAll();

    if (rc == IBDIAG_SUCCESS_CODE) {
        rc = ibDiagClbck.GetState();
        if (rc)
            this->SetLastError(ibDiagClbck.GetLastError());
        else if (!ports_errors.empty())
            rc = IBDIAG_ERR_CODE_FABRIC_ERROR;
    }

    IBDIAG_RETURN(rc);
}


//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS / ERR_CODE_NO_MEM / ERR_CODE_CHECK_FAILED
int IBDiag::CheckSMInfo(list_p_fabric_general_err& sm_errors)
{
    IBDIAG_ENTER;
    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    int rc = IBDIAG_SUCCESS_CODE;
    list_p_sm_info_obj p_sm_masters_list;
    list_p_sm_info_obj p_sm_not_masters_list;

    //find all master sm
    for (list_p_sm_info_obj::iterator it = this->fabric_extended_info.getSMPSMInfoListRef().begin();
            it != this->fabric_extended_info.getSMPSMInfoListRef().end(); ++it) {
        if ( ((*it)->smp_sm_info.SmState == IBIS_IB_SM_STATE_NOT_ACTIVE) ||
                ((*it)->smp_sm_info.SmState == IBIS_IB_SM_STATE_DISCOVER) ) {
            continue;           //nothing to do
        } else if ((*it)->smp_sm_info.SmState == IBIS_IB_SM_STATE_MASTER) {
            p_sm_masters_list.push_back(*it);
        } else if ((*it)->smp_sm_info.SmState == IBIS_IB_SM_STATE_STANDBY) {
            p_sm_not_masters_list.push_back(*it);
        } else {
            FabricErrSMUnknownState *p_curr_fabric_sm_err =
                    new FabricErrSMUnknownState((*it));
            if (!p_curr_fabric_sm_err) {
                this->SetLastError("Failed to allocate FabricErrSMUnknownState");
                rc = IBDIAG_ERR_CODE_NO_MEM;
                goto check_sm_exit;
            }
            sm_errors.push_back(p_curr_fabric_sm_err);
            rc = IBDIAG_ERR_CODE_CHECK_FAILED;
            continue;   //check next one
        }
    }

    //no master
    if (p_sm_masters_list.empty()) {
        FabricErrSMNotFound *p_curr_fabric_sm_err =  new FabricErrSMNotFound(NULL);
        if (!p_curr_fabric_sm_err) {
            this->SetLastError("Failed to allocate FabricErrSMNotFound");
            rc = IBDIAG_ERR_CODE_NO_MEM;
            goto check_sm_exit;
        }
        sm_errors.push_back(p_curr_fabric_sm_err);
        IBDIAG_RETURN(IBDIAG_ERR_CODE_CHECK_FAILED);
    }

    //one sm
    if (p_sm_masters_list.size() == 1) {
        //verify correct sm
        sm_info_obj_t *p_sm_master_obj = p_sm_masters_list.front();
        for (list_p_sm_info_obj::iterator it = p_sm_not_masters_list.begin();
                    it != p_sm_not_masters_list.end(); ++it) {
            //check priority
            if ( ((*it)->smp_sm_info.Priority > p_sm_master_obj->smp_sm_info.Priority) ||
                    (((*it)->smp_sm_info.Priority == p_sm_master_obj->smp_sm_info.Priority) &&
                            ((*it)->smp_sm_info.GUID < p_sm_master_obj->smp_sm_info.GUID)) ) {
                FabricErrSMNotCorrect *p_curr_fabric_sm_err =
                        new FabricErrSMNotCorrect((*it));
                if (!p_curr_fabric_sm_err) {
                    this->SetLastError("Failed to allocate FabricErrSMNotCorrect");
                    rc = IBDIAG_ERR_CODE_NO_MEM;
                    goto check_sm_exit;
                }
                sm_errors.push_back(p_curr_fabric_sm_err);
                rc = IBDIAG_ERR_CODE_CHECK_FAILED;
                break;  //already in error
            }
        }
    } else {    //more than one sm
        for (list_p_sm_info_obj::iterator it = p_sm_masters_list.begin();
                it != p_sm_masters_list.end(); ++it) {
            FabricErrSMManyExists *p_curr_fabric_sm_err =  new FabricErrSMManyExists((*it));
            if (!p_curr_fabric_sm_err) {
                this->SetLastError("Failed to allocate FabricErrSMManyExists");
                rc = IBDIAG_ERR_CODE_NO_MEM;
                goto check_sm_exit;
            }
            sm_errors.push_back(p_curr_fabric_sm_err);
            rc = IBDIAG_ERR_CODE_CHECK_FAILED;
        }
    }

check_sm_exit:
    p_sm_masters_list.clear();
    p_sm_not_masters_list.clear();
    IBDIAG_RETURN(rc);
}


//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS
int IBDiag::DumpSMInfoCSVTable(CSVOut &csv_out)
{
    IBDIAG_ENTER;
    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    csv_out.DumpStart(SECTION_SM_INFO);

    stringstream sstream;
    sstream << "NodeGUID," << "PortGUID," "PortNumber,"
            << "GUID," << "Sm_Key," << "ActCount,"
            << "SmState," << "Priority"
            << endl;
    csv_out.WriteLine(sstream.str());

    char curr_counters_line[1024];
    for (list_p_sm_info_obj::iterator it = this->fabric_extended_info.getSMPSMInfoListRef().begin();
            it != this->fabric_extended_info.getSMPSMInfoListRef().end(); ++it) {
        memset(curr_counters_line, 0, sizeof(curr_counters_line));
        sstream.str("");
        sprintf(curr_counters_line, U64H_FMT "," U64H_FMT "," U32D_FMT ","   \
                U64H_FMT "," U64H_FMT "," U32D_FMT ","   \
                U32D_FMT "," U32D_FMT,
                (*it)->p_port->p_node->guid_get(),
                (*it)->p_port->guid_get(),
                (*it)->p_port->num,
                (*it)->smp_sm_info.GUID,
                (*it)->smp_sm_info.Sm_Key,
                (*it)->smp_sm_info.ActCount,
                (*it)->smp_sm_info.SmState,
                (*it)->smp_sm_info.Priority);
        sstream << curr_counters_line << endl;
        csv_out.WriteLine(sstream.str());
    }

    csv_out.DumpEnd(SECTION_SM_INFO);
    IBDIAG_RETURN(IBDIAG_SUCCESS_CODE);
}


//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS / ERR_CODE_IO_ERR
int IBDiag::WriteSMFile(const char *file_name)
{
    IBDIAG_ENTER;
    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    ofstream sout;
    int rc = this->OpenFile(file_name, sout,
                            false, //no append
                            true); //add_header

    if (rc)
        IBDIAG_RETURN(rc);
    this->DumpSMInfo(sout);
    sout.close();
    IBDIAG_RETURN(IBDIAG_SUCCESS_CODE);
}
