/*
 * © Copyright (c) 2004-2018 Mellanox Technologies LTD. All rights reserved
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */


#include "infiniband/ibdiag/ibdiag.h"
#include "infiniband/ibdiag/ibdiag_clbck.h"
#include "infiniband/ibdiag/ibdiagnet/ibdiagnet_plugins_ifc.h"

extern IBDiagClbck ibDiagClbck;

//Router related definitions
#define IB_ROUTER_ELEMENTS_IN_ADJACENT_BLOCK 8
#define IB_ROUTER_ELEMENTS_IN_NEXT_HOP_BLOCK 4
#define NUM_OF_ROUTER_TABLE_ENTRIES(tbl_top, num_elements_in_block) \
    ((tbl_top + num_elements_in_block - 1) / num_elements_in_block)


/****************************************************/
/******************private methods*******************/
/****************************************************/
void IBDiag::DumpNodesInfo(ofstream &sout)
{
    IBDIAG_ENTER;
    struct VendorSpec_GeneralInfo *p_curr_general_info = NULL;
    char curr_general_info_line[2096];
    for (u_int32_t i = 0; i < this->fabric_extended_info.getNodesVectorSize(); ++i) {
        IBNode *p_curr_node = this->fabric_extended_info.getNodePtr(i);
        if (!p_curr_node)               //port not exist
            continue;
        p_curr_general_info = this->fabric_extended_info.getVSGeneralInfo(i);
        if (!p_curr_general_info)      //info not exist
            continue;

        sout << "-------------------------------------------------------" << endl;
        sout << "Node Name=" << p_curr_node->getName() << endl;
        sout << "-------------------------------------------------------" << endl;
        memset(curr_general_info_line, 0, sizeof(curr_general_info_line));
        string psid = (char *)p_curr_general_info->FWInfo.PSID.PSID;
        sprintf(curr_general_info_line,
                "GUID=" U64H_FMT "\n"   \
                "HWInfo_DeviceID=" U16H_FMT "\n"    \
                "HWInfo_DeviceHWRevision=" U16H_FMT "\n"    \
                "HWInfo_UpTime=" U32H_FMT "\n"  \
                "FWInfo_SubMinor=" U8H_FMT "\n" \
                "FWInfo_Minor=" U8H_FMT "\n"    \
                "FWInfo_Major=" U8H_FMT "\n"    \
                "FWInfo_BuildID=" U32H_FMT "\n" \
                "FWInfo_Year=" U16H_FMT "\n"    \
                "FWInfo_Day=" U8H_FMT "\n"  \
                "FWInfo_Month=" U8H_FMT "\n"    \
                "FWInfo_Hour=" U16H_FMT "\n"    \
                "FWInfo_PSID=" STR_FMT  "\n"    \
                "FWInfo_INI_File_Version=" U32H_FMT "\n"    \
                "FWInfo_Extended_Major=" U32H_FMT "\n"  \
                "FWInfo_Extended_Minor=" U32H_FMT "\n"  \
                "FWInfo_Extended_SubMinor=" U32H_FMT "\n"   \
                "SWInfo_SubMinor=" U8H_FMT "\n" \
                "SWInfo_Minor=" U8H_FMT "\n"    \
                "SWInfo_Major=" U8H_FMT "\n",
                p_curr_node->guid_get(),
                p_curr_general_info->HWInfo.DeviceID,
                p_curr_general_info->HWInfo.DeviceHWRevision,
                p_curr_general_info->HWInfo.UpTime,
                p_curr_general_info->FWInfo.SubMinor,
                p_curr_general_info->FWInfo.Minor,
                p_curr_general_info->FWInfo.Major,
                p_curr_general_info->FWInfo.BuildID,
                p_curr_general_info->FWInfo.Year,
                p_curr_general_info->FWInfo.Day,
                p_curr_general_info->FWInfo.Month,
                p_curr_general_info->FWInfo.Hour,
                (psid == "") ? "UNKNOWN" : psid.c_str(),
                p_curr_general_info->FWInfo.INI_File_Version,
                p_curr_general_info->FWInfo.Extended_Major,
                p_curr_general_info->FWInfo.Extended_Minor,
                p_curr_general_info->FWInfo.Extended_SubMinor,
                p_curr_general_info->SWInfo.SubMinor,
                p_curr_general_info->SWInfo.Minor,
                p_curr_general_info->SWInfo.Major);
        sout << curr_general_info_line << endl;
    }
    IBDIAG_RETURN_VOID;
}


//*****************************************************************************
//   Functions for printing to mlnx_cntrs file
//*****************************************************************************
void IBDiag::DumpDiagnosticCountersDescriptionP0(ofstream &sout)
{
    IBDIAG_ENTER;
    sout << "#---------- Transport errors and flows (Page0 Ver" << PAGE0_LATEST_VER <<
            ")----------------------------" << endl;
    sout << "#rq_num_lle: Responder - number of local length errors" << endl;
    sout << "#sq_num_lle: Requester - number of local length errors" << endl;
    sout << "#rq_num_lqpoe: Responder - number local QP operation error" <<
        endl;
    sout << "#sq_num_lqpoe: Requester - number local QP operation error" <<
        endl;
    sout << "#rq_num_leeoe: Responder - number local EE operation error" <<
        endl;
    sout << "#sq_num_leeoe: Requester - number local EE operation error" <<
        endl;
    sout << "#rq_num_lpe: Responder - number of local protection errors" <<
        endl;
    sout << "#sq_num_lpe: Requester - number of local protection errors" <<
        endl;
    sout << "#rq_num_wrfe: Responder - number of WR flushed errors" << endl;
    sout << "#sq_num_wrfe: Requester - number of WR flushed errors" << endl;
    sout << "#sq_num_mwbe: Requester - number of memory window bind errors" <<
        endl;
    sout << "#sq_num_bre: Requester - number of bad response errors" << endl;
    sout << "#rq_num_lae: Responder - number of local access errors" << endl;
    sout << "#rq_num_rire: Responder - number of remote invalid request"  \
            " errors" << endl;
    sout << "#sq_num_rire: Requester - number of remote invalid request"  \
            " errors" << endl;
    sout << "#rq_num_rae: Responder - number of remote access errors" << endl;
    sout << "#sq_num_rae: Requester - number of remote access errors" << endl;
    sout << "#rq_num_roe: Responder - number of remote operation errors" <<
        endl;
    sout << "#sq_num_roe: Requester - number of remote operation errors" <<
        endl;
    sout << "#sq_num_rnr: Responder - number of RNR Naks recieved" << endl;
    sout << "#rq_num_oos: Responder - number of out of sequence requests"  \
            " received" << endl;
    sout << "#sq_num_oos: Requester - number of out of sequence Naks "  \
            "received" << endl;
    sout << "#rq_num_dup: Responder - number of duplicate requests recieved" <<
        endl;
    sout << "#sq_num_to: Requester - number of time out recieved" << endl;
    sout << "#sq_num_tree: Requester - number of transport retries exceeded"  \
            " errors" << endl;
    sout << "#sq_num_rree: Requester - number of RNR nak retries exceeded"  \
            " errors" << endl;
    sout << "#sq_num_rabrte: Requester - number of remote aborted errors" <<
        endl;
    sout << "#rq_num_mce: Responder - number of bad multicast packets"  \
            " received"  << endl;
    sout << "#rq_num_retrans_rsync: Responder - number of retransmission"  \
            " RESYNC operations" << endl;
    sout << "#sq_num_retrans_rsync: Requester - number of retransmission"  \
            "  RESYNC operations" << endl;
    sout << "#sq_num_ldb_drops: Requester - number of non-fatal ack/response"  \
            " mismatches" << endl;

}


void IBDiag::DumpDiagnosticCountersDescriptionP1(ofstream &sout)
{
    IBDIAG_ENTER;
    sout << "#---------- HCA Extended Flows (Page1 Ver" << PAGE1_LATEST_VER <<  \
            ")----------------------------" << endl;
    sout << "#rq_num_sig_err: Responder - number of wire signature errors" <<
        endl;
    sout << "#sq_num_sig_err: Requester - number of memory signature errors" <<
        endl;
    sout << "#sq_num_cnak: Requester - number of DC Connect Nacks received" <<
        endl;
    sout << "#sq_reconnect: Requester - number of DC reconnect requrests"  \
            " sent" << endl;
    sout << "#sq_reconnect_ack: Requester - number of DC reconnect acks "  \
            "received" << endl;
    sout << "#rq_open_gb: Responder - number of DC ghost busters opened" <<
        endl;
     sout << "#rq_num_no_dcrs: Responder-number of loopback Connects which"  \
              " got no DCRs" << endl;
     sout << "#rq_num_cnak_sent: Responder-number of loopback Connects"  \
              " which got no DCRs and sent a CNAK" << endl;
     sout << "#sq_reconnect_ack_bad: Requester-number of DC reconnect acks"  \
              " dropped" << endl;
     sout << "#rq_open_gb_cnak: Responder-number of DC ghost busters open"  \
              " required/connect while connect with ghost buster - but no"  \
              " DCRs available" << endl;
     sout << "#rq_gb_trap_cnak: Responder-number of cnaks sent on connect"  \
              " packet trapped at ghost buster" << endl;
     sout << "#rq_not_gb_connect: Responder-connect while connected, no"  \
              " ghost buster involved (dropped disconnect)" << endl;
     sout << "#rq_not_gb_reconnect: Responder-reconnect without ghost buster"  \
              " received = ghost buster installer " << endl;
     sout << "#rq_curr_gb_connect: Responder-connect received trapped by"  \
              " ghost buster" << endl;
     sout << "#rq_curr_gb_reconnect: Responder-ghost buster reactivated"  \
              " (ghost buster installer while ghost buster exists)" << endl;
     sout << "#rq_close_non_gb_gc: Responder-number of regular DCRs closed"  \
              " due to garbage collection" << endl;
     sout << "#rq_dcr_inhale_events: Responder-number of DCR inhale"  \
              " events triggered" << endl;
     sout << "#rq_state_active_gb: Responder-number of currently open"  \
              " ghost busters" << endl;
     sout << "#rq_state_avail_dcrs: Responder-number of currently"  \
              " available DCRs" << endl;
     sout << "#rq_state_dcr_lifo_size: Responder-current DCR"  \
              " stack size" << endl;
     sout << "#sq_cnak_drop: Requester - number of CNAKs dropped" << endl;
     sout << "#minimum_dcrs: Responder - lowest level of DCRs observed" << endl;
     sout << "#maximum_dcrs: Responder - maximum number of used DCRs observed" << endl;
     sout << "#max_cnak_fifo_size: Responder - highest number of pending CNAKs" << endl;
    sout << "#rq_num_dc_cacks: Responder - number of connect-ack packets sent" << endl;
    sout << "#sq_num_dc_cacks: Requestor - number of connect-ack packets received" << endl;
    sout << endl;
}


void IBDiag::DumpDiagnosticCountersP0(
        ofstream &sout,
        struct VS_DiagnosticData *p_p0)
{
    IBDIAG_ENTER;

    struct VS_DC_TransportErrorsAndFlowsV2 * p_p0_v2 =
        &(p_p0->data_set.TransportErrorsAndFlowsV2);

    sout << "#---------- Transport errors and flows (Page0 Rev2)------------"  \
            "----------------" << endl;
    char curr_counters_line[2096] = {'\0'};
    sprintf(curr_counters_line,
            "rq_num_lle=" U32H_FMT "\n"  \
            "sq_num_lle=" U32H_FMT "\n"  \
            "rq_num_lqpoe=" U32H_FMT "\n"  \
            "sq_num_lqpoe=" U32H_FMT "\n"  \
            "rq_num_leeoe=" U32H_FMT "\n"  \
            "sq_num_leeoe=" U32H_FMT "\n"  \
            "rq_num_lpe=" U32H_FMT "\n"  \
            "sq_num_lpe=" U32H_FMT "\n"  \
            "rq_num_wrfe=" U32H_FMT "\n"  \
            "sq_num_wrfe=" U32H_FMT "\n"  \
            "sq_num_mwbe=" U32H_FMT "\n"  \
            "sq_num_bre=" U32H_FMT "\n"  \
            "rq_num_lae=" U32H_FMT "\n"  \
            "rq_num_rire=" U32H_FMT "\n"  \
            "sq_num_rire=" U32H_FMT "\n"  \
            "rq_num_rae=" U32H_FMT "\n"  \
            "sq_num_rae=" U32H_FMT "\n"  \
            "rq_num_roe=" U32H_FMT "\n"  \
            "sq_num_roe=" U32H_FMT "\n"  \
            "sq_num_rnr=" U32H_FMT "\n"  \
            "rq_num_oos=" U32H_FMT "\n"  \
            "sq_num_oos=" U32H_FMT "\n"  \
            "rq_num_dup=" U32H_FMT "\n"  \
            "sq_num_to=" U32H_FMT "\n"  \
            "sq_num_tree=" U32H_FMT "\n"  \
            "sq_num_rree=" U32H_FMT "\n"  \
            "sq_num_rabrte=" U32H_FMT "\n"  \
            "rq_num_mce=" U32H_FMT "\n"  \
            "rq_num_retrans_rsync=" U32H_FMT "\n"  \
            "sq_num_retrans_rsync=" U32H_FMT "\n"  \
            "sq_num_ldb_drops=" U32H_FMT "\n",
            p_p0_v2->rq_num_lle,
            p_p0_v2->sq_num_lle,
            p_p0_v2->rq_num_lqpoe,
            p_p0_v2->sq_num_lqpoe,
            p_p0_v2->rq_num_leeoe,
            p_p0_v2->sq_num_leeoe,
            p_p0_v2->rq_num_lpe,
            p_p0_v2->sq_num_lpe,
            p_p0_v2->rq_num_wrfe,
            p_p0_v2->sq_num_wrfe,
            p_p0_v2->sq_num_mwbe,
            p_p0_v2->sq_num_bre,
            p_p0_v2->rq_num_lae,
            p_p0_v2->rq_num_rire,
            p_p0_v2->sq_num_rire,
            p_p0_v2->rq_num_rae,
            p_p0_v2->sq_num_rae,
            p_p0_v2->rq_num_roe,
            p_p0_v2->sq_num_roe,
            p_p0_v2->sq_num_rnr,
            p_p0_v2->rq_num_oos,
            p_p0_v2->sq_num_oos,
            p_p0_v2->rq_num_dup,
            p_p0_v2->sq_num_to,
            p_p0_v2->sq_num_tree,
            p_p0_v2->sq_num_rree,
            p_p0_v2->sq_num_rabrte,
            p_p0_v2->rq_num_mce,
            p_p0_v2->rq_num_retrans_rsync,
            p_p0_v2->sq_num_retrans_rsync,
            p_p0_v2->sq_num_ldb_drops);
        sout << curr_counters_line;
}


void IBDiag::DumpDiagnosticCountersP1(
        ofstream &sout,
        struct VS_DiagnosticData *p_p1)
{
    IBDIAG_ENTER;

    struct VS_DC_Page1LatestVersion * p_p1_last_ver =
        &(p_p1->data_set.Page1LatestVersion);

    char curr_counters_line[2096] = {'\0'};
    sout << "#---------- HCA Extended Flows (Page1  Rev4)------------"  \
            "----------------" << endl;
    sprintf(curr_counters_line,
            "rq_num_sig_err=" U32H_FMT "\n"  \
            "sq_num_sig_err=" U32H_FMT "\n"  \
            "sq_num_cnak=" U32H_FMT "\n"  \
            "sq_reconnect=" U32H_FMT "\n"  \
            "sq_reconnect_ack=" U32H_FMT "\n"  \
            "rq_open_gb=" U32H_FMT "\n"  \
            "rq_num_no_dcrs=" U32H_FMT "\n"  \
            "rq_num_cnak_sent=" U32H_FMT "\n"  \
            "sq_reconnect_ack_bad=" U32H_FMT "\n"  \
            "rq_open_gb_cnak=" U32H_FMT "\n"  \
            "rq_gb_trap_cnak=" U32H_FMT "\n"  \
            "rq_not_gb_connect=" U32H_FMT "\n"  \
            "rq_not_gb_reconnect=" U32H_FMT "\n"  \
            "rq_curr_gb_connect=" U32H_FMT "\n"  \
            "rq_curr_gb_reconnect=" U32H_FMT "\n"  \
            "rq_close_non_gb_gc=" U32H_FMT "\n"  \
            "rq_dcr_inhale_events=" U32H_FMT "\n"  \
            "rq_state_active_gb=" U32H_FMT "\n"  \
            "rq_state_avail_dcrs=" U32H_FMT "\n"  \
            "rq_state_dcr_lifo_size=" U32H_FMT "\n"  \
            "sq_cnak_drop=" U32H_FMT "\n"  \
            "minimum_dcrs=" U32H_FMT "\n"  \
            "maximum_dcrs=" U32H_FMT "\n"  \
            "max_cnak_fifo_size=" U32H_FMT "\n"  \
            "rq_num_dc_cacks=" U32H_FMT "\n"  \
            "sq_num_dc_cacks=" U32H_FMT "\n",
            p_p1_last_ver->rq_num_sig_err,
            p_p1_last_ver->sq_num_sig_err,
            p_p1_last_ver->sq_num_cnak,
            p_p1_last_ver->sq_reconnect,
            p_p1_last_ver->sq_reconnect_ack,
            p_p1_last_ver->rq_open_gb,
            p_p1_last_ver->rq_num_no_dcrs,
            p_p1_last_ver->rq_num_cnak_sent,
            p_p1_last_ver->sq_reconnect_ack_bad,
            p_p1_last_ver->rq_open_gb_cnak,
            p_p1_last_ver->rq_gb_trap_cnak,
            p_p1_last_ver->rq_not_gb_connect,
            p_p1_last_ver->rq_not_gb_reconnect,
            p_p1_last_ver->rq_curr_gb_connect,
            p_p1_last_ver->rq_curr_gb_reconnect,
            p_p1_last_ver->rq_close_non_gb_gc,
            p_p1_last_ver->rq_dcr_inhale_events,
            p_p1_last_ver->rq_state_active_gb,
            p_p1_last_ver->rq_state_avail_dcrs,
            p_p1_last_ver->rq_state_dcr_lifo_size,
            p_p1_last_ver->sq_cnak_drop,
            p_p1_last_ver->minimum_dcrs,
            p_p1_last_ver->maximum_dcrs,
            p_p1_last_ver->max_cnak_fifo_size,
            p_p1_last_ver->rq_num_dc_cacks,
            p_p1_last_ver->sq_num_dc_cacks);
    sout << curr_counters_line;
}

void IBDiag::DumpDiagnosticCountersP255(
        ofstream &sout,
        struct VS_DiagnosticData *p_p255)
{
    IBDIAG_ENTER;

    struct VS_DC_Page255LatestVersion * p_p255_data =
        &(p_p255->data_set.Page255LatestVersion);

    char curr_counters_line[2096] = {'\0'};
    sout << "#---------- Internal HCA Debug (Page 255  Rev3) ------------"  \
            "----------------" << endl;

    sprintf(curr_counters_line,
            "field0=" U32H_FMT "\n"  \
            "field1=" U32H_FMT "\n"  \
            "field2=" U32H_FMT "\n"  \
            "field3=" U32H_FMT "\n"  \
            "field4=" U32H_FMT "\n"  \
            "field5=" U32H_FMT "\n"  \
            "field6=" U32H_FMT "\n"  \
            "field7=" U32H_FMT "\n"  \
            "field8=" U32H_FMT "\n"  \
            "field9=" U32H_FMT "\n"  \
            "field10=" U32H_FMT "\n"  \
            "field11=" U32H_FMT "\n"  \
            "field12=" U32H_FMT "\n"  \
            "field13=" U32H_FMT "\n"  \
            "field14=" U32H_FMT "\n"  \
            "field15=" U32H_FMT "\n"  \
            "field16=" U32H_FMT "\n",
            p_p255_data->num_cqovf,
            p_p255_data->num_eqovf,
            p_p255_data->sq_num_ds_ex_pi_retry,
            p_p255_data->rq_next_gb_connect,
            p_p255_data->rq_next_gb_reconnect,
            p_p255_data->rq_psn_close_gb,
            p_p255_data->rq_psn_close_gb_cwc,
            p_p255_data->rq_close_gb_gc,
            p_p255_data->rq_cwc_ghost_discard_connect,
            p_p255_data->rq_cwc_ghost_discard_reconnect,
            p_p255_data->rq_cwc_reconnect_stale_connection,
            p_p255_data->rq_cwc_reconnect_probable_current_connection,
            p_p255_data->rq_cwc_uncertainty_discard,
            p_p255_data->rq_num_dc_cacks_full_hash,
            p_p255_data->rq_num_dc_cacks_collision,
            p_p255_data->dc_hash_curr_used_lines,
            p_p255_data->dc_has_max_used_entries_in_line);
    sout << curr_counters_line << endl;
}

void IBDiag::DumpDiagnosticCounters(ofstream &sout)
{
    IBDIAG_ENTER;
    struct VS_DiagnosticData *p_curr_mlnx_cntrs_p0 = NULL;
    struct VS_DiagnosticData *p_curr_mlnx_cntrs_p1 = NULL;
    struct VS_DiagnosticData *p_curr_mlnx_cntrs_p255 = NULL;

    char curr_counters_line[2096];

    //add header with counters description
    DumpDiagnosticCountersDescriptionP0(sout);
    DumpDiagnosticCountersDescriptionP1(sout);

    for (u_int32_t i = 0; i < this->fabric_extended_info.getPortsVectorSize();
         ++i) {
        IBPort *p_curr_port = this->fabric_extended_info.getPortPtr(i);
        if (!p_curr_port)               //port not exist
            continue;

        p_curr_mlnx_cntrs_p0 =
            this->fabric_extended_info.getVSDiagnosticCountersPage0(i);
        p_curr_mlnx_cntrs_p1 =
            this->fabric_extended_info.getVSDiagnosticCountersPage1(i);
        p_curr_mlnx_cntrs_p255 =
            this->fabric_extended_info.getVSDiagnosticCountersPage255(i);

        if (!p_curr_mlnx_cntrs_p0 && !p_curr_mlnx_cntrs_p1)    //info not exist
            continue;

        memset(curr_counters_line, 0, sizeof(curr_counters_line));
        sprintf(curr_counters_line, "Port=" U32D_FMT    \
                " Lid=" U16H_FMT    \
                " Node GUID=" U64H_FMT   \
                " Port GUID=" U64H_FMT   \
                " Device=" U32D_FMT \
                " Port Name=" STR_FMT,
                p_curr_port->num,
                p_curr_port->base_lid,
                p_curr_port->p_node->guid_get(),
                p_curr_port->guid_get(),
                p_curr_port->p_node->devId,
                p_curr_port->getName().c_str());
        sout << "-------------------------------------------------------" <<
            endl;
        sout << curr_counters_line << endl;
        sout << "-------------------------------------------------------" <<
            endl;


        if (p_curr_mlnx_cntrs_p0)
            DumpDiagnosticCountersP0(sout, p_curr_mlnx_cntrs_p0);

        if (p_curr_mlnx_cntrs_p1)
            DumpDiagnosticCountersP1(sout, p_curr_mlnx_cntrs_p1);

        if (p_curr_mlnx_cntrs_p255)
            DumpDiagnosticCountersP255(sout, p_curr_mlnx_cntrs_p255);
    }
}


/****************************************************/
/*******************public methods*******************/
/****************************************************/
//Returns: IBDIAG_SUCCESS_CODE / IBDIAG_ERR_CODE_FABRIC_ERROR
int IBDiag::CheckVSGeneralInfo(IBNode *p_curr_node, struct VendorSpec_GeneralInfo *p_curr_general_info)
{
    IBDIAG_ENTER;
    int rc = IBDIAG_SUCCESS_CODE;

    if (p_curr_general_info->FWInfo.Year < 0x2000 ||
        p_curr_general_info->FWInfo.Year > 0x2050 ||
        p_curr_general_info->FWInfo.Day == 0    ||
        p_curr_general_info->FWInfo.Day  > 0x31   ||
        p_curr_general_info->FWInfo.Month == 0  ||
        p_curr_general_info->FWInfo.Month > 0x12) {

        IBDIAG_LOG(TT_LOG_LEVEL_DEBUG, "Node %s ,device=%u has invalid FW date %x/%x/%x \n",
                p_curr_node->getName().c_str(),
                p_curr_node->devId,
                p_curr_general_info->FWInfo.Month,
                p_curr_general_info->FWInfo.Day,
                p_curr_general_info->FWInfo.Year);
        rc = IBDIAG_ERR_CODE_FABRIC_ERROR;

    }

    IBDIAG_RETURN(rc);
}


//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS
int IBDiag::DumpNodesInfoCSVTable(CSVOut &csv_out)
{
    IBDIAG_ENTER;
    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    csv_out.DumpStart(SECTION_NODES_INFO);

    stringstream sstream;
    sstream << "NodeGUID,"
            << "HWInfo_DeviceID," << "HWInfo_DeviceHWRevision," << "HWInfo_UpTime,"
            << "FWInfo_SubMinor," << "FWInfo_Minor," << "FWInfo_Major,"
            << "FWInfo_BuildID,"
            << "FWInfo_Year," << "FWInfo_Day," << "FWInfo_Month," << "FWInfo_Hour,"
            << "FWInfo_PSID,"
            << "FWInfo_INI_File_Version,"
            << "FWInfo_Extended_Major," << "FWInfo_Extended_Minor," << "FWInfo_Extended_SubMinor,"
            << "SWInfo_SubMinor," << "SWInfo_Minor," << "SWInfo_Major"
            << endl;
    csv_out.WriteLine(sstream.str());

    struct VendorSpec_GeneralInfo *p_curr_general_info = NULL;
    char curr_general_info_line[2096];
    for (u_int32_t i = 0; i < this->fabric_extended_info.getNodesVectorSize(); ++i) {
        IBNode *p_curr_node = this->fabric_extended_info.getNodePtr(i);
        if (!p_curr_node)                   //node not exist
            continue;
        p_curr_general_info = this->fabric_extended_info.getVSGeneralInfo(i);
        if (!p_curr_general_info)           //info not exist
            continue;

        memset(curr_general_info_line, 0, sizeof(curr_general_info_line));
        sstream.str("");
        string psid = (char *)p_curr_general_info->FWInfo.PSID.PSID;
        sprintf(curr_general_info_line, U64H_FMT ","    \
                U16H_FMT "," U16H_FMT "," U32H_FMT ","  \
                U8H_FMT "," U8H_FMT "," U8H_FMT ","  \
                U32H_FMT ","  \
                U16H_FMT "," U8H_FMT "," U8H_FMT "," U16H_FMT ","   \
                STR_FMT "," \
                U32H_FMT ","  \
                U32H_FMT "," U32H_FMT "," U32H_FMT ","  \
                U8H_FMT "," U8H_FMT "," U8H_FMT,
                p_curr_node->guid_get(),
                p_curr_general_info->HWInfo.DeviceID,
                p_curr_general_info->HWInfo.DeviceHWRevision,
                p_curr_general_info->HWInfo.UpTime,
                p_curr_general_info->FWInfo.SubMinor,
                p_curr_general_info->FWInfo.Minor,
                p_curr_general_info->FWInfo.Major,
                p_curr_general_info->FWInfo.BuildID,
                p_curr_general_info->FWInfo.Year,
                p_curr_general_info->FWInfo.Day,
                p_curr_general_info->FWInfo.Month,
                p_curr_general_info->FWInfo.Hour,
                (psid == "") ? "UNKNOWN" : psid.c_str(),
                p_curr_general_info->FWInfo.INI_File_Version,
                p_curr_general_info->FWInfo.Extended_Major,
                p_curr_general_info->FWInfo.Extended_Minor,
                p_curr_general_info->FWInfo.Extended_SubMinor,
                p_curr_general_info->SWInfo.SubMinor,
                p_curr_general_info->SWInfo.Minor,
                p_curr_general_info->SWInfo.Major);
        sstream << curr_general_info_line << endl;
        csv_out.WriteLine(sstream.str());
    }

    csv_out.DumpEnd(SECTION_NODES_INFO);
    IBDIAG_RETURN(IBDIAG_SUCCESS_CODE);
}


//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS / ERR_CODE_IO_ERR
int IBDiag::WriteNodesInfoFile(const char *file_name)
{
    IBDIAG_ENTER;
    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    ofstream sout;
    int rc = this->OpenFile(file_name, sout,
                            false, //no append
                            true); //add_header

    if (rc)
        IBDIAG_RETURN(rc);
    this->DumpNodesInfo(sout);
    sout.close();
    IBDIAG_RETURN(IBDIAG_SUCCESS_CODE);
}


//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS / ERR_CODE_DB_ERR / ERR_CODE_CHECK_FAILED
int IBDiag::CheckFWVersion(list_p_fabric_general_err& fw_errors)
{
    IBDIAG_ENTER;
    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    int rc = IBDIAG_SUCCESS_CODE;
    map_devid_psid_to_fw_version_obj_t max_fw_version_per_devid_psid;

    fw_version_obj_t curr_devid_fw_version;
    fw_version_obj_t max_devid_fw_version;

    //find maximum fw version for each device id
    for (u_int32_t i = 0; i < this->fabric_extended_info.getNodesVectorSize(); ++i) {
        IBNode *p_curr_node = this->fabric_extended_info.getNodePtr(i);
        if (!p_curr_node)                   //node not exists
            continue;
        VendorSpec_GeneralInfo *p_curr_general_info = this->fabric_extended_info.getVSGeneralInfo(i);
        if (!p_curr_general_info)           //info not exists
            continue;

        int rc2 = this->CheckVSGeneralInfo(p_curr_node, p_curr_general_info);
        if (rc2) {
            //mark this as invalid data
            p_curr_general_info->FWInfo.Day=0;
            continue;
        }

        curr_devid_fw_version = *p_curr_general_info;
        IBDIAG_LOG(TT_LOG_LEVEL_DEBUG, "Node %s has FW Version "    \
                U32D_FMT "." U32D_FMT "." U32D_FMT    \
                " ,device=%u\n",
                p_curr_node->getName().c_str(),
                curr_devid_fw_version.major, curr_devid_fw_version.minor, curr_devid_fw_version.sub_minor,
                p_curr_node->devId);

        string psid_string((char *)p_curr_general_info->FWInfo.PSID.PSID, 16);

        map_devid_psid_to_fw_version_obj_t::iterator it2 =
           max_fw_version_per_devid_psid.find(pair_device_id_t_string(
                       p_curr_node->devId,
                       psid_string));

        if (it2 == max_fw_version_per_devid_psid.end()) {      //first time reached this devid
            max_fw_version_per_devid_psid[pair_device_id_t_string(
                    p_curr_node->devId, psid_string)] =
                p_curr_general_info;
            continue;
        }
        max_devid_fw_version = *(it2->second);
        if (curr_devid_fw_version > max_devid_fw_version) {
            max_fw_version_per_devid_psid[pair_device_id_t_string(
                    p_curr_node->devId,
                    psid_string)] =
                p_curr_general_info;
        }
    }
    for (map_devid_psid_to_fw_version_obj_t::iterator it =
            max_fw_version_per_devid_psid.begin();
         it != max_fw_version_per_devid_psid.end();
         it++) {
        pair_device_id_t_string pair_devid_psid = (it->first);
        curr_devid_fw_version = *(it->second);
        INFO_PRINT("Devid: " U32D_FMT "(" U16H_FMT "), PSID: " STR_FMT ", Latest FW Version:"    \
                   U32D_FMT "." U32D_FMT "." U32D_FMT "\n",
                   pair_devid_psid.first,
                   pair_devid_psid.first,
                   pair_devid_psid.second.c_str(),
                   curr_devid_fw_version.major,
                   curr_devid_fw_version.minor,
                   curr_devid_fw_version.sub_minor);
    }

    //find nodes with not maximum fw version
    for (u_int32_t i = 0; i < this->fabric_extended_info.getNodesVectorSize(); ++i) {
        IBNode *p_curr_node = this->fabric_extended_info.getNodePtr(i);
        if (!p_curr_node)                   //node not exists
            continue;
        VendorSpec_GeneralInfo *p_curr_general_info = this->fabric_extended_info.getVSGeneralInfo(i);
        if (!p_curr_general_info)           //info not exists
            continue;
        if (p_curr_general_info->FWInfo.Day==0) {
            FabricErrNodeNotSupportCap *p_curr_fabric_node_err =
                    new FabricErrNodeNotSupportCap(p_curr_node,
                            "The firmware of this device returned invalid general info data");
            if (!p_curr_fabric_node_err) {
                this->SetLastError("Failed to allocate FabricErrNodeNotSupportCap");
                IBDIAG_RETURN(IBDIAG_ERR_CODE_NO_MEM);
            }
            fw_errors.push_back(p_curr_fabric_node_err);
            rc = IBDIAG_ERR_CODE_CHECK_FAILED;
            continue;
        }
        curr_devid_fw_version = *p_curr_general_info;
        string psid_string((char *)p_curr_general_info->FWInfo.PSID.PSID, 16);

        map_devid_psid_to_fw_version_obj_t::iterator it =
            max_fw_version_per_devid_psid.find(pair_device_id_t_string(
                        p_curr_node->devId,
                        psid_string));
        if (it == max_fw_version_per_devid_psid.end()) {      //first time reached this devid
            this->SetLastError("DB Error - failed to find maximum FW version for node=%s, device=%u",
                    p_curr_node->getName().c_str(), p_curr_node->devId);
            rc = IBDIAG_ERR_CODE_DB_ERR;
            goto check_fw_version_exit;
        }
        max_devid_fw_version = *(it->second);
        if (curr_devid_fw_version != max_devid_fw_version) {
            char buff[1024];
            sprintf(buff,
                    "Node with Devid:" U32D_FMT "(" U16H_FMT ")"    \
                    ",PSID:" STR_FMT " has FW version " \
                    U32D_FMT "." U32D_FMT "." U32D_FMT  \
                    " while the latest FW version for the same Devid/PSID on this fabric is "   \
                    U32D_FMT "." U32D_FMT "." U32D_FMT,
                    p_curr_node->devId, p_curr_node->devId,
                    psid_string.c_str(),
                    curr_devid_fw_version.major, curr_devid_fw_version.minor, curr_devid_fw_version.sub_minor,
                    max_devid_fw_version.major, max_devid_fw_version.minor, max_devid_fw_version.sub_minor);
            FabricErrNodeWrongFWVer *p_curr_fabric_node_err =
                    new FabricErrNodeWrongFWVer(p_curr_node, buff);
            if (!p_curr_fabric_node_err) {
                this->SetLastError("Failed to allocate FabricErrNodeWrongFWVer");
                IBDIAG_RETURN(IBDIAG_ERR_CODE_NO_MEM);
            }
            p_curr_fabric_node_err->SetLevel(EN_FABRIC_ERR_WARNING);
            fw_errors.push_back(p_curr_fabric_node_err);
            rc = IBDIAG_ERR_CODE_CHECK_FAILED;
        }
    }

check_fw_version_exit:
    max_fw_version_per_devid_psid.clear();
    IBDIAG_RETURN(rc);
}


/*****************************************************************************/
/*       Vendor Specific Capability SMP                                      */
/*****************************************************************************/

//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS / ERR_CODE_DB_ERR / ERR_CODE_NO_MEM / ERR_CODE_FABRIC_ERROR
int IBDiag::BuildVsCapSmpFwInfo(list_p_fabric_general_err &vs_cap_smp_errors,
                                progress_func_nodes_t progress_func)
{
    IBDIAG_ENTER;
    int rc = IBDIAG_SUCCESS_CODE;
    progress_bar_nodes_t progress_bar_nodes;
    CLEAR_STRUCT(progress_bar_nodes);

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &ibDiagClbck;
    clbck_data.m_handle_data_func = IBDiagSMPVSGeneralInfoFwInfoGetClbck;

    // go over all nodes and collect FW
    struct FWInfo_Block_Element curr_general_info;
    for (map_str_pnode::iterator nI = this->discovered_fabric.NodeByName.begin();
            nI != this->discovered_fabric.NodeByName.end();
            ++nI) {
        IBNode *p_curr_node = (*nI).second;
        if (!p_curr_node) {
            this->SetLastError("DB error - found null node in NodeByName map for key = %s", (*nI).first.c_str());
            IBDIAG_RETURN(IBDIAG_ERR_CODE_DB_ERR);
        }
        ++progress_bar_nodes.nodes_found;
        if (p_curr_node->type == IB_SW_NODE)
            ++progress_bar_nodes.sw_found;
        else
            ++progress_bar_nodes.ca_found;
        if (progress_func)
            progress_func(&progress_bar_nodes, &this->discover_progress_bar_nodes);

        //currently we will send SMP FW MAD only for acquiring
        //SMP capability mask purposes
        //change it once, we need SMP FW for other purposes
        if (capability_module.IsSMPMaskKnown(p_curr_node->guid_get()))
            continue;

        //send general info mad get for this node, if it allows it (i.e supports such mad)
        capability_mask_t mask;
        if (capability_module.IsSMPUnsupportedMadDevice(p_curr_node->vendId,
                                                        p_curr_node->devId,
                                                        mask))
            continue;
        clbck_data.m_data1 = p_curr_node;
        direct_route_t *p_direct_route = this->GetDirectRouteByNodeGuid(p_curr_node->guid_get());
        this->ibis_obj.SMPVSGeneralInfoFwInfoMadGetByDirect(p_direct_route,
                                                            &curr_general_info,
                                                            &clbck_data);

        if(ibDiagClbck.GetState())
            goto exit;
    }
exit:
    this->ibis_obj.MadRecAll();

    if (rc == IBDIAG_SUCCESS_CODE) {
        rc = ibDiagClbck.GetState();
        if (rc)
            this->SetLastError(ibDiagClbck.GetLastError());
        else if (!vs_cap_smp_errors.empty())
            rc = IBDIAG_ERR_CODE_FABRIC_ERROR;
    }

    IBDIAG_RETURN(rc);
}

int IBDiag::BuildVsCapSmpCapabilityMask(list_p_fabric_general_err &vs_cap_smp_errors,
                                        progress_func_nodes_t progress_func)
{
    IBDIAG_ENTER;
    int rc = IBDIAG_SUCCESS_CODE;
    progress_bar_nodes_t progress_bar_nodes;
    CLEAR_STRUCT(progress_bar_nodes);

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &ibDiagClbck;
    clbck_data.m_handle_data_func = IBDiagSMPVSGeneralInfoCapabilityMaskGetClbck;
    bool query_per_matched_guid;

    // go over all nodes and collect Capability Mask
    struct GeneralInfoCapabilityMask curr_general_info;
    for (map_str_pnode::iterator nI = this->discovered_fabric.NodeByName.begin();
            nI != this->discovered_fabric.NodeByName.end();
            ++nI) {
        IBNode *p_curr_node = (*nI).second;
        if (!p_curr_node) {
            this->SetLastError("DB error - found null node in NodeByName"
                               " map for key = %s", (*nI).first.c_str());
            IBDIAG_RETURN(IBDIAG_ERR_CODE_DB_ERR);
        }
        ++progress_bar_nodes.nodes_found;
        if (p_curr_node->type == IB_SW_NODE)
            ++progress_bar_nodes.sw_found;
        else
            ++progress_bar_nodes.ca_found;
        if (progress_func)
            progress_func(&progress_bar_nodes, &this->discover_progress_bar_nodes);

        //check if this node has already known mask, i.e in the cache
        //we already took care of fw configured masks in BuildVsCapSmpFwInfo,
        //so the mask should be known in that case
        if (capability_module.IsSMPMaskKnown(p_curr_node->guid_get()))
            continue;

        //check if there is configuration per GUID that says query
        query_per_matched_guid = false;
        u_int8_t prefix_len = 0;
        u_int64_t matched_guid = 0;
        query_or_mask_t qmask;

        //these checks are to prevent programming errors
        //as we already took care of 'unsupported devices', 'guid to mask' and
        //'ven id, dev id, fw, mask' configurations
        //so now only 'guid query', 'ven id dev id fw query' are left
        if (capability_module.IsLongestSMPPrefixMatch(p_curr_node->guid_get(),
                                                      prefix_len,
                                                      matched_guid,
                                                      qmask) &&
            qmask.to_query == true)
            query_per_matched_guid = true;

        capability_mask_t mask;

        mask.clear();

        //no match by guid with 'query'
        if (!query_per_matched_guid) {
            //check if it matches unsupported device then
            if (capability_module.IsSMPUnsupportedMadDevice(p_curr_node->vendId,
                                                            p_curr_node->devId,
                                                            mask))
                continue;
        }

        clbck_data.m_data1 = p_curr_node;
        direct_route_t *p_direct_route =
            this->GetDirectRouteByNodeGuid(p_curr_node->guid_get());
        this->ibis_obj.SMPVSGeneralInfoCapabilityMaskMadGetByDirect(
            p_direct_route, &curr_general_info, &clbck_data);

        if(ibDiagClbck.GetState())
            goto exit;
    }
exit:
    this->ibis_obj.MadRecAll();

    if (rc == IBDIAG_SUCCESS_CODE) {
        rc = ibDiagClbck.GetState();
        if (rc)
            this->SetLastError(ibDiagClbck.GetLastError());
        else if (!vs_cap_smp_errors.empty())
            rc = IBDIAG_ERR_CODE_FABRIC_ERROR;
    }

    IBDIAG_RETURN(rc);
}

//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS / ERR_CODE_DB_ERR / ERR_CODE_NO_MEM / ERR_CODE_FABRIC_ERROR
int IBDiag::BuildExtendedPortInfo(list_p_fabric_general_err &vs_ext_pi_errors,
                                  progress_func_nodes_t progress_func)
{
    IBDIAG_ENTER;

    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    int rc = IBDIAG_SUCCESS_CODE;
    ibDiagClbck.Set(this, &fabric_extended_info, &vs_ext_pi_errors,
                    NULL, &capability_module);

    progress_bar_nodes_t progress_bar_nodes;
    CLEAR_STRUCT(progress_bar_nodes);

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &ibDiagClbck;
    clbck_data.m_handle_data_func = IBDiagSMPVSExtendedPortInfoGetClbck;

    // go over all nodes and collect extended port info
    for (map_str_pnode::iterator nI = this->discovered_fabric.NodeByName.begin();
            nI != this->discovered_fabric.NodeByName.end();
            ++nI) {
        IBNode *p_curr_node = (*nI).second;
        if (!p_curr_node) {
            this->SetLastError("DB error - found null node in NodeByName map for key = %s", (*nI).first.c_str());
            IBDIAG_RETURN(IBDIAG_ERR_CODE_DB_ERR);
        }
        ++progress_bar_nodes.nodes_found;
        if (p_curr_node->type == IB_SW_NODE)
            ++progress_bar_nodes.sw_found;
        else
            ++progress_bar_nodes.ca_found;
        if (progress_func)
            progress_func(&progress_bar_nodes, &this->discover_progress_bar_nodes);
        // go over all ports of this node:
        for (u_int32_t i = 1; i <= p_curr_node->numPorts; ++i) { //start from port 1, no need to reset port 0
            IBPort *p_curr_port = p_curr_node->getPort((phys_port_t)i);
            if (!p_curr_port)                       //port not exist
                continue;
            //Skip ports that are down or have undefined state
            //or are not in the scope of the fabric.
            if (!p_curr_port->is_data_worthy())
                continue;
            SMP_PortInfo *curr_port_info = this->fabric_extended_info.getSMPPortInfo(p_curr_port->createIndex);
            if (!curr_port_info) {
                this->SetLastError("DB error - found connected port=%s"
                                   " without SMPPortInfo",
                                   p_curr_port->getName().c_str());
                IBDIAG_RETURN(IBDIAG_ERR_CODE_DB_ERR);
            }
            /* check for MLNX speed if needed and LLR active cell size
               if we had extended speed than we sure not QDR and
               we do not need to check for MLNX speed */
            /*
             * Check if need to send MAD
             * to query for mlnx extended speeds and LLR Active Cell size
             */
            if (this->no_mepi)
                continue;
            if (curr_port_info->PortPhyState != IB_PORT_PHYS_STATE_LINK_UP)
                continue;
            //is this device supports Mlnx Extended Port Info mad
            bool dev_support_mepi = this->capability_module.IsSupportedSMPCapability(
                p_curr_port->p_node, EnSMPCapIsExtendedPortInfoSupported);

            //Don't send the mad to device that don't support it
            if (!dev_support_mepi) {
                IBDIAG_LOG(TT_LOG_LEVEL_DEBUG,
                           "The port %s doesn't support VS"
                           " SMPExtendedPortInfo MAD\n",
                           p_curr_port->getName().c_str());
                continue;
            }


            SMP_MlnxExtPortInfo curr_mlnx_ext_port_info;

            /*
             * Send the MAD
             */
            clbck_data.m_data1 = p_curr_port;

            this->ibis_obj.SMPMlnxExtPortInfoMadGetByDirect(
                this->GetDirectRouteByNodeGuid(p_curr_port->p_node->guid_get()),
                p_curr_port->num,
                &curr_mlnx_ext_port_info,
                &clbck_data);

            if(ibDiagClbck.GetState())
                goto exit;
        }
    }
exit:
    this->ibis_obj.MadRecAll();

    if (rc) {
        if (last_error.empty())
            this->SetLastError("Retrieve of VS ExtendedPortInfo Failed.");
        else
            IBDIAG_LOG(TT_LOG_LEVEL_ERROR, "Retrieve of VS ExtendedPortInfo Failed. \n");
    } else {
        rc = ibDiagClbck.GetState();
        if (rc)
            this->SetLastError(ibDiagClbck.GetLastError());
        else if (!vs_ext_pi_errors.empty())
            rc = IBDIAG_ERR_CODE_FABRIC_ERROR;
    }

    IBDIAG_RETURN(rc);
}

/*****************************************************************************/
/*       Vendor Specific Capability GMP                                      */
/*****************************************************************************/
//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS / ERR_CODE_DB_ERR / ERR_CODE_NO_MEM / ERR_CODE_FABRIC_ERROR
int IBDiag::BuildVsCapGmpInfo(list_p_fabric_general_err &vs_cap_gmp_errors,
                                progress_func_nodes_t progress_func)
{
    IBDIAG_ENTER;
    int rc = IBDIAG_SUCCESS_CODE;
    progress_bar_nodes_t progress_bar_nodes;
    CLEAR_STRUCT(progress_bar_nodes);

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &ibDiagClbck;
    clbck_data.m_handle_data_func = IBDiagVSGeneralInfoGetClbck;

    bool query_per_matched_guid;

    // go over all nodes and collect FW
    struct VendorSpec_GeneralInfo curr_general_info;
    for (map_str_pnode::iterator nI = this->discovered_fabric.NodeByName.begin();
            nI != this->discovered_fabric.NodeByName.end();
            ++nI) {
        IBNode *p_curr_node = (*nI).second;
        if (!p_curr_node) {
            this->SetLastError("DB error - found null node in NodeByName map for key = %s", (*nI).first.c_str());
            IBDIAG_RETURN(IBDIAG_ERR_CODE_DB_ERR);
        }
        ++progress_bar_nodes.nodes_found;
        if (p_curr_node->type == IB_SW_NODE)
            ++progress_bar_nodes.sw_found;
        else
            ++progress_bar_nodes.ca_found;
        if (progress_func)
            progress_func(&progress_bar_nodes, &this->discover_progress_bar_nodes);

        IBPort *p_curr_port = NULL;
        if(p_curr_node->type == IB_SW_NODE) {
            p_curr_port = p_curr_node->getPort(0);  //getting port 0
            if (!p_curr_port || p_curr_port->base_lid == 0 || p_curr_port->base_lid >= 0xc000)   {   //port not exist
                continue;
            }
        } else {
            for (unsigned int i = 1 ; i <=  p_curr_node->numPorts ; i++) {
                p_curr_port = p_curr_node->getPort((phys_port_t)i);
                if (!p_curr_port || p_curr_port->base_lid == 0 || p_curr_port->base_lid >= 0xc000)      //port not exist
                    continue;
                else
                    break;
            }
            if (!p_curr_port || p_curr_port->base_lid == 0){      //all ports not exist
                this->SetLastError("DB Error - failed to find HCA port for node=%s", p_curr_node->getName().c_str());
                rc = IBDIAG_ERR_CODE_DB_ERR;
                goto exit;
            }
        }

        //send general info mad get for this node, if it supports the mad
        capability_mask_t mask;
        mask.clear();

        //check if there is configuration per GUID that says query
        query_per_matched_guid = false;
        u_int8_t prefix_len = 0;
        u_int64_t matched_guid = 0;
        query_or_mask_t qmask;

        //don't send the mad for devices that don't support it
        //however the capability configuration 'guid -> query', precedes
        //this rule
        if (capability_module.IsLongestGMPPrefixMatch(p_curr_node->guid_get(),
                                                      prefix_len,
                                                      matched_guid,
                                                      qmask) &&
            qmask.to_query == true)
            query_per_matched_guid = true;

        if (!query_per_matched_guid &&
            capability_module.IsGMPUnsupportedMadDevice(p_curr_node->vendId,
                                                        p_curr_node->devId,
                                                        mask))
                continue;

        //send mad anyway for Nodes Info file
        clbck_data.m_data1 = p_curr_node;
        this->ibis_obj.VSGeneralInfoGet(p_curr_port->base_lid,
                                        &curr_general_info, &clbck_data);

        if(ibDiagClbck.GetState())
            goto exit;
    }
exit:
    this->ibis_obj.MadRecAll();

    if (rc == IBDIAG_SUCCESS_CODE) {
        rc = ibDiagClbck.GetState();
        if (rc)
            this->SetLastError(ibDiagClbck.GetLastError());
        else if (!vs_cap_gmp_errors.empty())
            rc = IBDIAG_ERR_CODE_FABRIC_ERROR;
    }

    IBDIAG_RETURN(rc);
}

/****************************************************/
/*******************public methods*******************/
/****************************************************/
//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS / ERR_CODE_DB_ERR / ERR_CODE_NO_MEM / ERR_CODE_FABRIC_ERROR
int IBDiag::BuildVsCapSmpDB(list_p_fabric_general_err &vs_cap_smp_errors,
                            progress_func_nodes_t progress_func)
{
    IBDIAG_ENTER;

    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    int rc1, rc2;

    ibDiagClbck.Set(this, &fabric_extended_info, &vs_cap_smp_errors, NULL, &capability_module);
    //retrieve fw info
    rc1 = this->BuildVsCapSmpFwInfo(vs_cap_smp_errors, progress_func);

    //retrieve capability mask
    rc2 = this->BuildVsCapSmpCapabilityMask(vs_cap_smp_errors, progress_func);

    IBDIAG_RETURN((rc1 || rc2) ? IBDIAG_ERR_CODE_FABRIC_ERROR : IBDIAG_SUCCESS_CODE);
}


//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS / ERR_CODE_DB_ERR / ERR_CODE_NO_MEM / ERR_CODE_FABRIC_ERROR
int IBDiag::BuildVsCapGmpDB(list_p_fabric_general_err &vs_cap_gmp_errors,
                            progress_func_nodes_t progress_func)
{
    IBDIAG_ENTER;

    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    int rc = IBDIAG_SUCCESS_CODE;
    ibDiagClbck.Set(this, &fabric_extended_info, &vs_cap_gmp_errors,
                    NULL, &capability_module);
    //retrieve fw info
    rc = this->BuildVsCapGmpInfo(vs_cap_gmp_errors, progress_func);

    IBDIAG_RETURN(rc);
}

/*****************************************************************************/
/*       Vendor Specific Diagnostic Counters                                 */
/*****************************************************************************/
//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS / ERR_CODE_DB_ERR /
//ERR_CODE_NO_MEM / ERR_CODE_FABRIC_ERROR
int IBDiag::BuildVsDiagnosticCounters(
        list_p_fabric_general_err& mlnx_cntrs_errors,
        progress_func_nodes_t progress_func)
{
    IBDIAG_ENTER;
    int rc = IBDIAG_SUCCESS_CODE;

    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    ResetAppData();
    ibDiagClbck.Set(this, &fabric_extended_info, &mlnx_cntrs_errors);

    progress_bar_nodes_t progress_bar_nodes;
    CLEAR_STRUCT(progress_bar_nodes);

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &ibDiagClbck;

    // go over all nodes and collect counters
    struct VS_DiagnosticData p_mlnx_cntrs;
    for (map_str_pnode::iterator nI =
            this->discovered_fabric.NodeByName.begin();
         nI != this->discovered_fabric.NodeByName.end();
         ++nI) {
        IBNode *p_curr_node = (*nI).second;
        if (!p_curr_node) {
            this->SetLastError("DB error - found null node in NodeByName map"  \
                               " for key = %s", (*nI).first.c_str());
            IBDIAG_RETURN(IBDIAG_ERR_CODE_DB_ERR);
        }
        ++progress_bar_nodes.nodes_found;
        if (p_curr_node->type == IB_SW_NODE)
            ++progress_bar_nodes.sw_found;
        else
            ++progress_bar_nodes.ca_found;
        if (progress_func)
            progress_func(&progress_bar_nodes,
                          &this->discover_progress_bar_nodes);

        IBPort *p_curr_port = NULL;
        if(p_curr_node->type == IB_SW_NODE)
                continue;
        else {
             struct SMP_NodeInfo * p_curr_node_info =
                    this->fabric_extended_info.getSMPNodeInfo(
                            p_curr_node->createIndex);
             if (!p_curr_node_info) {
                 this->SetLastError(
                         "DB error - failed to get SMPNodeInfo for node: %s",
                         p_curr_node->name.c_str());
                 rc = IBDIAG_ERR_CODE_DB_ERR;
                 goto exit;
             }

            if (!this->capability_module.IsSupportedGMPCapability(
                        p_curr_node, EnGMPCapIsDiagnosticDataSupported)) {
                IBDIAG_LOG(TT_LOG_LEVEL_DEBUG,
                           "node %s: DeviceID %u (0x%x) Does not support"  \
                           " Diagnostic Counters, skipping \n",
                        p_curr_node->name.c_str(),
                        p_curr_node_info->DeviceID,
                        p_curr_node_info->DeviceID);
                continue;
            }

            for (unsigned int i = 1 ; i <=  p_curr_node->numPorts ; i++) {
                p_curr_port = p_curr_node->getPort((phys_port_t)i);
                //Skip ports that don't exist or are down or have undefined
                //state or are not in the scope of the fabric.
                if (!p_curr_port || !p_curr_port->is_data_worthy())
                    continue;

                clbck_data.m_data1 = p_curr_port;
                //get counters per node, so the port number is irrelevant, i.e 0
                //page 0
                clbck_data.m_handle_data_func =
                    IBDiagVSDiagnosticCountersPage0GetClbck;
                this->ibis_obj.VSDiagnosticDataGet(
                        p_curr_port->base_lid,
                        0,
                        VS_MLNX_CNTRS_PAGE0,
                        &p_mlnx_cntrs,
                        &clbck_data);
                //page 1
                clbck_data.m_handle_data_func =
                    IBDiagVSDiagnosticCountersPage1GetClbck;
                this->ibis_obj.VSDiagnosticDataGet(
                        p_curr_port->base_lid,
                        0,
                        VS_MLNX_CNTRS_PAGE1,
                        &p_mlnx_cntrs,
                        &clbck_data);

                //page 255
                clbck_data.m_handle_data_func = IBDiagVSDiagnosticCountersPage255GetClbck;
                this->ibis_obj.VSDiagnosticDataGet(
                        p_curr_port->base_lid,
                        0,
                        VS_MLNX_CNTRS_PAGE255,
                        &p_mlnx_cntrs,
                        &clbck_data);

                //the information is node related, so one port is enough
                //when will add info per port, add NOT_SUPPORT_MLNX_COUNTERS
               break;
            }
        }
    }

exit:
    this->ibis_obj.MadRecAll();

    if (rc == IBDIAG_SUCCESS_CODE) {
        rc = ibDiagClbck.GetState();
        if (rc)
            this->SetLastError(ibDiagClbck.GetLastError());
        else if (!mlnx_cntrs_errors.empty())
            rc = IBDIAG_ERR_CODE_FABRIC_ERROR;
    }

    IBDIAG_RETURN(rc);
}

//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS / ERR_CODE_DB_ERR /
//ERR_CODE_NO_MEM / ERR_CODE_FABRIC_ERROR
int IBDiag::ResetDiagnosticCounters(
        list_p_fabric_general_err& mlnx_cntrs_errors,
        progress_func_nodes_t progress_func)
{
    IBDIAG_ENTER;
    int rc = IBDIAG_SUCCESS_CODE;
    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    ResetAppData();
    ibDiagClbck.Set(this, &fabric_extended_info, &mlnx_cntrs_errors);

    progress_bar_nodes_t progress_bar_nodes;
    CLEAR_STRUCT(progress_bar_nodes);

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &ibDiagClbck;

    // go over all nodes and collect FW
    struct VS_DiagnosticData p_mlnx_cntrs;
    for (map_str_pnode::iterator nI =
            this->discovered_fabric.NodeByName.begin();
         nI != this->discovered_fabric.NodeByName.end();
         ++nI) {
        IBNode *p_curr_node = (*nI).second;
        if (!p_curr_node) {
            this->SetLastError("DB error - found null node in NodeByName map"  \
                               " for key = %s", (*nI).first.c_str());
            IBDIAG_RETURN(IBDIAG_ERR_CODE_DB_ERR);
        }
        ++progress_bar_nodes.nodes_found;
        if (p_curr_node->type == IB_SW_NODE)
            continue;

        IBPort *p_curr_port = NULL;
        struct SMP_NodeInfo * p_curr_node_info =
            this->fabric_extended_info.getSMPNodeInfo(p_curr_node->createIndex);
        if (!p_curr_node_info) {
            this->SetLastError(
                    "DB error - failed to get SMPNodeInfo for node: %s",
                    p_curr_node->name.c_str());
            rc = IBDIAG_ERR_CODE_DB_ERR;
            goto exit;
        }

        if (!this->capability_module.IsSupportedGMPCapability(
                    p_curr_node, EnGMPCapIsDiagnosticDataSupported)) {
            IBDIAG_LOG(TT_LOG_LEVEL_DEBUG, "node %s: DeviceID %u (0x%x)"   \
                                           " Does not support Mellanox"  \
                                           " Counters, skipping \n",
                    p_curr_node->name.c_str(),
                    p_curr_node_info->DeviceID,
                    p_curr_node_info->DeviceID);
            continue;
        }

        if (p_curr_node->type == IB_CA_NODE)
            ++progress_bar_nodes.ca_found;
        if (progress_func)
            progress_func(&progress_bar_nodes,
                          &this->discover_progress_bar_nodes);

        for (unsigned int i = 1 ; i <=  p_curr_node->numPorts ; i++) {
            p_curr_port = p_curr_node->getPort((phys_port_t)i);
            //Skip ports that don't exist or are down or have undefined state
            //or are not in the scope of the fabric.
            if (!p_curr_port || !p_curr_port->is_data_worthy())
                continue;

            clbck_data.m_data1 = p_curr_port;
            // Clear all the diagnostic pages: pages 0, 1, 255
            // page 0
            clbck_data.m_handle_data_func =
                IBDiagVSDiagnosticCountersClearClbck;
            this->ibis_obj.VSDiagnosticDataPageClear(
                    p_curr_port->base_lid,
                    0, // clear per node, so port number is irrelevant
                    VS_MLNX_CNTRS_PAGE0, &p_mlnx_cntrs,
                    &clbck_data);
            //page 1
            this->ibis_obj.VSDiagnosticDataPageClear(
                    p_curr_port->base_lid,
                    0, // clear per node, so port number is irrelevant
                    VS_MLNX_CNTRS_PAGE1, &p_mlnx_cntrs,
                    &clbck_data);
            //page 255
            this->ibis_obj.VSDiagnosticDataPageClear(
                    p_curr_port->base_lid,
                    0, // clear per node, so port number is irrelevant
                    VS_MLNX_CNTRS_PAGE255, &p_mlnx_cntrs,
                    &clbck_data);
            //the information is node related, so one port is enough
            break;
        }
    }

exit:
    this->ibis_obj.MadRecAll();

    if (rc == IBDIAG_SUCCESS_CODE) {
        rc = ibDiagClbck.GetState();
        if (rc)
            this->SetLastError(ibDiagClbck.GetLastError());
        else if (!mlnx_cntrs_errors.empty())
            rc = IBDIAG_ERR_CODE_FABRIC_ERROR;
    }

    IBDIAG_RETURN(rc);
}


//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS
int IBDiag::DumpDiagnosticCountersCSVTable(CSVOut &csv_out)
{
    IBDIAG_ENTER;
    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    struct VS_DiagnosticData *p_curr_mlnx_cntrs_p0 = NULL;
    struct VS_DiagnosticData *p_curr_mlnx_cntrs_p1 = NULL;
    struct VS_DiagnosticData *p_curr_mlnx_cntrs_p255 = NULL;
    struct VS_DC_TransportErrorsAndFlowsV2 *p_p0_v2 = NULL;
    struct VS_DC_Page1LatestVersion *p_p1_last_ver = NULL;
    struct VS_DC_Page255LatestVersion *p_p255_last_ver = NULL;

    char curr_counters_line[1024] = {'\0'};

    csv_out.DumpStart(SECTION_MLNX_CNTRS_INFO);

    stringstream sstream;
    sstream << "NodeGUID," << "PortGUID," "PortNumber," "LID,";
    // Page0
    sstream << "rq_num_lle," << "sq_num_lle," << "rq_num_lqpoe,"
            << "sq_num_lqpoe," << "rq_num_leeoe," << "sq_num_leeoe,"
            << "rq_num_lpe," << "sq_num_lpe," << "rq_num_wrfe,"
            << "sq_num_wrfe," << "sq_num_mwbe," << "sq_num_bre,"
            << "rq_num_lae," << "rq_num_rire,"
            << "sq_num_rire," << "rq_num_rae,"
            << "sq_num_rae,"
            << "rq_num_roe," << "sq_num_roe,"
            << "sq_num_rnr," << "rq_num_oos,"
            << "sq_num_oos," << "rq_num_dup,"
            << "sq_num_to," << "sq_num_tree," << "sq_num_rree,"
            << "sq_num_rabrte," << "rq_num_mce," << "rq_num_retrans_rsync,"
            << "sq_num_retrans_rsync," << "sq_num_ldb_drops,";
    // Page1
    sstream << "rq_num_sig_err," << "sq_num_sig_err," << "sq_num_cnak,"
            << "sq_reconnect," << "sq_reconnect_ack," << "rq_open_gb,"
            << "rq_num_no_dcrs," << "rq_num_cnak_sent,"
            << "sq_reconnect_ack_bad," << "rq_open_gb_cnak,"
            << "rq_gb_trap_cnak," << "rq_not_gb_connect,"
            << "rq_not_gb_reconnect," << "rq_curr_gb_connect,"
            << "rq_curr_gb_reconnect," << "rq_close_non_gb_gc,"
            << "rq_dcr_inhale_events," << "rq_state_active_gb,"
            << "rq_state_avail_dcrs," << "rq_state_dcr_lifo_size,"
            << "sq_cnak_drop," << "minimum_dcrs," << "maximum_dcrs,"
            << "max_cnak_fifo_size," << "rq_num_dc_cacks," << "sq_num_dc_cacks";

    // Page 255
    for (int i = 0; i < PAGE255_NUM_FIELDS; ++i) {
        sstream << ",field" << i;
    }

    sstream << endl;
    csv_out.WriteLine(sstream.str());

    for (u_int32_t i = 0; i < this->fabric_extended_info.getPortsVectorSize(); ++i) {

        IBPort *p_curr_port = this->fabric_extended_info.getPortPtr(i);
        if (!p_curr_port)               //port not exist
            continue;

        p_curr_mlnx_cntrs_p0 =
            this->fabric_extended_info.getVSDiagnosticCountersPage0(i);
        p_curr_mlnx_cntrs_p1 =
            this->fabric_extended_info.getVSDiagnosticCountersPage1(i);
        p_curr_mlnx_cntrs_p255 =
            this->fabric_extended_info.getVSDiagnosticCountersPage255(i);

        if (!p_curr_mlnx_cntrs_p0 && !p_curr_mlnx_cntrs_p1 && !p_curr_mlnx_cntrs_p255)  //info not exist
            continue;

        sstream.str("");
        sprintf(curr_counters_line,
                U64H_FMT "," U64H_FMT "," U32D_FMT "," U16H_FMT ",",
                p_curr_port->p_node->guid_get(),
                p_curr_port->guid_get(),
                p_curr_port->num,
                p_curr_port->base_lid);

        sstream << curr_counters_line;

        memset(curr_counters_line, 0, sizeof(curr_counters_line));
        if (p_curr_mlnx_cntrs_p0) {
            p_p0_v2 =
                &(p_curr_mlnx_cntrs_p0->data_set.TransportErrorsAndFlowsV2);
            sprintf(curr_counters_line,
                    U32D_FMT "," U32D_FMT "," U32D_FMT ","  \
                    U32D_FMT "," U32D_FMT "," U32D_FMT ","  \
                    U32D_FMT "," U32D_FMT "," U32D_FMT ","  \
                    U32D_FMT "," U32D_FMT "," U32D_FMT ","  \
                    U32D_FMT "," U32D_FMT "," U32D_FMT ","  \
                    U32D_FMT "," U32D_FMT "," U32D_FMT ","  \
                    U32D_FMT "," U32D_FMT "," U32D_FMT ","  \
                    U32D_FMT "," U32D_FMT "," U32D_FMT ","  \
                    U32D_FMT "," U32D_FMT "," U32D_FMT ","  \
                    U32D_FMT "," U32D_FMT "," U32D_FMT ","  \
                    U32D_FMT ",",
                    p_p0_v2->rq_num_lle,
                    p_p0_v2->sq_num_lle,
                    p_p0_v2->rq_num_lqpoe,
                    p_p0_v2->sq_num_lqpoe,
                    p_p0_v2->rq_num_leeoe,
                    p_p0_v2->sq_num_leeoe,
                    p_p0_v2->rq_num_lpe,
                    p_p0_v2->sq_num_lpe,
                    p_p0_v2->rq_num_wrfe,
                    p_p0_v2->sq_num_wrfe,
                    p_p0_v2->sq_num_mwbe,
                    p_p0_v2->sq_num_bre,
                    p_p0_v2->rq_num_lae,
                    p_p0_v2->rq_num_rire,
                    p_p0_v2->sq_num_rire,
                    p_p0_v2->rq_num_rae,
                    p_p0_v2->sq_num_rae,
                    p_p0_v2->rq_num_roe,
                    p_p0_v2->sq_num_roe,
                    p_p0_v2->sq_num_rnr,
                    p_p0_v2->rq_num_oos,
                    p_p0_v2->sq_num_oos,
                    p_p0_v2->rq_num_dup,
                    p_p0_v2->sq_num_to,
                    p_p0_v2->sq_num_tree,
                    p_p0_v2->sq_num_rree,
                    p_p0_v2->sq_num_rabrte,
                    p_p0_v2->rq_num_mce,
                    p_p0_v2->rq_num_retrans_rsync,
                    p_p0_v2->sq_num_retrans_rsync,
                    p_p0_v2->sq_num_ldb_drops);

        } else {
            sprintf(curr_counters_line, "-1"                      \
                    ",-1,-1,-1,-1,-1,-1,-1,-1,-1,-1" \
                    ",-1,-1,-1,-1,-1,-1,-1,-1,-1,-1" \
                    ",-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,");
        }
        sstream << curr_counters_line;

        // Page 1
        if (p_curr_mlnx_cntrs_p1) {
            p_p1_last_ver = &(p_curr_mlnx_cntrs_p1->data_set.Page1LatestVersion);

            sstream << p_p1_last_ver->rq_num_sig_err << ","
                    << p_p1_last_ver->sq_num_sig_err << ","
                    << p_p1_last_ver->sq_num_cnak << ","
                    << p_p1_last_ver->sq_reconnect << ","
                    << p_p1_last_ver->sq_reconnect_ack << ","
                    << p_p1_last_ver->rq_open_gb << ","
                    << p_p1_last_ver->rq_num_no_dcrs << ","
                    << p_p1_last_ver->rq_num_cnak_sent << ","
                    << p_p1_last_ver->sq_reconnect_ack_bad << ","
                    << p_p1_last_ver->rq_open_gb_cnak << ","
                    << p_p1_last_ver->rq_gb_trap_cnak << ","
                    << p_p1_last_ver->rq_not_gb_connect << ","
                    << p_p1_last_ver->rq_not_gb_reconnect << ","
                    << p_p1_last_ver->rq_curr_gb_connect << ","
                    << p_p1_last_ver->rq_curr_gb_reconnect << ","
                    << p_p1_last_ver->rq_close_non_gb_gc << ","
                    << p_p1_last_ver->rq_dcr_inhale_events << ","
                    << p_p1_last_ver->rq_state_active_gb << ","
                    << p_p1_last_ver->rq_state_avail_dcrs << ","
                    << p_p1_last_ver->rq_state_dcr_lifo_size << ","
                    << p_p1_last_ver->sq_cnak_drop << ","
                    << p_p1_last_ver->minimum_dcrs << ","
                    << p_p1_last_ver->maximum_dcrs << ","
                    << p_p1_last_ver->max_cnak_fifo_size << ","
                    << p_p1_last_ver->rq_num_dc_cacks << ","
                    << p_p1_last_ver->sq_num_dc_cacks << ",";
        } else {
            sstream << "-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1" \
                    ",-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,";
        }

        // Page 255
        if (p_curr_mlnx_cntrs_p255) {
            p_p255_last_ver = &(p_curr_mlnx_cntrs_p255->data_set.Page255LatestVersion);

            sstream << p_p255_last_ver->num_cqovf << ","
                    << p_p255_last_ver->num_eqovf << ","
                    << p_p255_last_ver->sq_num_ds_ex_pi_retry << ","
                    << p_p255_last_ver->rq_next_gb_connect << ","
                    << p_p255_last_ver->rq_next_gb_reconnect << ","
                    << p_p255_last_ver->rq_psn_close_gb << ","
                    << p_p255_last_ver->rq_psn_close_gb_cwc << ","
                    << p_p255_last_ver->rq_close_gb_gc << ","
                    << p_p255_last_ver->rq_cwc_ghost_discard_connect << ","
                    << p_p255_last_ver->rq_cwc_ghost_discard_reconnect << ","
                    << p_p255_last_ver->rq_cwc_reconnect_stale_connection << ","
                    << p_p255_last_ver->rq_cwc_reconnect_probable_current_connection << ","
                    << p_p255_last_ver->rq_cwc_uncertainty_discard << ","
                    << p_p255_last_ver->rq_num_dc_cacks_full_hash << ","
                    << p_p255_last_ver->rq_num_dc_cacks_collision << ","
                    << p_p255_last_ver->dc_hash_curr_used_lines << ","
                    << p_p255_last_ver->dc_has_max_used_entries_in_line;
        } else {
            sstream << "-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1";
        }

        sstream << endl;
        csv_out.WriteLine(sstream.str());
    }

    csv_out.DumpEnd(SECTION_MLNX_CNTRS_INFO);
    IBDIAG_RETURN(IBDIAG_SUCCESS_CODE);
}


//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS / ERR_CODE_IO_ERR
int IBDiag::WriteMlnxCntrsFile(const char *file_name)
{
    IBDIAG_ENTER;
    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    ofstream sout;
    int rc = this->OpenFile(file_name, sout,
                            false, //no append
                            true); //add_header
    if (rc)
        IBDIAG_RETURN(rc);

    this->DumpDiagnosticCounters(sout);
    sout.close();
    IBDIAG_RETURN(IBDIAG_SUCCESS_CODE);
}

int IBDiag::CheckCapabilityForQoSConfigSL(IBNode *p_curr_node, bool is_vports,
                                          list_p_fabric_general_err& qos_config_sl_errors,
                                          bool& has_capability)
{
    IBDIAG_ENTER;

    EnSMPCapabilityMaskBit_t capability_mask_bit_qos_config_sl_rate_limit;
    EnSMPCapabilityMaskBit_t capability_mask_bit_qos_config_sl_bw;
    string buff_port_type;
    if (is_vports) {
        capability_mask_bit_qos_config_sl_rate_limit = EnSMPCapIsQoSConfigSLVPortRateLimitSupported;
        capability_mask_bit_qos_config_sl_bw = EnSMPCapIsQoSConfigSLVPortAllocBWSupported;
        buff_port_type = " for vports";
    } else {
        capability_mask_bit_qos_config_sl_rate_limit = EnSMPCapIsQoSConfigSLRateLimitSupported;
        capability_mask_bit_qos_config_sl_bw = EnSMPCapIsQoSConfigSLAllocBWSupported;
        buff_port_type = "";
    }

    bool rate_limit_flag = this->capability_module.IsSupportedSMPCapability(
            p_curr_node, capability_mask_bit_qos_config_sl_rate_limit);

    bool bw_alloc_flag = this->capability_module.IsSupportedSMPCapability(
            p_curr_node, capability_mask_bit_qos_config_sl_bw);

    has_capability = true;
    if (!rate_limit_flag && !bw_alloc_flag) {
        char buff_msg_error[256] = {0};
        sprintf(buff_msg_error, "This device doesn't support SMPQoSConfigSL MAD%s",
                buff_port_type.c_str());
        FabricErrNodeNotSupportCap *p_fabric_info_err =
                    new FabricErrNodeNotSupportCap(p_curr_node, buff_msg_error);
        if (!p_fabric_info_err) {
            SetLastError("Failed to allocate FabricErrNodeNotSupportCap");
            IBDIAG_RETURN(IBDIAG_ERR_CODE_NO_MEM);
        } else
            qos_config_sl_errors.push_back(p_fabric_info_err);

        struct SMP_NodeInfo * p_curr_node_info =
        this->fabric_extended_info.getSMPNodeInfo(p_curr_node->createIndex);
        if (!p_curr_node_info) {
            this->SetLastError("DB error - failed to get SMPNodeInfo for node: %s",
                    p_curr_node->name.c_str());
            IBDIAG_RETURN(IBDIAG_ERR_CODE_DB_ERR);
        }

        IBDIAG_LOG(TT_LOG_LEVEL_DEBUG,
                "node %s: DeviceID %u (0x%x) Does not support"  \
                " QoS Config SL MAD %s, skipping \n",
                p_curr_node->name.c_str(),
                p_curr_node_info->DeviceID,
                p_curr_node_info->DeviceID,
                buff_port_type.c_str());

        has_capability = false;
    }

    IBDIAG_RETURN(IBDIAG_SUCCESS_CODE);
}

int IBDiag::BuildSMPQoSConfigSL(list_p_fabric_general_err& qos_config_sl_errors,
                                bool is_vports,
                                progress_func_nodes_t progress_func)
{
    IBDIAG_ENTER;
    int rc = IBDIAG_SUCCESS_CODE;

    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    ibDiagClbck.Set(this, &fabric_extended_info, &qos_config_sl_errors);

    progress_bar_nodes_t progress_bar_nodes;
    CLEAR_STRUCT(progress_bar_nodes);

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &ibDiagClbck;

    if (is_vports)
        clbck_data.m_handle_data_func = IBDiagSMPVPortQoSConfigSLGetClbck;
    else
        clbck_data.m_handle_data_func = IBDiagSMPQoSConfigSLGetClbck;

    struct SMP_QosConfigSL qos_config_sl;

    // go over all nodes and collect qos_config_sl
    for (map_str_pnode::iterator nI = this->discovered_fabric.NodeByName.begin();
            nI != this->discovered_fabric.NodeByName.end(); ++nI) {

        IBNode *p_curr_node = (*nI).second;
        if (!p_curr_node) {
            this->SetLastError("DB error - found null node in NodeByName map"  \
                               " for key = %s", (*nI).first.c_str());
            IBDIAG_RETURN(IBDIAG_ERR_CODE_DB_ERR);
        }

        ++progress_bar_nodes.nodes_found;
        if (p_curr_node->type == IB_SW_NODE)
            ++progress_bar_nodes.sw_found;
        else
            ++progress_bar_nodes.ca_found;
        if (progress_func)
            progress_func(&progress_bar_nodes, &this->discover_progress_bar_nodes);

        // send this MAD only for CAs.
        if (p_curr_node->type != IB_CA_NODE)
            continue;

        bool has_capability;
        rc = CheckCapabilityForQoSConfigSL(p_curr_node, is_vports, qos_config_sl_errors,
                                           has_capability);
        if (rc)
            goto exit;

        // go over all ports of this node:
        for (u_int32_t i = 0; i <= p_curr_node->numPorts; ++i) {
            IBPort *p_curr_port = p_curr_node->getPort((phys_port_t)i);

            if (!p_curr_port)
                continue;

            //Skip ports that are down or have undefined state
            //or are not in the scope of the fabric.
            if (!p_curr_port->is_data_worthy())
                continue;

            clbck_data.m_data1 = p_curr_port;

            //get direct route for this port
            direct_route_t *p_curr_direct_route =
                    this->GetDirectRouteByPortGuid(p_curr_port->guid_get());
            if (!p_curr_direct_route) {
                this->SetLastError("DB error - can't find direct route to node=%s, port=%u",
                                   p_curr_node->getName().c_str(), (u_int8_t)p_curr_port->num);
                rc = IBDIAG_ERR_CODE_DB_ERR;
                goto exit;
            }

            if (has_capability && !is_vports) {
                this->ibis_obj.SMPQosConfigSLGetByDirect(p_curr_direct_route,
                                                         &qos_config_sl,
                                                         &clbck_data,
                                                         0); // for CAs - ignore port number
                if(ibDiagClbck.GetState())
                    goto exit;
            }

            if (has_capability && is_vports) {
                map_vportnum_vport vports = p_curr_port->VPorts;
                for (map_vportnum_vport::iterator vpI = vports.begin();
                        vpI != vports.end(); ++vpI) {
                     IBVPort *p_vport = vpI->second;

                     if(!p_vport)
                         continue;

                     clbck_data.m_data2 = p_vport;
                     this->ibis_obj.SMPVPortQoSConfigSLGetByDirect(p_curr_direct_route,
                                                                   &qos_config_sl,
                                                                   &clbck_data,
                                                                   p_vport->getVPortNum());

                     if(ibDiagClbck.GetState())
                        goto exit;
                }
            }
        } // ports loop
    } // nodes loop

exit:

    this->ibis_obj.MadRecAll();

    if (rc == IBDIAG_SUCCESS_CODE) {
        rc = ibDiagClbck.GetState();
        if (rc)
            this->SetLastError(ibDiagClbck.GetLastError());
        else if (!qos_config_sl_errors.empty())
            rc = IBDIAG_ERR_CODE_FABRIC_ERROR;
    }

    IBDIAG_RETURN(rc);
}

int IBDiag::DumpQoSConfigSLCSVTable(CSVOut &csv_out)
{
    IBDIAG_ENTER;
    int rc = IBDIAG_SUCCESS_CODE;

    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    csv_out.DumpStart(SECTION_QOS_CONFIG_SL);

    stringstream sstream;
    sstream << "NodeGUID,PortGUID,PortNum,SL,BandwidthShare,RateLimit" << endl;
    csv_out.WriteLine(sstream.str());

    struct SMP_QosConfigSL *p_curr_qos_config_sl = NULL;
    char curr_qos_config_sl_line[1024];

    for (u_int32_t nI = 0; nI < this->fabric_extended_info.getNodesVectorSize(); ++nI) {
        IBNode *p_curr_node = this->fabric_extended_info.getNodePtr(nI);
        if (!p_curr_node)                   //node not exist
            continue;

        if (p_curr_node->type != IB_CA_NODE)
            continue;

        bool rate_limit_flag = this->capability_module.IsSupportedSMPCapability(
                p_curr_node, EnSMPCapIsQoSConfigSLRateLimitSupported);

        bool bw_alloc_flag = this->capability_module.IsSupportedSMPCapability(
                p_curr_node, EnSMPCapIsQoSConfigSLAllocBWSupported);

        // go over all ports of this node:
        for (u_int32_t pI = 0; pI <= p_curr_node->numPorts; ++pI) {
            IBPort *p_curr_port = p_curr_node->getPort((phys_port_t)pI);

            if (!p_curr_port)
                continue;
            if (!p_curr_port->is_data_worthy())
                continue;

            p_curr_qos_config_sl = fabric_extended_info.getSMPQosConfigSL(p_curr_port->createIndex);
            if (!p_curr_qos_config_sl)           //info not exist
                continue;

            for (u_int32_t slI = 0; slI < IB_NUM_SL; ++slI) {
                memset(curr_qos_config_sl_line, 0, sizeof(curr_qos_config_sl_line));
                sstream.str("");
                sprintf(curr_qos_config_sl_line, U64H_FMT "," U64H_FMT",%d,%d,",
                        p_curr_node->guid_get(),
                        p_curr_port->guid_get(),
                        p_curr_port->num,
                        slI);

                sstream << curr_qos_config_sl_line;
                bw_alloc_flag? sstream << p_curr_qos_config_sl->BandwidthPerSL[slI].BandwidthShare
                        : sstream << "N/A";
                sstream << ",";
                rate_limit_flag? sstream << p_curr_qos_config_sl->BandwidthPerSL[slI].RateLimit
                        : sstream << "N/A";
                sstream << endl;

                csv_out.WriteLine(sstream.str());
            }
        }
    }

    csv_out.DumpEnd(SECTION_QOS_CONFIG_SL);
    IBDIAG_RETURN(rc);
}

int IBDiag::DumpVPortsQoSConfigSLCSVTable(CSVOut &csv_out)
{
    IBDIAG_ENTER;
    int rc = IBDIAG_SUCCESS_CODE;

    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    csv_out.DumpStart(SECTION_VPORTS_QOS_CONFIG_SL);

    stringstream sstream;
    sstream << "NodeGUID,PortGUID,PortNum,VPortGUID,VPortNum,SL,BandwidthShare,RateLimit" << endl;
    csv_out.WriteLine(sstream.str());

    struct SMP_QosConfigSL *p_curr_vport_qos_config_sl = NULL;
    char curr_vport_qos_config_sl_line[1024];

    for (u_int32_t nI = 0; nI < this->fabric_extended_info.getNodesVectorSize(); ++nI) {
        IBNode *p_curr_node = this->fabric_extended_info.getNodePtr(nI);
        if (!p_curr_node)                   //node not exist
            continue;

        if (p_curr_node->type != IB_CA_NODE)
            continue;

        bool vport_rate_limit_flag = this->capability_module.IsSupportedSMPCapability(
                p_curr_node, EnSMPCapIsQoSConfigSLVPortRateLimitSupported);

        bool vport_bw_alloc_flag = this->capability_module.IsSupportedSMPCapability(
                p_curr_node, EnSMPCapIsQoSConfigSLVPortAllocBWSupported);

        // go over all ports of this node:
        for (u_int32_t pI = 0; pI <= p_curr_node->numPorts; ++pI) {
            IBPort *p_curr_port = p_curr_node->getPort((phys_port_t)pI);

            if (!p_curr_port)
                continue;
            if (!p_curr_port->is_data_worthy())
                continue;

            map_vportnum_vport vports = p_curr_port->VPorts;
            for (map_vportnum_vport::iterator vpI = vports.begin(); vpI != vports.end(); ++vpI) {
                 IBVPort *p_vport = vpI->second;

                 if(!p_vport)
                    continue;

                 p_curr_vport_qos_config_sl =
                              fabric_extended_info.getSMPVPortQosConfigSL(p_vport->createIndex);
                 if (!p_curr_vport_qos_config_sl)           //info not exist
                     continue;

                 for (u_int32_t slI = 0; slI < IB_NUM_SL; ++slI) {
                     memset(curr_vport_qos_config_sl_line, 0, sizeof(curr_vport_qos_config_sl_line));
                     sstream.str("");
                     sprintf(curr_vport_qos_config_sl_line, U64H_FMT "," U64H_FMT",%d," U64H_FMT
                             ",%d,%d,",
                             p_curr_node->guid_get(),
                             p_curr_port->guid_get(),
                             p_curr_port->num,
                             p_vport->guid_get(),
                             p_vport->getVPortNum(),
                             slI);

                     sstream << curr_vport_qos_config_sl_line;
                     vport_bw_alloc_flag?
                     sstream << p_curr_vport_qos_config_sl->BandwidthPerSL[slI].BandwidthShare :
                     sstream << "N/A";
                     sstream << ",";
                     vport_rate_limit_flag?
                     sstream << p_curr_vport_qos_config_sl->BandwidthPerSL[slI].RateLimit :
                     sstream << "N/A";
                     sstream << endl;

                     csv_out.WriteLine(sstream.str());
                 }
            }
        }
    }

    csv_out.DumpEnd(SECTION_VPORTS_QOS_CONFIG_SL);
    IBDIAG_RETURN(rc);
}

//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS / ERR_CODE_IO_ERR
int IBDiag::BuildSMPTempSensing(
        list_p_fabric_general_err& temp_sensing_errors,
        progress_func_nodes_t progress_func)
{
    IBDIAG_ENTER;
    int rc = IBDIAG_SUCCESS_CODE;

    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    ResetAppData();
    ibDiagClbck.Set(this, &fabric_extended_info, &temp_sensing_errors);

    progress_bar_nodes_t progress_bar_nodes;
    CLEAR_STRUCT(progress_bar_nodes);

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &ibDiagClbck;

    // go over all nodes and collect counters
    struct SMP_TempSensing p_temp_sense;
    for (map_str_pnode::iterator nI =
            this->discovered_fabric.NodeByName.begin();
         nI != this->discovered_fabric.NodeByName.end();
         ++nI) {
        IBNode *p_curr_node = (*nI).second;
        if (!p_curr_node) {
            this->SetLastError("DB error - found null node in NodeByName map"  \
                               " for key = %s", (*nI).first.c_str());
            IBDIAG_RETURN(IBDIAG_ERR_CODE_DB_ERR);
        }

        ++progress_bar_nodes.nodes_found;
        if (p_curr_node->type == IB_SW_NODE)
            ++progress_bar_nodes.sw_found;
        else
            ++progress_bar_nodes.ca_found;
        if (progress_func)
            progress_func(&progress_bar_nodes,
                          &this->discover_progress_bar_nodes);

        struct SMP_NodeInfo * p_curr_node_info =
        this->fabric_extended_info.getSMPNodeInfo(
                p_curr_node->createIndex);
        if (!p_curr_node_info) {
            this->SetLastError(
                    "DB error - failed to get SMPNodeInfo for node: %s",
                    p_curr_node->name.c_str());
            rc = IBDIAG_ERR_CODE_DB_ERR;
            goto exit;
        }

        if (!this->capability_module.IsSupportedSMPCapability(
                    p_curr_node, EnSMPCapIsTemperatureSensingSupported)) {
            IBDIAG_LOG(TT_LOG_LEVEL_DEBUG,
                    "node %s: DeviceID %u (0x%x) Does not support"  \
                    " Temprature Sensing, skipping \n",
                    p_curr_node->name.c_str(),
                    p_curr_node_info->DeviceID,
                    p_curr_node_info->DeviceID);
            continue;
        }

        clbck_data.m_handle_data_func =
            IBDiagSMPTempSensingGetClbck;
        clbck_data.m_data1 = p_curr_node;

        //get direct route for this port
        direct_route_t *p_curr_direct_route =
        this->GetDirectRouteByNodeGuid(p_curr_node->guid_get());
        if (!p_curr_direct_route) {
            this->SetLastError("DB error - can't find direct route to node=%s",
                               p_curr_node->getName().c_str());
            rc = IBDIAG_ERR_CODE_DB_ERR;
            continue;
        }

        this->ibis_obj.SMPTempSensingDataGetByDirect(
                p_curr_direct_route,
                &p_temp_sense,
                &clbck_data);
    }

exit:
    this->ibis_obj.MadRecAll();

    if (rc == IBDIAG_SUCCESS_CODE) {
        rc = ibDiagClbck.GetState();
        if (rc)
            this->SetLastError(ibDiagClbck.GetLastError());
        else if (!temp_sensing_errors.empty())
            rc = IBDIAG_ERR_CODE_FABRIC_ERROR;
    }

    IBDIAG_RETURN(rc);
}


//Returns: SUCCESS_CODE / ERR_CODE_DISCOVERY_NOT_SUCCESS
int IBDiag::DumpTempSensingCSVTable(CSVOut &csv_out)
{
    IBDIAG_ENTER;
    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    csv_out.DumpStart(SECTION_TEMP_SENSING);

    stringstream sstream;
    sstream << "NodeGUID," << "CurrentTemperature" << endl;
    csv_out.WriteLine(sstream.str());

    struct SMP_TempSensing *p_curr_temp_sensing = NULL;
    char curr_temp_sensing_line[1024];

    for (u_int32_t i = 0; i < this->fabric_extended_info.getNodesVectorSize(); ++i) {
        IBNode *p_curr_node = this->fabric_extended_info.getNodePtr(i);
        if (!p_curr_node)                   //node not exist
            continue;
        p_curr_temp_sensing = this->fabric_extended_info.getSMPTempSensing(i);
        if (!p_curr_temp_sensing)           //info not exist
            continue;

        memset(curr_temp_sensing_line, 0, sizeof(curr_temp_sensing_line));
        sstream.str("");
        sprintf(curr_temp_sensing_line, U64H_FMT ",%d",
                p_curr_node->guid_get(),
                (int32_t)p_curr_temp_sensing->current_temp);
        sstream << curr_temp_sensing_line << endl;
        csv_out.WriteLine(sstream.str());
    }

    csv_out.DumpEnd(SECTION_TEMP_SENSING);
    IBDIAG_RETURN(IBDIAG_SUCCESS_CODE);
}

int IBDiag::BuildRouterInfoDB(list_p_fabric_general_err& errors,
                              progress_func_nodes_t progress_func)
{
    IBDIAG_ENTER;
    int rc = IBDIAG_SUCCESS_CODE;

    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    ResetAppData();
    ibDiagClbck.Set(this, &fabric_extended_info, &errors);

    progress_bar_nodes_t progress_bar_nodes;
    CLEAR_STRUCT(progress_bar_nodes);

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &ibDiagClbck;

    // go over all nodes and collect counters
    struct SMP_RouterInfo router_info;

    for (map_str_pnode::iterator nI =
            this->discovered_fabric.NodeByName.begin();
         nI != this->discovered_fabric.NodeByName.end();
         ++nI) {
        IBNode *p_curr_node = (*nI).second;
        if (!p_curr_node) {
            this->SetLastError("DB error - found null node in NodeByName map"  \
                               " for key = %s", (*nI).first.c_str());
            rc = IBDIAG_ERR_CODE_DB_ERR;
            goto exit;
        }

        ++progress_bar_nodes.nodes_found;
        if (p_curr_node->type == IB_SW_NODE)
            ++progress_bar_nodes.sw_found;
        else
            ++progress_bar_nodes.ca_found;
        if (progress_func)
            progress_func(&progress_bar_nodes,
                          &this->discover_progress_bar_nodes);
        if (p_curr_node->type != IB_RTR_NODE)
            continue;

        //TODO: add check of cpapbility mask, if the mad is supported

        clbck_data.m_handle_data_func =
            IBDiagSMPRouterInfoGetClbck;
        clbck_data.m_data1 = p_curr_node;

        //get direct route for this port
        direct_route_t *p_curr_direct_route =
        this->GetDirectRouteByNodeGuid(p_curr_node->guid_get());
        if (!p_curr_direct_route) {
            this->SetLastError("DB error - can't find direct route to node=%s",
                               p_curr_node->getName().c_str());
            rc = IBDIAG_ERR_CODE_DB_ERR;
            continue;
        }

        this->ibis_obj.SMPRouterInfoGetByDirect(
                p_curr_direct_route,
                &router_info,
                &clbck_data);
        if (ibDiagClbck.GetState())
            goto exit;
    }

exit:
    this->ibis_obj.MadRecAll();

    if (rc == IBDIAG_SUCCESS_CODE) {
        rc = ibDiagClbck.GetState();
        if (rc)
            this->SetLastError(ibDiagClbck.GetLastError());
        else if (!errors.empty())
            rc = IBDIAG_ERR_CODE_FABRIC_ERROR;
    }

    IBDIAG_RETURN(rc);
}

int IBDiag::BuildRouterTable(list_p_fabric_general_err& errors,
                             progress_func_nodes_t progress_func)
{
    IBDIAG_ENTER;
    int rc = IBDIAG_SUCCESS_CODE;

    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    ibDiagClbck.Set(this, &fabric_extended_info, &errors);

    progress_bar_nodes_t progress_bar_nodes;
    CLEAR_STRUCT(progress_bar_nodes);

    clbck_data_t clbck_data;
    clbck_data.m_p_obj = &ibDiagClbck;

    // go over all nodes and collect counters
    for (map_str_pnode::iterator nI =
            this->discovered_fabric.NodeByName.begin();
         nI != this->discovered_fabric.NodeByName.end();
         ++nI) {
        IBNode *p_curr_node = (*nI).second;
        if (!p_curr_node) {
            this->SetLastError("DB error - found null node in NodeByName map"  \
                               " for key = %s", (*nI).first.c_str());
            rc = IBDIAG_ERR_CODE_DB_ERR;
	    goto exit;
        }

        ++progress_bar_nodes.nodes_found;
        if (p_curr_node->type == IB_SW_NODE)
            ++progress_bar_nodes.sw_found;
        else
            ++progress_bar_nodes.ca_found;
        if (progress_func)
            progress_func(&progress_bar_nodes,
                          &this->discover_progress_bar_nodes);
        if (p_curr_node->type != IB_RTR_NODE)
            continue;

        struct SMP_RouterInfo *p_router_info = this->fabric_extended_info.
                                               getSMPRouterInfo(
                                                   p_curr_node->createIndex);
        // the info doesn't exist or there are no entries
        // in neither adjacent site local subnets router table
        // nor in the next hop table
        if (!p_router_info ||
            (p_router_info->AdjSiteLocalSubnetsTblTop == 0 &&
             p_router_info->NextHopTableTop == 0))
            continue;

        u_int8_t adj_num_blocks =
            (u_int8_t)NUM_OF_ROUTER_TABLE_ENTRIES(
                          p_router_info->AdjSiteLocalSubnetsTblTop,
                          IB_ROUTER_ELEMENTS_IN_ADJACENT_BLOCK);
        //TODO: add check of cpapbility mask, if the mad is supported

        clbck_data.m_data1 = p_curr_node;

        //get direct route for this port
        direct_route_t *p_curr_direct_route =
        this->GetDirectRouteByNodeGuid(p_curr_node->guid_get());
        if (!p_curr_direct_route) {
            this->SetLastError("DB error - can't find direct route to node=%s",
                               p_curr_node->getName().c_str());
            rc = IBDIAG_ERR_CODE_DB_ERR;
            goto exit;
        }

        clbck_data.m_handle_data_func =
            IBDiagSMPAdjRouterTableGetClbck;
        struct SMP_AdjSiteLocalSubnTbl adj_router_table;
        for (u_int8_t i = 0; i < adj_num_blocks; i++) {
            //save the block number
            clbck_data.m_data2 = (void *)(uintptr_t)i;
            this->ibis_obj.SMPAdjRouterTableGetByDirect(
                        p_curr_direct_route,
                        i, //send block num
                        &adj_router_table,
                        &clbck_data);
            if (ibDiagClbck.GetState())
                goto exit;
        }

        // now get the next hop table
        u_int32_t nh_num_blocks = (u_int32_t)NUM_OF_ROUTER_TABLE_ENTRIES(
                                      p_router_info->NextHopTableTop,
                                      IB_ROUTER_ELEMENTS_IN_NEXT_HOP_BLOCK);
        clbck_data.m_handle_data_func =
            IBDiagSMPNextHopRouterTableGetClbck;
        struct SMP_NextHopTbl nh_router_table;
        for (u_int32_t j = 0; j < nh_num_blocks; j++) {
            //save the block number
            clbck_data.m_data2 = (void *)(uintptr_t)j;
            this->ibis_obj.SMPNextHopRouterTableGetByDirect(
                        p_curr_direct_route,
                        j, //send block num
                        &nh_router_table,
                        &clbck_data);
            if (ibDiagClbck.GetState())
                goto exit;
        }
    }

exit:
    this->ibis_obj.MadRecAll();

    if (rc == IBDIAG_SUCCESS_CODE) {
        rc = ibDiagClbck.GetState();
        if (rc)
            this->SetLastError(ibDiagClbck.GetLastError());
        else if (!errors.empty())
            rc = IBDIAG_ERR_CODE_FABRIC_ERROR;
    }

    IBDIAG_RETURN(rc);
}

int IBDiag::DumpRoutersInfoCSVTable(CSVOut &csv_out)
{
    IBDIAG_ENTER;
    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    csv_out.DumpStart(SECTION_ROUTERS_INFO);

    stringstream sstream;
    sstream << "NodeGUID,CapabilityMask,NextHopTableCap,NextHopTableTop,"
         << "AdjChange,NHChange,IsGlbSA,AdjacentSiteLocalSubnetsTableTop,"
         << "AdjacentSiteLocalSubnetsTableCap,MaxMulticastTTL" << endl;
    csv_out.WriteLine(sstream.str());

    struct SMP_RouterInfo *p_router_info = NULL;
    char line[1024] = {'\0'};

    for (u_int32_t i = 0; i < this->fabric_extended_info.getNodesVectorSize();
         i++) {
        IBNode *p_curr_node = this->fabric_extended_info.getNodePtr(i);
        if (!p_curr_node)                   //node not exist
            continue;
        p_router_info = this->fabric_extended_info.getSMPRouterInfo(i);
        if (!p_router_info)           //info not exist
            continue;

        sstream.str("");
        sprintf(line,
                U64H_FMT ","
                U32H_FMT "," U32H_FMT "," U32H_FMT ","
                U8H_FMT "," U8H_FMT "," U8H_FMT ","
                U8H_FMT "," U8H_FMT "," U8H_FMT,
                p_curr_node->guid_get(),
                p_router_info->CapabilityMask,
                p_router_info->NextHopTableCap, p_router_info->NextHopTableTop,
                p_router_info->AdjChange, p_router_info->NHChange,
                p_router_info->IsGlbSA,
                p_router_info->AdjSiteLocalSubnetsTblTop,
                p_router_info->AdjSiteLocalSubnetsTblCap,
                p_router_info->MaxMulticastTTL);
        sstream << line << endl;
        csv_out.WriteLine(sstream.str());
    }

    csv_out.DumpEnd(SECTION_ROUTERS_INFO);
    IBDIAG_RETURN(IBDIAG_SUCCESS_CODE);
}

int IBDiag::DumpRoutersAdjSiteLocalSubnetCSVTable(CSVOut &csv_out)
{
    IBDIAG_ENTER;
    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    csv_out.DumpStart(SECTION_ROUTERS_ADJ_TBL);

    stringstream sstream;
    sstream << "NodeGUID,BlockNum,RecordNum,SubnetPrefix,Pkey,MasterSMLID" << endl;
    csv_out.WriteLine(sstream.str());

    struct SMP_RouterInfo *p_router_info = NULL;
    char line[1024] = {'\0'};

    for (u_int32_t i = 0; i < this->fabric_extended_info.getNodesVectorSize();
         i++) {
        IBNode *p_curr_node = this->fabric_extended_info.getNodePtr(i);
        if (!p_curr_node)                   //node not exist
            continue;
        p_router_info = this->fabric_extended_info.getSMPRouterInfo(i);
        if (!p_router_info)           //info not exist
            continue;

        u_int8_t num_entries =
            p_router_info->AdjSiteLocalSubnetsTblTop;

        u_int8_t block_num = 0, idx = 0;
        SMP_AdjSiteLocalSubnTbl *p_adj_rtr_tbl = NULL;
        for (u_int8_t j = 0; j < num_entries; j++) {
                /* get the next block, initialized already to first */
                if ((j % IB_ROUTER_ELEMENTS_IN_ADJACENT_BLOCK) == 0) {
                        block_num = j / IB_ROUTER_ELEMENTS_IN_ADJACENT_BLOCK;
                        p_adj_rtr_tbl = this->fabric_extended_info.
                                                  getSMPAdjSiteLocalSubnTbl(
                                                      i, block_num);
                }
                if (!p_adj_rtr_tbl)
                    continue;

                idx = (u_int8_t)(j % IB_ROUTER_ELEMENTS_IN_ADJACENT_BLOCK);
                sstream.str("");
                sprintf(line,
                        U64H_FMT "," U8H_FMT "," U8H_FMT "," U16H_FMT ","
                        U16H_FMT "," U16H_FMT,
                        p_curr_node->guid_get(),
                        block_num, idx, p_adj_rtr_tbl->Record[idx].SubnetPrefix,
                        p_adj_rtr_tbl->Record[idx].Pkey,
                        p_adj_rtr_tbl->Record[idx].MasterSMLID);
                sstream << line << endl;
                csv_out.WriteLine(sstream.str());
        }

    }

    csv_out.DumpEnd(SECTION_ROUTERS_ADJ_TBL);
    IBDIAG_RETURN(IBDIAG_SUCCESS_CODE);
}

int IBDiag::DumpRoutersNextHopCSVTable(CSVOut &csv_out)
{
    IBDIAG_ENTER;
    if (!this->IsDiscoveryDone())
        IBDIAG_RETURN(IBDIAG_ERR_CODE_DISCOVERY_NOT_SUCCESS);

    csv_out.DumpStart(SECTION_ROUTERS_NEXT_HOP_TBL);

    stringstream sstream;
    sstream << "NodeGUID,BlockNum,RecordNum,SubnetPrefix,Pkey,Weight" << endl;
    csv_out.WriteLine(sstream.str());

    struct SMP_RouterInfo *p_router_info = NULL;
    char line[1024] = {'\0'};

    for (u_int32_t i = 0; i < this->fabric_extended_info.getNodesVectorSize();
         i++) {
        IBNode *p_curr_node = this->fabric_extended_info.getNodePtr(i);
        if (!p_curr_node)                   //node not exist
            continue;
        p_router_info = this->fabric_extended_info.getSMPRouterInfo(i);
        if (!p_router_info)           //info not exist
            continue;

        u_int32_t num_entries =
            p_router_info->NextHopTableTop;

        u_int32_t block_num = 0, idx = 0;
        SMP_NextHopTbl *p_nh_rtr_tbl = NULL;
        for (u_int32_t j = 0; j < num_entries; j++) {
                /* get the next block, initialized already to first */
                if ((j % IB_ROUTER_ELEMENTS_IN_NEXT_HOP_BLOCK) == 0) {
                        block_num = j / IB_ROUTER_ELEMENTS_IN_NEXT_HOP_BLOCK;
                        p_nh_rtr_tbl = this->fabric_extended_info.
                                                  getSMPNextHopTbl(
                                                      i, block_num);
                }
                if (!p_nh_rtr_tbl)
                    continue;

                idx = (u_int32_t)(j % IB_ROUTER_ELEMENTS_IN_NEXT_HOP_BLOCK);
                sstream.str("");
                sprintf(line,
                        U64H_FMT "," U32H_FMT "," U32H_FMT "," U64H_FMT ","
                        U16H_FMT "," U8H_FMT,
                        p_curr_node->guid_get(),
                        block_num, idx, p_nh_rtr_tbl->Record[idx].SubnetPrefix,
                        p_nh_rtr_tbl->Record[idx].Pkey,
                        p_nh_rtr_tbl->Record[idx].Weight);
                sstream << line << endl;
                csv_out.WriteLine(sstream.str());
        }

    }
    csv_out.DumpEnd(SECTION_ROUTERS_NEXT_HOP_TBL);
    IBDIAG_RETURN(IBDIAG_SUCCESS_CODE);
}
