/*
 *  Copyright (c) 2004-2019 Mellanox Technologies LTD. All rights reserved
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */


#ifndef _IBDIAGAPP_H_
#define _IBDIAGAPP_H_

#include <stdlib.h>

#include <infiniband/ibdiag/ibdiag.h>
#include <misc/cmdparser/cmdparser.h>

#include "ibdiagapp_types.h"
#include "ibdiagnet_stages.h"
#include "ibdiagnet_plugins.h"
#include "ibdiagapp_lc_plugin.h"

/******************************************************/
class IBDiagapp : public CommandLineRequester {
private:

protected:
    ////////////////////
    //members
    ////////////////////
    typedef struct skip_obj {
        u_int32_t value;
        string name;
    } skip_obj_t;

    vector <struct skip_obj> skip_options_list;

    enum {FAILED_TO_CONSTRUCT, NEED_TO_GET_ARGS, NOT_INITILIAZED, READY} ibdiagnet_status;
    class Stage *stages[NUM_OF_STAGES];
    list_p_symbols_plugin loaded_plugins_list;
    list_string not_to_load_plugins_list;
    IBDiag ibdiag_obj;
    CommandLineParser *p_main_parser;

    unsigned int num_of_errors_to_screen;
    u_int32_t m_max_smps_on_wire;
    u_int32_t m_max_gmps_on_wire;

    string base_path;

    CSVOut csv_out;

    LCPluginMgr m_lc_plugins_mgr;   //the manager of loosely coupled plugins

    FilesBasedMKeyManager *p_mkeymngr;

    string config_file;
    string generated_config_file;
    string mkey_val;
    string mkey_path;

    ////////////////////
    //methods
    ////////////////////
    int ParseSkipList(const char *skip_list);
    int ParsePOption(const char *option_p_list, map_str_uint64& counters_to_threshold_map);
    int ParseSkipPluginList(const char *skip_plugin_list);
    int ParseMKeyValue();
    int ParseMKeyFiles();
    int ParseConfigFile();
    int CreateConfigFile(string file_path);
    int HandleMKeyParams();
    void PrintSummary();

    void LoadPlugins();
    void UnloadPlugins();

    //Returns: 0 - OK / 1 - error / 2 - OK but need to exit
    int GetLocalGUIDFromUser(u_int64_t& guid);

    int CreateOutputPath(string value);

    //Returns: 0 - OK / 1 - error
    int DumpStartInfoCSVTable(int argc, char **argv);

    //Returns: 0 - OK / 1 - error / 2 - OK but need to exit / 3 - parsing error
    virtual int HandleOption(string name, string value);

    virtual Stage* CreateStage(uint8_t stageIndex) = 0;
    virtual string GetToolName() = 0;

public:
    ////////////////////
    //methods
    ////////////////////
    IBDiagapp(const char * default_output_path);
    virtual ~IBDiagapp();

    int PostInit();
    //Returns: 0 - OK / 1 - error / 2 - OK but need to exit
    int GetOpt(int argc, char **argv);

    //Returns: 0 - OK / 1 - error
    virtual int Init();
    int RunAllStages();
    int Run(int argc, char **argv);

    int IsValidLid(lid_t lid) {
        if(lid > IB_MAX_UCAST_LID) {
            PRINT("lid should be less than %d\n",
                  IB_MAX_UCAST_LID);
            IBDIAGNET_RETURN(1);
        }
        IBDIAGNET_RETURN(0);
    }

    bool isValidLoadFromFilePath();
};


#endif          /* not defined _IBDIAGAPP_H_ */
