/*
 *  Copyright (c) 2004-2014 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#ifndef _IBDIAGAPP_LC_PLUGIN_H_
#define _IBDIAGAPP_LC_PLUGIN_H_

#include <infiniband/ibdiag/ibdiagnet/ibdiagnet_plugins_ifc.h>
#include <infiniband/ibdiag/ibdiagnet/ibdiagapp_lc_plugin_api.h>

#define OPTION_PLUGIN_ACT_SUFFIX            "_plugin_act"
#define OPTION_PLUGIN_ACT_SHORT             ' '
#define OPTION_PLUGIN_ACT_ARG_DESC          ""
#define OPTION_PLUGIN_ACT_DESC_PREFIX       "Activates "

#define OPTION_PLUGIN_OPT_SUFFIX           "_plugin_opt"
#define OPTION_PLUGIN_OPT_SHORT             ' '
#define OPTION_PLUGIN_OPT_ARG_DESC          "<\"plugin parameters\">"
#define OPTION_PLUGIN_OPT_DESC_PREFIX       "Specifies the parameters " \
                                            "to pass, using \"\" to "
#define LC_PLUGIN_DESC                      "This is ibdiagnet plugin."


//forward declarations
class LCPlugin;
class LCPluginMgr;
struct symbols_plugin;

extern LCPluginMgr *g_p_lc_plugins_mgr;

//registry data per one lc plugin's object
typedef struct plugin_object_registry_data {
    LCPlugin *p_lc_plugin_object; //LCPlugin that holds that object
    PF_RegisterParams registry_params;//registry params of the object
} plugin_object_registry_data_t;

//maps between plugin's object name and its registry data
typedef map<string, plugin_object_registry_data_t> map_object_str_register_data_t;

//this struct holds lc plugin's objects, and exit function
typedef struct lc_plugin_registry {
    map_object_str_register_data_t objects_data;
    PF_ExitFunc exit_func;
} lc_plugin_registry_data_t;

//maps between lc plugin's name and its data
typedef map<string, lc_plugin_registry_data_t> map_plugin_registry_t;
typedef vector<plPortObject_t> pl_vec_port_t;
typedef list<string> pl_list_names_t;

//This plugin is another abstraction level to represent
//loosely coupled plugin's objects to ibdiagnet, as plugins that ibdiagnet
//already familiar with.
//This class doesn't represent the whole loosely couple plugin,
//rather one of its registerd objects. It's name, is the name of the object,
//that plugin registers within ibdiagnet.
class LCPlugin : public Plugin {
    ////////////////////
    //members
    ////////////////////

    //object of lc plugin's object
    IObjectLCPlugin     *m_p_plugin_object;
    PF_DestroyFunc       m_plugin_object_destroy_func;
    string               m_plugin_act_option;
    string               m_plugin_opt_option;
    LCPluginMgr         *m_p_plugins_mgr;
    // a replica of current transaction id from ibis
    u_int64_t            m_tid;

public:
    ////////////////////
    //methods
    ////////////////////
    LCPlugin(IBDiag              *p_ibdiag,
             const string        &plugin_object_name,
             IObjectLCPlugin     *x_object,
             PF_DestroyFunc       plugin_object_destroy_func,
             LCPluginMgr         *p_plugins_mgr);
    virtual ~LCPlugin();
    virtual void Init();
    virtual int HandleOption(string opt_name, string value);
    virtual int Prepare();
    virtual int RetrieveInfo();
    virtual int RunCheck();
};

//The manager of loosely coupled plugins
class LCPluginMgr {
private:
    ////////////////////
    //members
    ////////////////////

    //all the plugins' objects
    IBDiag                *m_p_ibdiag;
    //a name of the plugin, whose objects are currently being registered
    //TBD, remove it and use only plugin_handle
    string                 m_plugin_to_register;
    pl_list_names_t        m_plugin_names;
    //store plugin's registry information
    map_plugin_registry_t  m_plugins_registry_map;
    //discovered fabric to be passed to plugins
    pl_vec_port_t          m_fabric_ports;
    PF_PluginAPI_Version   m_version;//ibdiagnet version of api

    //methods
    int AcquireFabricPorts();
    void ReleasePlugin(string &plugin_name);
    void ReleasePlugin(map_plugin_registry_t::iterator &plugin_it);
public:
    //methods
    LCPluginMgr();
    virtual ~LCPluginMgr();

    int Init(IN IBDiag *p_ibdiag);
    int LoadLCPlugin(IN string plugin_name,
                     IN PF_InitFunc init_plugin,
                     OUT struct symbols_plugin *p_res);

    int GetPortsPtr(const plPortObject_t ** pp_fabric_ports,
                    u_int32_t &ports_num);
    int32_t RegisterLCPluginObject(const char *plugin_object_type,
                                   const PF_RegisterParams *params);
    int32_t InvokeService(const char * serviceName,
                          void * serviceParams);
};


#endif //_IBDIAGAPP_LC_PLUGIN_H_
