/*
 *  Copyright (c) 2004-2013 Mellanox Technologies LTD. All rights reserved
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */


#ifndef _IBDIAGNET_DEBUG_WRAP_H_
#define _IBDIAGNET_DEBUG_WRAP_H_


#include <errno.h>
#include <sys/stat.h>

#include <misc/tool_trace/tool_trace.h>

//#include "ibdiagnet_types.h"

/******************************************************/
class DebugWrap {
public:
    DebugWrap(string path) {
        //the constructor is called before the function main begins.
        umask(0); //Any created files will have mode S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH (0666)

        //create output dir
        if (mkdir(path.c_str(), S_IRWXU | S_IRWXG | S_IRWXO) < 0) {
            if (errno != EEXIST) {
                printf("Failed to set output path to %s\n", path.c_str());
                exit(1);
            }
        }

        path += DEBUG_FILE;
        if (tt_log_construct_v2(0xff,   /* need_to_flush */
                path.c_str(),           /* log file */
                DEFAULT_LOG_SIZE,       /* max size (in MB) */
                0)) {                   /* append to prev log */
            printf("-E- Failed to open log file=%s\n", DEBUG_FILE);
            exit(1);
        }
    }

    ~DebugWrap() {
        tt_log_destroy();
    }

    void ChangeDebugLog(string new_path) {
        tt_log_destroy();
        string path = new_path;
        path += DEBUG_FILE;
        if (tt_log_construct_v2(0xff,   /* need_to_flush */
                path.c_str(),           /* log file */
                DEFAULT_LOG_SIZE,       /* max size (in MB) */
                0)) {                   /* append to prev log */
            printf("-E- Failed to open log file=%s\n", DEBUG_FILE);
            exit(1);
        }
    }

};


#endif          /* not defined _IBDIAGNET_DEBUG_WRAP_H_ */
