/*
 *  Copyright (c) 2004-2013 Mellanox Technologies LTD. All rights reserved
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */


#ifndef _CABLE_DIAG_ERRS_H_
#define _CABLE_DIAG_ERRS_H_

#include <stdio.h>
#include <stdlib.h>

#include <infiniband/ibdiag/ibdiag_fabric_errs.h>
#include <infiniband/ibdiag/ibdiagnet/ibdiagnet_plugins_ifc.h>


/******************************************************/
/*
 * NOTE: All the following defines must not contain ','
 */

/* event names */
#define FER_EYE_OPEN_INFO_AUTONEG       "EYE_OPEN_INFO_AUTONEG_IN_PROGRESS"
#define FER_EYE_OPEN_INFO_ERR_GENERAL   "EYE_OPEN_INFO_ERR_GENERAL"
#define FER_EYE_BOUND_BELOW_THRESH      "EYE_BOUND_BELOW_THRESH"
#define FER_EYE_BOUND_ABOVE_THRESH      "EYE_BOUND_ABOVE_THRESH"
#define FER_CABLE_INFO_NO_EEPROM        "CABLE_INFO_NO_EEPROM"
#define FER_CABLE_INFO_BAD_QSFP         "CABLE_INFO_BAD_QSFP_CABLE"
#define FER_CABLE_INFO_ERR_GENERAL      "CABLE_INFO_ERR_GENERAL"


/******************************************************/
enum CABLE_INFO_VS_STATUS {
    CABLEINFO_VS_STATUS_SUCCESS = 0x00,
    CABLEINFO_VS_STATUS_INVALID_PORT = 0x01,
    CABLEINFO_VS_STATUS_PORT_NOT_QSFP = 0x02,
    CABLEINFO_VS_STATUS_CABLE_NOT_CONNECTED = 0x03,
    CABLEINFO_VS_STATUS_NO_EEPROM = 0x0004,
    CABLEINFO_VS_STATUS_INVALID_PAGE_NUM = 0x05,
    CABLEINFO_VS_STATUS_INVALID_ADDR_OR_SIZE = 0x06,
    CABLEINFO_VS_STATUS_INVALID_SLAVE_ADDR = 0x07,
    CABLEINFO_VS_STATUS_BAD_QSFP_CABLE = 0x08
};


enum EYE_OPEN_INFO_AUTONEGO_VALUE {
    AUTONEGO_VALUE_SUCCESS = 0x00,
    AUTONEGO_VALUE_NOT_QDR = 0x01,
    AUTONEGO_VALUE_IN_PROGRESS = 0x02,
    AUTONEGO_VALUE_LINK_DOWN = 0x03
};


/******************************************************/
string ConvertCableInfoVSStatusToStr(IN u_int8_t vs_status);
string ConvertAutonegValueToStr(IN u_int8_t autoneg_val);


/******************************************************/
class FabricErrEyeOpenInfoRetrieveAutonegInProgress : public FabricErrPort {
public:
    FabricErrEyeOpenInfoRetrieveAutonegInProgress(IBPort *p_port);
    ~FabricErrEyeOpenInfoRetrieveAutonegInProgress() {}
};

class FabricErrEyeOpenInfoRetrieveGeneral : public FabricErrPort {
public:
    FabricErrEyeOpenInfoRetrieveGeneral(IBPort *p_port, u_int8_t autoneg_val);
    ~FabricErrEyeOpenInfoRetrieveGeneral() {}
};


/******************************************************/
class FabricErrEyeBoundBelowThresh : public FabricErrPort {
private:
    int lane_num;
    u_int16_t negative_bound;
    u_int16_t positive_bound;
    u_int16_t sum_bound_threshold;
public:
    FabricErrEyeBoundBelowThresh(IBPort *p_port, int lane_num,
            u_int16_t negative_bound, u_int16_t positive_bound, u_int16_t threshold);
    ~FabricErrEyeBoundBelowThresh() {}
};

class FabricErrEyeBoundAboveThresh : public FabricErrPort {
private:
    int lane_num;
    u_int16_t negative_bound;
    u_int16_t positive_bound;
    u_int16_t sum_bound_threshold;
public:
    FabricErrEyeBoundAboveThresh(IBPort *p_port, int lane_num,
            u_int16_t negative_bound, u_int16_t positive_bound, u_int16_t threshold);
    ~FabricErrEyeBoundAboveThresh() {}
};


/******************************************************/
class FabricErrCableInfoRetrieveBadQSFPFound : public FabricErrNode {
public:
    FabricErrCableInfoRetrieveBadQSFPFound(IBNode *p_node);
    ~FabricErrCableInfoRetrieveBadQSFPFound() {}
};

class FabricErrCableInfoRetrieveNoEEprom : public FabricErrPort {
public:
    FabricErrCableInfoRetrieveNoEEprom(IBPort *p_port);
    ~FabricErrCableInfoRetrieveNoEEprom() {}
};

class FabricErrCableInfoRetrieveGeneral : public FabricErrPort {
public:
    FabricErrCableInfoRetrieveGeneral(IBPort *p_port,
                                      u_int8_t eeprom_index,
                                      u_int8_t page_num,
                                      u_int8_t vs_status);
    ~FabricErrCableInfoRetrieveGeneral() {}
};

#endif          /* not defined _CABLE_DIAG_ERRS_H_ */

