/*
 *  Copyright (c) 2004-2018 Mellanox Technologies LTD. All rights reserved
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */


#ifndef _CABLE_DIAG_TYPES_H_
#define _CABLE_DIAG_TYPES_H_


/******************************************************/
/* some defines */
#define NOT_SUPPORT_CHECK 0
#define NOT_SUPPORT_CAPABILTY 2
#define SUPPORT_CAPABILTY 1

#define NOT_VISITED 0
#define VISITED 1

#define MAX(a, b) ((a) > (b) ? (a) : (b))

#define TO_STR_(x)  #x
#define TO_STR(x)   TO_STR_(x)


/******************************************************/
/* infiniband defines */
#define IB_ATTR_SMP_CABLE_INFO 0xff60
#define IB_ATTR_SMP_EYE_OPENER 0xff70

#define SIZE_OF_DATE_CODE 6
#define LOT_OFFSET 6

#define MLNX_OUI "0x2c9"

#define MAX_GET_ADDR_ARR 2

#define MAX_PHASES  3

#define MAX_GROUP_LANES 3

enum EEPROM_CABLE {
    EEPROM_CABLE_ADDR_FIRST_GET = 128,
    EEPROM_CABLE_ADDR_SECOND_GET = 168,
    EEPROM_CABLE_ADDR_THIRD_GET = 0,
    CABLE_ADDR_48 = 48, //ADDR_48_79
    CABLE_ADDR_80 = 80, //ADDR_80_127
    CABLE_ADDR_212 = 212, //ADDR_212_255
    CABLE_ADDR_128 = 128, //ADDR_128_171
    CABLE_ADDR_176 = 176, //ADDR_176_211

    EEPROM_CABLE_SIZE_FIRST_GET = 40,
    EEPROM_CABLE_SIZE_SECOND_GET = 44,
    EEPROM_CABLE_SIZE_THIRD_GET = 48,
    CABLE_SIZE_48 = 48, //SIZE_80_127
    CABLE_SIZE_43 = 43, //SIZE_212_255
    CABLE_SIZE_44 = 44, //SIZE_128_171
    CABLE_SIZE_36 = 36,
    CABLE_SIZE_32 = 32, //SIZE_48_79

    EEPROM_CABLE_PAGE_FIRST_GET = 0x0,
    CABLE_PAGE_03 = 0x3,
    CABLE_PAGE_F0 = 0xF0,
    CABLE_PAGE_E9 = 0xE9,

    CABLE_PAGE_F0_PASSWOED = 0x3a4105c,
    CABLE_PAGE_E9_PASSWOED = 0x23581945,

    EEPROM_CABLE_DEVICE_ADDR = 0x50
};

#define QSFP_ID_VALUE       0x0c
#define QSFP_PLUS_ID_VALUE  0x0d
#define QSFP28_ID_VALUE     0x11

#define EYE_MIN_DEFAULT_THRESH 22
#define EYE_MAX_DEFAULT_THRESH 65

#define CABLE_TYPE_COPPER_UNEQUALIZED       0xa
#define CABLE_TYPE_COPPER_PASSIVE_EQUALIZED 0xb

/* This temp value is only for initialize so if we can't pull the temperature from the
 * cable we can use this value to write "N/A" in the cables/db_csv files */
#define CABLE_UNDEFINED_TEMPERATURE   "N/A"
#define CABLE_UNDEFINED   "N/A"

#define CABLE_MIN_SUPPORT_TEMPERATURE -40
#define CABLE_MAX_SUPPORT_TEMPERATURE 125
/******************************************************/
/* sections defines */
#define SECTION_EYE_OPEN_INFO               "EYE_OPEN_INFO"
#define SECTION_CABLE_INFO                  "CABLE_INFO"
#define SECTION_CABLE_INFO_DB_1             "MOD_DB_1"

#define PageE9Addr176_FIELDS_SIZE           16
#define PageE9Addr128_FIELDS_SIZE           22
#define Page0_FIELDS_SIZE                   6
#define SECTION_CABLE_INFO_DB_1_FIELDS_NUM  Page0_FIELDS_SIZE + PageE9Addr128_FIELDS_SIZE +   \
                                            PageE9Addr176_FIELDS_SIZE

/* checks defines */
#define CHECK_NAME_EYE_OPEN_INFO_RETRIEVE   "Eye Open Info retrieving"
#define CHECK_NAME_CABLE_INFO_RETRIEVE      "Cable Info retrieving"
#define CHECK_NAME_CABLE_INFO_TEMP_RETRIEVE "Cable Info Temperature retrieving"
#define CHECK_NAME_EYE_OPEN_BOUND           "Eye open Bound Check"


/******************************************************/
/* options defines */
#define CABLES_FILE "ibdiagnet2.cables"
#define EYE_EXPERT_FILE "ibdiagnet2.port_attr"


#define PLUGIN_NAME "Cable Diagnostic (Plugin)"
#define PLUGIN_DESC "This plugin perform cable diagnostic. It can collect cable info "  \
    "(vendor, PN, OUI etc..) on each valid QSFP cable if specified.\n"  \
    "It produces the following files in the output directory (see below):\n"    \
    "(*) " TO_STR(CABLES_FILE) " - In case specify to collect cable info this file "    \
    "will contain all cables info that was collected."


#define OPTION_CABLE_INFO_GET "get_cable_info"
#define OPTION_CABLE_INFO_GET_SHORT ' '
#define OPTION_CABLE_INFO_GET_ARG_DESC ""
#define OPTION_CABLE_INFO_GET_DESC "Indicates to query all QSFP cables for cable "  \
    "information. Cable information will be stored in " TO_STR(CABLES_FILE) "."


#define OPTION_CABLE_INFO_DICONNECTED "cable_info_disconnected"
#define OPTION_CABLE_INFO_DICONNECTED_SHORT ' '
#define OPTION_CABLE_INFO_DICONNECTED_ARG_DESC ""
#define OPTION_CABLE_INFO_DICONNECTED_DESC "Get cable info on disconnected ports. " \
    "used only with get_cable_info flag."

#define OPTION_EYE_OPEN_INFO_GET "get_eye_open_info"
#define OPTION_EYE_OPEN_INFO_GET_SHORT ' '
#define OPTION_EYE_OPEN_INFO_GET_ARG_DESC ""
#define OPTION_EYE_OPEN_INFO_GET_DESC "Indicates to query all QDR ports for "   \
    "eye open information. (Note: This will cause positive and negative bounds to be "  \
    "dumped to " TO_STR(DB_FILE) ")."

#define OPTION_EYE_OPEN_INFO_CHECK "eye_check"
#define OPTION_EYE_OPEN_INFO_CHECK_SHORT ' '
#define OPTION_EYE_OPEN_INFO_CHECK_ARG_DESC ""
#define OPTION_EYE_OPEN_INFO_CHECK_DESC "Indicates to perform bounds sum check "    \
    "(meaningless if option " TO_STR(OPTION_EYE_OPEN_INFO_GET) " is not specified)."

#define OPTION_EYE_EXPERT "eye_expert"
#define OPTION_EYE_EXPERT_SHORT ' '
#define OPTION_EYE_EXPERT_ARG_DESC ""
#define OPTION_EYE_EXPERT_DESC "Note: will create " TO_STR(EYE_EXPERT_FILE) "."

#define OPTION_EYE_MIN_THRESH "eye_min_thresh"
#define OPTION_EYE_MIN_THRESH_SHORT ' '
#define OPTION_EYE_MIN_THRESH_ARG_DESC "<threshold>"
#define OPTION_EYE_MIN_THRESH_DESC "Set the threshold for warnings to be generated "    \
    "when the sum of the positive and negative bounds is below this value\n"    \
    "(default=" TO_STR(EYE_MIN_DEFAULT_THRESH) ", meaningless if "  \
    "option " TO_STR(OPTION_EYE_OPEN_INFO_CHECK) " is not specified)."
#define OPTION_EYE_MIN_THRESH_DEF_VAL TO_STR(EYE_MIN_DEFAULT_THRESH)

#define OPTION_EYE_MAX_THRESH "eye_max_thresh"
#define OPTION_EYE_MAX_THRESH_SHORT ' '
#define OPTION_EYE_MAX_THRESH_ARG_DESC "<threshold>"
#define OPTION_EYE_MAX_THRESH_DESC "Set the threshold for warnings to be generated "    \
    "when the sum of the positive and negative bounds is above this value\n"    \
    "(default=" TO_STR(EYE_MAX_DEFAULT_THRESH) ", meaningless if "  \
    "option " TO_STR(OPTION_EYE_OPEN_INFO_CHECK) " is not specified)."
#define OPTION_EYE_MAX_THRESH_DEF_VAL TO_STR(EYE_MAX_DEFAULT_THRESH)


#endif          /* not defined _CABLE_DIAG_TYPES_H_ */
