/*
 * Copyright (c) 2004-2019 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#include "acc_reg_key.h"


AccRegKeyPortLane::AccRegKeyPortLane(uint64_t ng,
                                     uint64_t pg,
                                     uint8_t pn,
                                     uint8_t ln,
                                     uint8_t idx)
{
    IBDIAGNET_ENTER;
    node_guid = ng;
    port_guid = pg;
    port_num = pn;
    lane = ln;
    idx_in_lane = idx;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyPortLane::DumpKeyData(stringstream &sstream)
{
    IBDIAGNET_ENTER;
    char buff[1024] = {0};
    sprintf(buff,
            U64H_FMT "," U64H_FMT "," U32D_FMT "," U32D_FMT ",",
            this->node_guid,
            this->port_guid,
            this->port_num,
            this->lane);

    sstream << buff;
    IBDIAGNET_RETURN_VOID;
}

AccRegKeyNode::AccRegKeyNode(uint64_t ng)
{
    IBDIAGNET_ENTER;
    node_guid = ng;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyNode::DumpKeyData(stringstream &sstream)
{
    IBDIAGNET_ENTER;
    char buff[1024] = {0};
    sprintf(buff,
            U64H_FMT ",",
            this->node_guid);

    sstream << buff;
    IBDIAGNET_RETURN_VOID;
}

AccRegKeyPort::AccRegKeyPort(uint64_t ng,
                             uint64_t pg,
                             uint8_t pn)
{
    IBDIAGNET_ENTER;
    node_guid = ng;
    port_guid = pg;
    port_num = pn;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyPort::DumpKeyData(stringstream &sstream)
{
    IBDIAGNET_ENTER;
    char buff[1024] = {0};
    sprintf(buff,
            U64H_FMT "," U64H_FMT "," U32D_FMT ",",
            this->node_guid,
            this->port_guid,
            this->port_num);

    sstream << buff;
    IBDIAGNET_RETURN_VOID;
}

AccRegKeyGroup::AccRegKeyGroup(uint64_t ng,
                               uint8_t gr)
{
    IBDIAGNET_ENTER;
    node_guid = ng;
    group_num = gr;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyGroup::DumpKeyData(stringstream &sstream)
{
    IBDIAGNET_ENTER;
    char buff[1024] = {0};
    sprintf(buff,
            U64H_FMT ",",
            this->node_guid);

    sstream << buff;
    IBDIAGNET_RETURN_VOID;
}

AccRegKeyNodeSensor::AccRegKeyNodeSensor(uint64_t node_guid,
                                         uint8_t sensor_id)
{
    IBDIAGNET_ENTER;
    this->node_guid = node_guid;
    this->sensor_id = sensor_id;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyNodeSensor::DumpKeyData(stringstream &sstream)
{
    IBDIAGNET_ENTER;
    char buff[1024] = {0};
    sprintf(buff,
            U64H_FMT "," U32D_FMT ",",
            this->node_guid,
            this->sensor_id);

    sstream << buff;
    IBDIAGNET_RETURN_VOID;
}

AccRegKeyTypeGroup::AccRegKeyTypeGroup(uint64_t ng,
                                       uint8_t grp_type,
                                       uint8_t grp_num,
                                       uint16_t start_idx,
                                       uint16_t indices_num)
{
    IBDIAGNET_ENTER;
    node_guid = ng;
    group_type = grp_type;
    group_num = grp_num;
    start_index = start_idx;
    num_of_indices = indices_num;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyTypeGroup::DumpKeyData(stringstream &sstream)
{
    IBDIAGNET_ENTER;
    char buff[1024] = {0};
    sprintf(buff,
            U64H_FMT ",%u,%u,%u,%u,",
            this->node_guid,
            this->group_type,
            this->group_num,
            this->start_index,
            this->num_of_indices);

    sstream << buff;
    IBDIAGNET_RETURN_VOID;
}

AccRegKeyDPN::AccRegKeyDPN(uint64_t ng,
                           uint8_t depth,
                           uint8_t pci_idx,
                           uint8_t pci_node)
{
    IBDIAGNET_ENTER;
    this->node_guid = ng;
    this->depth = depth;
    this->pci_idx = pci_idx;
    this->pci_node = pci_node;
    IBDIAGNET_RETURN_VOID;
}

void AccRegKeyDPN::DumpKeyData(stringstream &sstream)
{
    IBDIAGNET_ENTER;

    char buff[256] = {0};
    sprintf(buff,
            U64H_FMT ",%u,%u,%u,",
            this->node_guid,
            this->pci_idx,
            this->depth,
            this->pci_node);
    sstream << buff;

    IBDIAGNET_RETURN_VOID;
}

