/*
 * Copyright (c) 2004-2019 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#ifndef _ACC_REG_KEY_H_
#define _ACC_REG_KEY_H_

#include <infiniband/ibdiag/ibdiagnet/ibdiagnet_plugins_ifc.h>
#include "../internal_packets/internal_packets_layouts.h"

class AccRegKey {

public:
    AccRegKey(){}
    virtual ~AccRegKey(){}

    virtual void DumpKeyData(stringstream &sstream) = 0;

    virtual bool operator < (const AccRegKey &other) const = 0;
};


class AccRegKeyPortLane : public AccRegKey {

public:
    uint64_t node_guid;
    uint64_t port_guid;
    uint8_t port_num;
    uint8_t lane;
    uint8_t idx_in_lane;

    virtual bool operator < (const AccRegKey &other) const {
        if (node_guid < ((AccRegKeyPortLane&)other).node_guid)
            return true;
        if (node_guid == ((AccRegKeyPortLane&)other).node_guid &&
            port_guid < ((AccRegKeyPortLane&)other).port_guid)
            return true;
        if (node_guid == ((AccRegKeyPortLane&)other).node_guid &&
            port_guid == ((AccRegKeyPortLane&)other).port_guid &&
          port_num < ((AccRegKeyPortLane&)other).port_num)
            return true;
        if (node_guid == ((AccRegKeyPortLane&)other).node_guid &&
            port_guid == ((AccRegKeyPortLane&)other).port_guid &&
            port_num == ((AccRegKeyPortLane&)other).port_num &&
            lane < ((AccRegKeyPortLane&)other).lane)
            return true;
        if (node_guid == ((AccRegKeyPortLane&)other).node_guid &&
            port_guid == ((AccRegKeyPortLane&)other).port_guid &&
            port_num == ((AccRegKeyPortLane&)other).port_num &&
            lane == ((AccRegKeyPortLane&)other).lane &&
            idx_in_lane < ((AccRegKeyPortLane&)other).idx_in_lane)
            return true;

        return false;
    }

    AccRegKeyPortLane(uint64_t ng,
                      uint64_t pg,
                      uint8_t pn,
                      uint8_t ln,
                      uint8_t idx = 0);

    virtual ~AccRegKeyPortLane(){}

    virtual void DumpKeyData(stringstream &sstream);
};

class AccRegKeyNode : public AccRegKey {

public:
    uint64_t node_guid;

    virtual bool operator < (const AccRegKey &other) const {
        if (node_guid < ((AccRegKeyNode&)other).node_guid)
            return true;
        return false;
    }

    AccRegKeyNode(uint64_t ng);
    virtual ~AccRegKeyNode(){}

    virtual void DumpKeyData(stringstream &sstream);
};

class AccRegKeyPort : public AccRegKey {

public:
    uint64_t node_guid;
    uint64_t port_guid;
    uint8_t port_num;

    virtual bool operator < (const AccRegKey &other) const {
        if (node_guid < ((AccRegKeyPort&)other).node_guid)
            return true;
        if (node_guid == ((AccRegKeyPort&)other).node_guid &&
            port_guid < ((AccRegKeyPort&)other).port_guid)
            return true;
        if (node_guid == ((AccRegKeyPort&)other).node_guid &&
            port_guid == ((AccRegKeyPort&)other).port_guid &&
            port_num < ((AccRegKeyPort&)other).port_num)
            return true;
        return false;
    }
    AccRegKeyPort(uint64_t ng,
                  uint64_t pg,
                  uint8_t pn);

    virtual ~AccRegKeyPort(){}

    virtual void DumpKeyData(stringstream &sstream);
};

class AccRegKeyGroup : public AccRegKey {

public:
    uint64_t node_guid;
    uint8_t group_num;

    virtual bool operator < (const AccRegKey &other) const {
        if (node_guid < ((AccRegKeyGroup&)other).node_guid)
            return true;
        if (node_guid == ((AccRegKeyGroup&)other).node_guid &&
            group_num < ((AccRegKeyGroup&)other).group_num)
            return true;
        return false;
    }
    AccRegKeyGroup(uint64_t ng,
                   uint8_t gr);

    virtual ~AccRegKeyGroup(){}

    virtual void DumpKeyData(stringstream &sstream);
};

class AccRegKeyNodeSensor : public AccRegKey {

public:
    uint64_t node_guid;
    uint8_t sensor_id;

    virtual bool operator < (const AccRegKey &other) const {
        if (node_guid < ((AccRegKeyNodeSensor&)other).node_guid)
            return true;
        if (node_guid == ((AccRegKeyNodeSensor&)other).node_guid &&
            sensor_id < ((AccRegKeyNodeSensor&)other).sensor_id)
            return true;
        return false;
    }

    AccRegKeyNodeSensor(uint64_t node_guid, uint8_t sensor_id);
    virtual ~AccRegKeyNodeSensor(){}

    virtual void DumpKeyData(stringstream &sstream);
};


class AccRegKeyTypeGroup : public AccRegKey {

public:
    uint64_t node_guid;
    uint8_t group_type;
    uint8_t group_num;
    uint16_t start_index;
    uint16_t num_of_indices;

    virtual bool operator < (const AccRegKey &other) const {

        if (node_guid != ((AccRegKeyTypeGroup&)other).node_guid)
            return node_guid < ((AccRegKeyTypeGroup&)other).node_guid;
        if (group_type != ((AccRegKeyTypeGroup&)other).group_type)
            return group_type < ((AccRegKeyTypeGroup&)other).group_type;
        if (group_num != ((AccRegKeyTypeGroup&)other).group_num)
            return group_num < ((AccRegKeyTypeGroup&)other).group_num;
        if (start_index != ((AccRegKeyTypeGroup&)other).start_index)
            return start_index < ((AccRegKeyTypeGroup&)other).start_index;

        return false;
    }

    AccRegKeyTypeGroup(uint64_t ng,
                       uint8_t grp_type,
                       uint8_t grp_num,
                       uint16_t start_idx,
                       uint16_t indices_num);

    virtual ~AccRegKeyTypeGroup(){}

    virtual void DumpKeyData(stringstream &sstream);
};


class AccRegKeyDPN : public AccRegKey {

public:
    uint64_t node_guid;
    uint8_t depth;
    uint8_t pci_idx;
    uint8_t pci_node;


    virtual bool operator < (const AccRegKey &other) const {

        if (node_guid != ((AccRegKeyDPN&)other).node_guid)
            return node_guid < ((AccRegKeyDPN&)other).node_guid;
        if (pci_idx != ((AccRegKeyDPN&)other).pci_idx)
            return pci_idx < ((AccRegKeyDPN&)other).pci_idx;
        if (depth != ((AccRegKeyDPN&)other).depth)
            return depth < ((AccRegKeyDPN&)other).depth;
        if (pci_node != ((AccRegKeyDPN&)other).pci_node)
            return pci_node < ((AccRegKeyDPN&)other).pci_node;

        return false;
    }

    AccRegKeyDPN(uint64_t ng,
                 uint8_t depth,
                 uint8_t pci_idx,
                 uint8_t pci_node);

    virtual ~AccRegKeyDPN(){}

    virtual void DumpKeyData(stringstream &sstream);
};


#endif // _ACC_REG_KEY_H_
