/*
 * Copyright (c) 2004-2015 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#include <sstream>
#include "phy_diag_errs.h"

string ConvertAccRegStatusToStr(IN u_int8_t acc_reg_status)
{
    IBDIAGNET_ENTER;
    string result;

    switch (acc_reg_status) {
    case EN_GOOD:
        result = "Good. Operation Performed.";
        break;
    case EN_DEVICE_BUSY:
        result = "Device is busy. Can not perform the operation " \
                 "at the moment, requester should retry the operation later";
        break;
    case EN_VERSION_NOT_SUPPORTED:
        result = "Version not supported.";
        break;
    case EN_UNKNOWN_TLV:
        result = "Unknown TLV.";
        break;
    case EN_REG_UNSUPPORT:
        result = "Register not supported.";
        break;
    case EN_CLASS_UNSUPPORT:
        result = "Class not supported.";
        break;
    case EN_METHOD_UNSUPPORT:
        result = "Method not supported.";
        break;
    case EN_BAD_PARAM:
        result = "Bad parameter (e.g. port out of range, non stacking port).";
        break;
    case EN_RESOURCE_UNAVAILABLE:
        result = "Resource not available (e.g. attempt to write to a full " \
                 "FDB, allocation failed).";
        break;
    case EN_MSG_RECEIPT_ACK:
        result = "Message Receipt Acknowledgement. Will return answer later. " \
                 "Requester should rearm retransmission timer.";
        break;
    default:
        result = "UNKNOWN";
        break;
    }

    IBDIAGNET_RETURN(result);
};

FabricPortErrPhyRetrieveGeneral::FabricPortErrPhyRetrieveGeneral(
    IBPort *p_port, u_int8_t status) : FabricErrPort(p_port)
{
    IBDIAGNET_ENTER;
    ostringstream status_str;
    status_str << "0x" << std::hex << (unsigned int)status << std::dec;
    this->scope.assign(SCOPE_PORT);
    this->err_desc.assign(FER_PHY_ERR_GENERAL);
    this->description = "Failed to get phy information: status = " +
                        status_str.str() + ", ";
    this->description += ConvertAccRegStatusToStr(status);
    IBDIAGNET_RETURN_VOID;
}

FabricNodeErrPhyRetrieveGeneral::FabricNodeErrPhyRetrieveGeneral(
    IBNode *p_node, u_int8_t status) : FabricErrNode(p_node)
{
    IBDIAGNET_ENTER;
    ostringstream status_str;
    status_str << "0x" << std::hex << (unsigned int)status << std::dec;
    this->scope.assign(SCOPE_NODE);
    this->err_desc.assign(FER_PHY_ERR_GENERAL);
    this->description = "Failed to get phy information: status = " +
                        status_str.str() + ", ";
    this->description += ConvertAccRegStatusToStr(status);
    IBDIAGNET_RETURN_VOID;
}
