/*
 * Copyright (c) 2004-2019 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#ifndef _PHY_DIAG_TYPES_H_
#define _PHY_DIAG_TYPES_H_

/*****************************************************************************/
/* some defines */
#define TO_STR_(x)  #x
#define TO_STR(x)   TO_STR_(x)
#define FLT_FMT     "%.3f"

/*****************************************************************************/

enum PhyPluginSupportedNodesType {
    HCA_ONLY = 0,
    SW_ONLY,
    SUPPORT_ALL
};

bool IsPhyPluginSupportNodeType(PhyPluginSupportedNodesType supported_nodes, IBNodeType type);

#define PLUGIN_NAME "Phy Diagnostic (Plugin)"
#define PLUGIN_DESC "This plugin performs phy diagnostic."
#define NET_DUMP_EXT_FILE "ibdiagnet2.net_dump_ext"
#define BER_THRESHOLD_ERROR      (1e-8)
#define BER_THRESHOLD_WARNING    (1e-13)


/*****************************************************************************/
/* sections defines */
#define SECTION_PHY_CNTRS                        "PHY_DB1"
#define SECTION_RAW_BER                          "PHY_DB2"
#define SECTION_EFF_BER                          "EFF_BER"

#define SECTION_PCI_CNTRS                        "P_DB1"

#define DD_PHY_TYPE                              1
#define DD_PCI_TYPE                              2

/*****************************************************************************/
/* checks defines */
#define CHECK_NAME_PHY_CNTRS_CHECK                      "DD checking"
#define CHECK_NAME_PHY_CNTRS_RESET(section_name)        section_name + " reseting"
#define CHECK_NAME_PHY_CNTRS_RETRIEVE                   "DD retrieving"
#define CHECK_NAME_RETRIEVE(section_name)               section_name + " retrieving"
#define CHECK_NAME_EFF_BER                              "Effective BER Check"
#define CHECK_NAME_FW_BER                               "Effective BER Check 2"

/*****************************************************************************/
#define NO_DATA_VAL                             "NA"

// PCI macros
#define MAX_PCI_IDX         4
#define MAX_PCI_DEPTH       4
#define MAX_PCI_NODES       16
// end point
#define PCI_PORT_TYPE_EP    0
// upstream
#define PCI_PORT_TYPE_US    5
// downstream
#define PCI_PORT_TYPE_DS    6


//phy data fields in order of appearance
typedef enum {
    EN_TIME_SINCE_LAST_CLEAR = 0,
    EN_SYMBOL_ERRORS,
    EN_SYNC_HEADERS_ERRORS,
    EN_EDPL_LANE0,
    EN_EDPL_LANE1,
    EN_EDPL_LANE2,
    EN_EDPL_LANE3,
    EN_FC_FEC_CORRECTED_BLOCKS_LANE0,
    EN_FC_FEC_CORRECTED_BLOCKS_LANE1,
    EN_FC_FEC_CORRECTED_BLOCKS_LANE2,
    EN_FC_FEC_CORRECTED_BLOCKS_LANE3,
    EN_FC_FEC_UNCORRECTABLE_BLOCKS_LANE0,
    EN_FC_FEC_UNCORRECTABLE_BLOCKS_LANE1,
    EN_FC_FEC_UNCORRECTABLE_BLOCKS_LANE2,
    EN_FC_FEC_UNCORRECTABLE_BLOCKS_LANE3,
    EN_RS_FEC_CORRECTED_BLOCKS,
    EN_RS_FEC_UNCORRECTABLE_BLOCKS,
    EN_RS_FEC_NO_ERRORS_BLOCKS,
    EN_RS_FEC_SINGLE_ERROR_BLOCKS,
    EN_RS_FEC_CORRECTED_SYMBOLS_TOTAL,
/*
rs_fec_corrected_symbols_lane0_high
A8h
rs_fec_corrected_symbols_lane0_low
ACh
rs_fec_corrected_symbols_lane1_high
B0h
rs_fec_corrected_symbols_lane1_low
B4h
rs_fec_corrected_symbols_lane2_high
B8h rs_fec_corrected_symbols_lane2_low BCh
rs_fec_corrected_symbols_lane3_high
C0h
rs_fec_corrected_symbols_lane3_low
C4h
link_down_events
C8h
successful_recovery_events*/
} EnPhyDataFields_t;

typedef enum {
    VIA_SMP,
    VIA_GMP,
} AccRegVia_t;

//how many lanes to check
#define LANE_NUM                                4
#define PCI_LANE_NUM                            16
//it's currently empirically determined
#define ZERO_RAW_BER_EXPONENT                       255

/* infiniband access register defines */
#define IB_ATTR_SMP_ACCESS_REG                      0xff52
#define IB_ATTR_GMP_ACCESS_REG                      0x0051
#define ACCESS_REGISTER_ID_SLRG                     0x5028
#define ACCESS_REGISTER_ID_SLRP                     0x5026
#define ACCESS_REGISTER_ID_SLTP                     0x5027
#define ACCESS_REGISTER_ID_PTAS                     0x5029
#define ACCESS_REGISTER_ID_PPLL                     0x5030
#define ACCESS_REGISTER_ID_MPEIN                    0x9050
#define ACCESS_REGISTER_ID_MPIR                     0x9059
// fans registers
#define ACCESS_REGISTER_ID_MFCR                     0x9001
#define ACCESS_REGISTER_ID_MFSM                     0x9003
#define ACCESS_REGISTER_ID_MFSL                     0x9004
#define ACCESS_REGISTER_ID_FORE                     0x9007
#define ACCESS_REGISTER_ID_MFNR                     0x903b
// temperature registers
#define ACCESS_REGISTER_ID_MTCAP                    0x9009
#define ACCESS_REGISTER_ID_MTMP                     0x900a
#define ACCESS_REGISTER_ID_MTWE                     0x900b
// current/voltage register
#define ACCESS_REGISTER_ID_MVCAP                    0x902e
#define ACCESS_REGISTER_ID_MVCR                     0x900c
// power supplies register
#define ACCESS_REGISTER_ID_MSPS                     0x900d
// System General Information register
#define ACCESS_REGISTER_ID_MSGI                     0x9021

#define ACCESS_REGISTER_ID_SLRIP                    0x503a
#define ACCESS_REGISTER_ID_SLREG                    0x5042
#define ACCESS_REGISTER_ID_SLCCT                    0x503b
#define ACCESS_REGISTER_ID_PPAMP                    0x5025

/* EMAD fields definitions */
#define EMAD_OP_TLV_METHOD_QUERY                    0x1
//EMAD Class Types
#define EMAD_CLASS_ACC_REG                          0x1
//EMAD TLV Types
#define EMAD_TLV_TYPE_OPERATION                     0x1
#define EMAD_TLV_TYPE_REG                           0x3
//EMAD TLV Types length in dword (4 bytes)
#define EMAD_TLV_TYPE_OPERATION_LEN                 4

//SMP Access register types
#define ACC_REG_PNAT_IB_PORT                        0x1
#define ACC_REG_PNAT_OOB_PORT                       0x3

#define UNUSED_FIELDS_NUM                           -1

//SLRG
#define ACC_REG_SLRG_INTERNAL_SECTION_NAME          "PHY_DB3"
#define ACC_REG_SLRG_PCI_INTERNAL_SECTION_NAME      "P_DB3"
#define ACC_REG_SLRG_EXTERNAL_SECTION_NAME          "PHY_PORT_GRADE"
#define ACC_REG_SLRG28NM_FIELDS_NUM                 19
#define ACC_REG_SLRG16NM_FIELDS_NUM                 11

//SLTP
#define ACC_REG_SLTP_INTERNAL_SECTION_NAME          "PHY_DB5"
#define ACC_REG_SLTP_PCI_INTERNAL_SECTION_NAME      "P_DB5"
#define ACC_REG_SLTP_FIELDS_NUM                     9

//SLRP
#define ACC_REG_SLRP_INTERNAL_SECTION_NAME          "PHY_DB4"
#define ACC_REG_SLRP_PCI_INTERNAL_SECTION_NAME      "P_DB4"
#define ACC_REG_SLRP28NM_FIELDS_NUM                 38
#define ACC_REG_SLRP16NM_FIELDS_NUM                 48

//PTAS
#define ACC_REG_PTAS_INTERNAL_SECTION_NAME          "PHY_DB6"
#define ACC_REG_PTAS_FIELDS_NUM                     23

//PPLL
#define ACC_REG_PPLL_INTERNAL_SECTION_NAME          "PHY_DB7"
#define ACC_REG_PPLL28NM_FIELDS_NUM                 41
#define ACC_REG_PPLL16NM_FIELDS_NUM                 68

//MPEIN
#define ACC_REG_MPEIN_INTERNAL_SECTION_NAME          "P_DB8"
#define ACC_REG_MPEIN_FIELDS_NUM                     15

//MPIR
#define ACC_REG_MPIR_INTERNAL_SECTION_NAME          "P_DB2"
#define ACC_REG_MPIR_FIELDS_NUM                     10


// PDDR (DD)
#define SECTION_TROUBLESHOOTING_INFO                "PHY_DB9"
#define SECTION_OPERATION_INFO                      "PHY_DB10"
#define SECTION_PHY_INFO                            "PHY_DB11"
#define SECTION_MODULE_INFO                         "PHY_DB12"
#define SECTION_LINK_DOWN_INFO                      "PHY_DB13"

// PPCNT (DD)
#define SECTION_RS_FEC_HISTOGRAM                    "PHY_DB14"
#define SECTION_PLR_COUNTERS                        "PHY_DB15"
#define SECTION_PHY_LAYER_STATISTICS                "PHY_DB16"
#define SECTION_ZL_FEC_COUNTERS                     "PHY_DB17"

#define ACC_REG_SLRIP_INTERNAL_SECTION_NAME         "PHY_DB18"
#define ACC_REG_SLRIP_FIELDS_NUM                    25

#define ACC_REG_SLREG_INTERNAL_SECTION_NAME         "PHY_DB19"
#define ACC_REG_SLREG_FIELDS_NUM                    27

#define ACC_REG_SLCCT_INTERNAL_SECTION_NAME         "PHY_DB20"
#define ACC_REG_SLCCT_FIELDS_NUM                    37

#define ACC_REG_PPAMP_INTERNAL_SECTION_NAME         "PHY_DB21"
#define ACC_REG_PPAMP_FIELDS_NUM                    18


// MFCR
#define ACC_REG_MFCR_INTERNAL_SECTION_NAME          "AVAILABLE_FANS"
// MFSM
#define ACC_REG_MFSM_INTERNAL_SECTION_NAME          "FANS_SPEED"
// MFSL
#define ACC_REG_MFSL_INTERNAL_SECTION_NAME          "FANS_THRESHOLDS"
// FORE
#define ACC_REG_FORE_INTERNAL_SECTION_NAME          "FANS_ALERT"
// MFNR
#define ACC_REG_MFNR_INTERNAL_SECTION_NAME          "FANS_SERIAL_NUMBER"
// MTCAP
#define ACC_REG_MTCAP_INTERNAL_SECTION_NAME         "NUM_OF_TEMP_SENSORS"
// MTMP
#define ACC_REG_MTMP_INTERNAL_SECTION_NAME          "TEMPERATURE_SENSORS"
// MTWE
#define ACC_REG_MTWE_INTERNAL_SECTION_NAME          "TEMPERATURE_SENSORS_ALERT"
// MVCAP
#define ACC_REG_MVCAP_INTERNAL_SECTION_NAME         "AVAILABLE_POWER_SENSORS"
// MVCR
#define ACC_REG_MVCR_INTERNAL_SECTION_NAME          "POWER_SENSORS"
// MSPS
#define ACC_REG_MSPS_INTERNAL_SECTION_NAME          "POWER_SUPPLIES"
// MSGI
#define ACC_REG_MSGI_INTERNAL_SECTION_NAME          "SYSTEM_GENERAL_INFORMATION"

#define PORT_LANE_HEADER                            "NodeGuid,PortGuid,PortNum,Lane"
#define NODE_HEADER                                 "NodeGuid"
#define PORT_HEADER                                 "NodeGuid,PortGuid,PortNum"
#define GROUP_HEADER                                "NodeGuid"
#define NODE_SENSOR_HEADER                          "NodeGuid,SensorIndex"
#define TYPE_GROUP_HEADER                           "NodeGuid,Type,Group,Index,NumOfIndices"
#define PCI_DPN_HEADER                              "NodeGuid,PCIIndex,Depth,PCINode"

// fans
#define AVAILABLE_FANS                              ",AvailableFans"
#define FANS_SPEED_HEADER                           ",FanSpeed"
#define FANS_THRESHOLDS_HEADER                      ",MinSpeed,MaxSpeed"
#define FANS_ALERT_HEADER                           ",FansUnderLimit,FansOverLimit"
#define FANS_NUMBERS_HEADER                         ",SerialNumber"
// temperature
#define TEMPERATURE_SENSORS_COUNT                   ",TemperatureSensorsCount"
#define TEMPERATURE_SENSORS_HEADER                  ",SensorName,Temperature,MaxTemperature" \
                                                    ",LowThreshold,HighThreshold"
#define TEMPERATURE_SENSORS_ALERT_HEADER            ",SensorsOverThreshold"
// power sensors
#define AVAILABLE_POWER_SENSORS                     ",AvailablePowerSensors"
#define POWER_SENSORS_HEADER                        ",SensorName,Voltage,Current"
// power supplies
#define POWER_SUPPLIES_HEADER                       ",PSUIndex,IsPresent,IsFRU,ACInput,DCState" \
                                                    ",AlertState,FanState,TemperatureState,SerialNumber"
// System General Information
#define SYSTEM_GENERAL_INFORMATION_HEADER           ",SerialNumber,PartNumber,Revision,ProductName"


#define MAX_FANS                                    10
#define MAX_VOLTAGE_CURRENT_SENSORS                 64

#define SLRIP_IB_SEL_NUM                            4
#define SLREG_EIDX_NUM                              3
#define SLCCT_CYCLES_NUM                            2
#define SLCCT_CTLE_ARR_SIZE                         8

#define MAD_STATUS_UNSUP_REGISTER_GMP               (5 << 2)

#define NOT_SUPPORT_DIAGNOSTIC_DATA                 (0x1 << 0)
#define NOT_SUPPORT_DIAGNOSTIC_DATA_PHYS_CNTRS      (0x1 << 1)
#define NOT_SUPPORT_DIAGNOSTIC_DATA_PCIE_CNTRS     (0x1 << 2)

#define NOT_SUPPORT_SMP_ACCESS_REGISTER             (0x1 << 2)
#define NOT_SUPPORT_SLRG                            (0x1 << 3)
#define NOT_SUPPORT_SLRP                            (0x1 << 4)
#define NOT_SUPPORT_SLTP                            (0x1 << 5)
#define NOT_SUPPORT_PTAS                            (0x1 << 6)
#define NOT_SUPPORT_PPLL                            (0x1 << 7)
#define NOT_SUPPORT_MPEIN                           (0x1 << 8)

#define NOT_SUPPORT_MFCR                            (0x1 << 9)
#define NOT_SUPPORT_MFSM                            (0x1 << 10)
#define NOT_SUPPORT_MFSL                            (0x1 << 11)
#define NOT_SUPPORT_FORE                            (0x1 << 12)
#define NOT_SUPPORT_MTCAP                           (0x1 << 13)
#define NOT_SUPPORT_MTMP                            (0x1 << 14)
#define NOT_SUPPORT_MTWE                            (0x1 << 15)
#define NOT_SUPPORT_MVCAP                           (0x1 << 16)
#define NOT_SUPPORT_MVCR                            (0x1 << 17)
#define NOT_SUPPORT_MSPS                            (0x1 << 18)
#define NOT_SUPPORT_GMP_ACCESS_REGISTER             (0x1 << 19)
#define NOT_SUPPORT_MSGI                            (0x1 << 20)
#define NOT_SUPPORT_DD_TROUBLESHOOTING_INFO         (0x1 << 21)
#define NOT_SUPPORT_DD_OPERATION_INFO               (0x1 << 22)
#define NOT_SUPPORT_DD_PHY_INFO                     (0x1 << 23)
#define NOT_SUPPORT_DD_MODULE_INFO                  (0x1 << 24)
#define NOT_SUPPORT_DD_LINK_DOWN_INFO               (0x1 << 25)
#define NOT_SUPPORT_MFNR                            (0x1 << 26)
// PPCNT
#define NOT_SUPPORT_DD_RS_FEC_HISTOGRAM             (0x1 << 27)
#define NOT_SUPPORT_DD_PLR_COUNTERS                 (0x1 << 28)
#define NOT_SUPPORT_DD_PHY_LAYER_STATISTICS         (0x1 << 29)
#define NOT_SUPPORT_DD_ZL_FEC_COUNTERS              (0x1 << 30)

#define NOT_SUPPORT_SLRIP                           (1lu << 31)
#define NOT_SUPPORT_SLREG                           (1lu << 32)
#define NOT_SUPPORT_SLCCT                           (1lu << 33)
#define NOT_SUPPORT_PPAMP                           (1lu << 34)

#define NOT_SUPPORT_MPIR                            (1lu << 35)

#define PPAMP_IDX_DATA_SIZE                         16
#define MAX_OPAMP_GROUP_TYPE                        3

/*****************************************************************************/

/* options defines */
#define OPTION_PHY_INFO_GET "get_phy_info"
#define OPTION_PHY_INFO_GET_SHORT ' '
#define OPTION_PHY_INFO_GET_ARG_DESC ""
#define OPTION_PHY_INFO_GET_DESC "Indicates to query all ports for phy "  \
    "information."

#define OPTION_PPAMP_GET "get_ppamp"
#define OPTION_PPAMP_GET_SHORT ' '
#define OPTION_PPAMP_GET_ARG_DESC ""
#define OPTION_PPAMP_GET_DESC "Indicates to query all ports for PPAMP "  \
    "(Port Phy opAMP data), works with --get_phy_info."

#define OPTION_SHOW_CAP_REG "show_cap_reg"
#define OPTION_SHOW_CAP_REG_SHORT ' '
#define OPTION_SHOW_CAP_REG_ARG_DESC ""
#define OPTION_SHOW_CAP_REG_DESC "Indicates to dump also capability registers (hidden "  \
    "by default), works with --get_phy_info."

#define OPTION_PHY_INFO_RESET "reset_phy_info"
#define OPTION_PHY_INFO_RESET_SHORT ' '
#define OPTION_PHY_INFO_RESET_ARG_DESC ""
#define OPTION_PHY_INFO_RESET_DESC "Indicates to query and clear all ports phy information."

#define OPTION_PCI_INFO_GET "get_p_info"
#define OPTION_PCI_INFO_GET_SHORT ' '
#define OPTION_PCI_INFO_GET_ARG_DESC ""
#define OPTION_PCI_INFO_GET_DESC "Indicates to query all ports for PCI information."

#define OPTION_PCI_INFO_RESET "reset_p_info"
#define OPTION_PCI_INFO_RESET_SHORT ' '
#define OPTION_PCI_INFO_RESET_ARG_DESC ""
#define OPTION_PCI_INFO_RESET_DESC "Indicates to query and clear all ports PCI information."

#define OPTION_BER_THRESHOLD_ERROR "ber_thresh_error"
#define OPTION_BER_THRESHOLD_ERROR_SHORT ' '
#define OPTION_BER_THRESHOLD_ERROR_ARG_DESC "<thresh>"
#define OPTION_BER_THRESHOLD_ERROR_DESC "set threshold for ber errors (default=1e-8)"

#define OPTION_BER_THRESHOLD_WARNING "ber_thresh_warning"
#define OPTION_BER_THRESHOLD_WARNING_SHORT ' '
#define OPTION_BER_THRESHOLD_WARNING_ARG_DESC "<thresh>"
#define OPTION_BER_THRESHOLD_WARNING_DESC "set threshold for ber warnings (default=1e-13)"

#endif //_PHY_DIAG_TYPES_H_
