/*
 * Copyright (c) 2004-2010 Mellanox Technologies LTD. All rights reserved.
 *
 * This software is available to you under the terms of the
 * OpenIB.org BSD license included below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */


%{
    // #define DEBUG 1
    #pragma warning(disable:4242)
    #include <stdio.h>
    #include <stdlib.h>
    #include <string.h>
    
    #include <infiniband/ibdm/Fabric.h>
    #include <infiniband/ibdm/SysDef.h>

    #include "generated_code/ibnl_parser.hh"

    /*
     * inorder to compile in windows
     */
    #define YY_NEVER_INTERACTIVE 1

    extern long lineNum;
%}


%option yylineno
%option noyywrap
%option nounput
%option noinput
%s name

%%
([1-9][0-9]*|0) {
    ibnl_lval.ival = atoi(yytext);
#ifdef DEBUG
    printf("INT_NUM:%d\n", ibnl_lval.ival);
#endif
    return INT_NUM;
}

SYSTEM {
    ibnl_lval.ival = SYSTEM;
#ifdef DEBUG
    printf("SYSTEM\n");
#endif
    BEGIN(name);
    return SYSTEM;
}

TOPSYSTEM {
    ibnl_lval.ival = TOPSYSTEM;
#ifdef DEBUG
    printf("TOPSYSTEM\n");
#endif
    BEGIN(name);
    return TOPSYSTEM;
}

NODE {
    ibnl_lval.ival = NODE;
#ifdef DEBUG
    printf("NODE\n");
#endif
    return NODE;
}

1x|4x|8x|12x {
    ibnl_lval.sval = (char *)malloc(strlen(yytext) + 1);
    strcpy(ibnl_lval.sval, yytext);
#ifdef DEBUG
    printf("WIDTH:%s\n", ibnl_lval.sval);
#endif
    return WIDTH;
}

[-] {
    ibnl_lval.sval = NULL;
#ifdef DEBUG
    printf("-");
#endif
    BEGIN(INITIAL);
    return DASH;
}

[>] {
    ibnl_lval.sval = NULL;
#ifdef DEBUG
    printf(">");
#endif
    BEGIN(name);
    return END_DASH;
}

2.5G|5G|10G|14G|FDR10G|25G|EDR20G|50G {
    ibnl_lval.sval = (char *)malloc(strlen(yytext));
    strncpy(ibnl_lval.sval, yytext, strlen(yytext) - 1);
    ibnl_lval.sval[strlen(yytext)-1]='\0';
#ifdef DEBUG
    printf("SPEED:%s\n", ibnl_lval.sval);
#endif
    return SPEED;
}

SUBSYSTEM {
    ibnl_lval.ival = SUBSYSTEM;
#ifdef DEBUG
    printf("SUBSYSTEM\n");
#endif
    return SUBSYSTEM;
}

CFG: {
    ibnl_lval.ival = CFG;
#ifdef DEBUG
    printf("CFG\n");
#endif
    return CFG;
}

(SW|CA|HCA|RTR) {
    if (!strcmp(yytext,"SW")) {
        ibnl_lval.tval = IB_SW_NODE;
    } else if (!strcmp(yytext,"CA") || !strcmp(yytext,"HCA")) {
        ibnl_lval.tval = IB_CA_NODE;
    } else {
        ibnl_lval.tval = IB_RTR_NODE;
    }
#ifdef DEBUG
    printf("%s\n", yytext);
#endif
    return NODETYPE;
}

<name>[A-Za-z][-\[\]\\\*/A-Za-z0-9_.:%@~]+ {
    ibnl_lval.sval = (char *)malloc(strlen(yytext) + 1);
    strcpy(ibnl_lval.sval, yytext);
#ifdef DEBUG
    printf("NAME:%s\n", ibnl_lval.sval);
#endif
    return (NAME);
}

\n {
    lineNum++;
#ifdef DEBUG
    printf("LINE\n");
#endif
    if(lineNum % 10000==0)
        fprintf(stderr,"-I- Parsed %ld lines\r", lineNum);
    ibnl_lval.ival = LINE;
    return(LINE);
}

[ \t]+ {}

. {
#ifdef DEBUG
    printf("CHAR:%c\n",yytext[0]);
#endif
    return(yytext[0]);
}

%%

