/*
 *  Copyright (c) 2004-2013 Mellanox Technologies LTD. All rights reserved
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */


#include <string.h>

#include "ibis.h"


/****************************************************/

/* some new types */
typedef enum link_type {IB, ETH, VPI, FLASH_RECOVERY} link_type_t;

typedef struct dev_id {
    string opn;
    u_int16_t id;
    u_int16_t rev;
    string desc;
    link_type_t type;
} dev_id_t;

/* globals */
//switch devices
dev_id_t anafa_devices[] = {
        {"Unknown", 43132/*a87ch*/, 0x00, "Infiniscale Gen 1", IB},
        {"Unknown", 47396/*b924h*/, 0x00, "Infiniscale Gen 2", IB}
};
dev_id_t shaldag_devices[] = {  //Infiniscale Gen 4
        {"MT48436A0-FCC-Q", 48438/*bd36h*/, 0x00, "IB QDR", IB},
        {"MT48436A0-FCC-D", 48437/*bd35h*/, 0x00, "IB DDR", IB},
        {"MT48436A0-FCC-S", 48436/*bd34h*/, 0x00, "IB SDR", IB},
        {"V2036",           23120/*5A50h*/, 0x00, "IB DDR", IB},
        {"V4036",           23130/*5A5Ah*/, 0x00, "IB QDR", IB},
        {"sLB-4018",        23131/*5A5Bh*/, 0x00, "IB QDR", IB},
        {"sFB-4700",        23132/*5A5Ch*/, 0x00, "IB QDR", IB},
        {"sFB-4700X2",      23133/*5A5Dh*/, 0x00, "IB QDR", IB},
        {"sFB-4200",        23136/*5A60h*/, 0x00, "IB QDR", IB},
        {"V4036E",          23138/*5A62h*/, 0x00, "IB QDR", IB},
        {"IBM HSSM",        23134/*5A5Eh*/, 0x00, "IB QDR", IB},
        {"MT48436A0",       436/*1b3h*/,    0x00, "Flash recovery mode", FLASH_RECOVERY}
};
dev_id_t switchX_devices[] = {
        {"MT58100A0",       51000/*c738h*/, 0x00, "SwitchX - 36 40G/64 10Gports infiniband and Ethernet Layer 2/3+ Switch", VPI},
        {"MT58100A0",       581/*245h*/,    0x00, "Flash recovery mode", FLASH_RECOVERY},
        {"BL_12000964",       6914/*1b02h*/, 0x00, "FSB_IB_Switch : Bullx Chassis FSB 36-Port QSFP FDR SwitchX Infiniband Switch", VPI},
        {"BL_12000822",       6992/*1b50h*/, 0x00, "FSB_IB_Switch : Bullx B700 Chassis FSB 36-Port QSFP FDR SwitchX Infiniband Switch", VPI}
};

dev_id_t pelican_devices[] = {
        {"MT52000",       52000/*0xcb20*/, 0x00, "SwitchIB - 36 100G/64 25Gports infiniband Switch", IB},
};

//bridge devices
dev_id_t bridgeX_devices[] = {
        {"MT1016A0-FCC",        1016/*3f8h*/,   0xa0, "EN PHY with FC upgrade capability", ETH},    //TODO - check FC capability
        {"MT68102A0-FCC-EEF",   64122/*fa7a*/,  0xa0, "EN up, E/FC down", ETH},                     //TODO - check E/FC down
        {"MT68102A0-FCC-VEF",   64102/*fa66h*/, 0xa0, "VPI up, E/FC down", VPI},                    //TODO - check E/FC down
        {"MT68102A0",           102/*66h*/,     0xa0, "Flash recovery mode", FLASH_RECOVERY}
};

//hca devices
dev_id_t tavor_devices[] = {
        {"MT23108A1-LBC", 23110/*5a46h*/, 0xa1, "MT23108 PCI Bridge", IB},
        {"MT23108A1-LBC", 23109/*5a45h*/, 0xa1, "MT23108 [Infinihost HCA Flash Recovery]", FLASH_RECOVERY},
        {"MT23108A1-LBC", 23108/*5a44h*/, 0xa1, "MT23108 InfiniHost", IB},
        {"MT23108A0-LBC", 23110/*5a46h*/, 0xa0, "MT23108 PCI Bridge", IB},
        {"MT23108A0-LBC", 23109/*5a45h*/, 0xa0, "MT23108 [Infinihost HCA Flash Recovery]", FLASH_RECOVERY},
        {"MT23108A0-LBC", 23108/*5a44h*/, 0xa0, "MT23108 InfiniHost", IB}
};
dev_id_t sinai_devices[] = {
        {"MT25204A0-FCC-D", 24205/*5e8dh*/, 0xa0, "MT25204 [InfiniHost III Lx HCA Flash Recovery]", FLASH_RECOVERY},
        {"MT25204A0-FCC-D", 25204/*6274h*/, 0x20, "MT25204 [InfiniHost III Lx HCA]", IB},
        {"MT25204A0-FCC",   24205/*5e8dh*/, 0xa0, "MT25204 [InfiniHost III Lx HCA Flash Recovery]", FLASH_RECOVERY},
        {"MT25204A0-FCC",   25204/*6274h*/, 0xa0, "MT25204 [InfiniHost III Lx HCA]", IB}
};
dev_id_t arbel_devices[] = {
        {"MT25208A0-FCC-D", 25209/*6279h*/, 0xa0, "MT25208 [InfiniHost III Ex HCA Flash Recovery]", FLASH_RECOVERY},
        {"MT25208A0-FCC-D", 25218/*6282h*/, 0x20, "MT25208 [InfiniHost III Ex]", IB},
        {"MT25208A0-FCC-D", 25208/*6278h*/, 0x20, "MT25208 InfiniHost III Ex (Tavor compatibility mode)", IB},
        {"MT25208A0-FCC",   25209/*6279h*/, 0xa0, "MT25208 [InfiniHost III Ex HCA Flash Recovery]", FLASH_RECOVERY},
        {"MT25208A0-FCC",   25218/*6282h*/, 0xa0, "MT25208 [InfiniHost III Ex]", IB},
        {"MT25208A0-FCC",   25208/*6278h*/, 0xa0, "MT25208 InfiniHost III Ex (Tavor compatibility mode)", IB}
};
dev_id_t connectX_devices[] = {
        {"MT25408A0-FCC-TE",    26448/*6750h*/, 0xa0, "MT26448 [ConnectX EN 10GigE, PCIe 2.0 5GT/s]", ETH},
        {"MT25408A0-FCC-SE",    25448/*6368h*/, 0xa0, "MT25448 [ConnectX EN 10GigE, PCIe 2.0 2.5GT/s]", ETH},
        {"MT25408A0-FCC-QI",    26428/*673ch*/, 0xa0, "MMT26428 [ConnectX VPI PCIe 2.0 5GT/s - IB QDR / 10GigE]", VPI},
        {"MT25408A0-FCC-GI",    26418/*6732h*/, 0xa0, "MT26418 [ConnectX VPI PCIe 2.0 5GT/s - IB DDR / 10GigE]", VPI},
        {"MT25408A0-FCC-DI",    25418/*634ah*/, 0xa0, "MT25418 [ConnectX VPI PCIe 2.0 2.5GT/s - IB DDR / 10GigE]", VPI},
        {"MT25408A0-FCC-SI",    25408/*6340h*/, 0xa0, "MT25408 [ConnectX VPI - IB SDR / 10GigE]", VPI},
        {"MT25408",             401/*191h*/,    0xa0, "MT25408 [ConnectX IB SDR Flash Recovery]", FLASH_RECOVERY}
};
dev_id_t connectX_2_devices[] = {
        {"MT25408B0-FCC-QE",    26478/*676eh*/, 0xb0, "MT26478 [ConnectX EN 10GigE, PCIe 2.0 5GT/s]", ETH},
        {"MT25408B0-FCC-TE",    26448/*6750h*/, 0xb0, "MT26448 [ConnectX EN 10GigE, PCIe 2.0 5GT/s]", ETH},
        {"MT25408B0-FCC-SE",    25448/*6368h*/, 0xb0, "MT25448 [ConnectX EN 10GigE, PCIe 2.0 2.5GT/s]", ETH},
        {"MT25408B0-FCC-QI",    26438/*6746h*/, 0xb0, "MT26488 [ConnectX-2 VPI w/ Virtualization+]", VPI},
        {"MT25408B0-FCC-QI",    26428/*673ch*/, 0xb0, "MT26428 [ConnectX VPI PCIe 2.0 5GT/s - IB QDR / 10GigE]", VPI},
        {"MT25408B0-FCC-GI",    26418/*6732h*/, 0xb0, "MT26418 [ConnectX VPI PCIe 2.0 5GT/s - IB DDR / 10GigE]", VPI},
        {"MT25408B0-FCC-DI",    25418/*634ah*/, 0xb0, "MT25418 [ConnectX VPI PCIe 2.0 2.5GT/s - IB DDR / 10GigE]", VPI},
        {"MT25408B0-FCC-SI",    25408/*6340h*/, 0xb0, "MT25408 [ConnectX VPI - IB SDR / 10GigE]", VPI},
        {"MT25408",             401/*191h*/,    0xb0, "MT25408 [ConnectX IB SDR Flash Recovery]", FLASH_RECOVERY}
};
dev_id_t connectX_2_ENt_devices[] = {
        {"MT25458B0-FCC-TE", 26458/*675ah*/, 0xb0, "MT26458 [ConnectX EN 10GigE 10GBaseT, PCIe Gen2 5GT/s]", ETH},
        {"MT25458B0-FCC-SE", 25458/*6372h*/, 0xb0, "MT25458 [ConnectX EN 10GigE 10GBaseT, PCIe 2.0 2.5GT/s]", ETH},
        {"MT25408",          401/*191h*/,    0xb0, "MT25408 [ConnectX IB SDR Flash Recovery]", FLASH_RECOVERY}
};
dev_id_t connectX_2_Lx_devices[] = {
        {"MT25414B0-FCC-QES",   26478/*676eh*/, 0xb0, "MT26478 [ConnectX EN 10GigE, PCIe 2.0 5GT/s]", ETH},
        {"MT25414B0-FCC-TES",   26448/*6750h*/, 0xb0, "MT26448 [ConnectX EN 10GigE, PCIe 2.0 5GT/s]", ETH},
        {"MT25414B0-FCC-QIS",   26438/*6746h*/, 0xb0, "MT26488 [ConnectX-2 VPI w/ Virtualization+]", VPI},
        {"MT25414B0-FCC-QIS",   26428/*673ch*/, 0xb0, "MT26428 [ConnectX VPI PCIe 2.0 5GT/s - IB QDR / 10GigE]", VPI},
        {"MT25414B0-FCC-GIS",   26418/*6732h*/, 0xb0, "MT26418 [ConnectX VPI PCIe 2.0 5GT/s - IB DDR / 10GigE]", VPI},
        {"MT25414",             401/*191h*/,    0xb0, "MT25408 [ConnectX IB SDR Flash Recovery]", FLASH_RECOVERY}
};
dev_id_t connectX_3_devices[] = {
        {"MT2750xA0", 4099/*1003h*/,    0xa0, "Ethernet class code", ETH},
        {"MT2750xA0", 4099/*1003h*/,    0xa0, "MT27500 [ConnectX-3]", VPI},
        {"MT27508A0", 501/*1f5h*/,      0xa0, "MT27500 [ConnectX-3 Flash Recovery]", FLASH_RECOVERY},
        {"BL_12000984CX3", 6963/*1b33h*/,      0xa0, "SCBF_ConnectX3 : Bullx B510 SCBF ConnectX3 FDR 1-Port Onboard Infiniband HCA", VPI},
        {"BL_12000984CX3F1", 6977/*1b41h*/,    0xa0, "CSP_ConnectX3F1 : bullx B710 DLC CSP ConnectX3 FDR 1-Port Onboard Infiniband HCA (SL)", VPI},
        {"BL_12000984CX3??", 6979/*1b43h*/,    0xa0, "CSP_ConnectX3?? : bullx B710 DLC CSP ConnectX3 FDR 1-Port Onboard Infiniband HCA (SH)", VPI},
        {"BL_12000989CX3", 7027/*1b73h*/,      0xa0, "GPGPU ConnectX3 FDR 1-Port Onboard Infiniband HCA", VPI}
};
dev_id_t golan_devices[] = {
        {"MT2760xA0", 4113/*1011h*/,    0xa0, "Ethernet class code", ETH},
        {"MT2760xA0", 4113/*1011h*/,    0xa0, "MT27600 [Connect-IB]", VPI},
        {"MT27608A0", 511/*1ffh*/,      0xa0, "MT27600 [Connect-IB Flash Recovery]", FLASH_RECOVERY}
};


/****************************************************/
/******************private methods*******************/
/****************************************************/
bool Ibis::IsSupportIB(void *type)
{
    IBIS_ENTER;
    if ( (*(link_type_t *)type == IB) || (*(link_type_t *)type == VPI) ) {
        IBIS_RETURN(true);
    }
    IBIS_RETURN(false);
}


bool Ibis::IsIBDevice(void *arr, unsigned arr_size, u_int16_t dev_id)
{
    IBIS_ENTER;
    for (unsigned i = 0; i < arr_size; ++i) {
        if ( this->IsSupportIB(&(((dev_id_t *)arr)[i].type)) &&
                (((dev_id_t *)arr)[i].id == dev_id)) {
            IBIS_RETURN(true);
        }
    }
    IBIS_RETURN(false);
}


/****************************************************/
/*******************public methods*******************/
/****************************************************/
bool Ibis::IsVenMellanox(u_int32_t vendor_id) {
    return (vendor_id == MELLANOX_VEN_ID);
}


//switch
bool Ibis::IsDevAnafa(u_int16_t dev_id) {
    return (this->IsIBDevice(anafa_devices, arrsize(anafa_devices), dev_id));
}

bool Ibis::IsDevShaldag(u_int16_t dev_id) {
    return (this->IsIBDevice(shaldag_devices, arrsize(shaldag_devices), dev_id));
}

bool Ibis::IsDevPelican(u_int16_t dev_id) {
    return (this->IsIBDevice(pelican_devices, arrsize(pelican_devices), dev_id));
}

bool Ibis::IsDevSwitchXIB(u_int16_t dev_id) {
    return (this->IsIBDevice(switchX_devices, arrsize(switchX_devices), dev_id));
}

void Ibis::GetSwitchXIBDevIds(device_id_list_t& mlnx_dev_ids_list,
                              device_id_list_t& bull_dev_ids_list) {
    for (unsigned int i = 0; i < arrsize(switchX_devices); i++) {
        if (switchX_devices[i].opn.at(0) == 'M')
            mlnx_dev_ids_list.push_back(switchX_devices[i].id);
        else if (switchX_devices[i].opn.at(0) == 'B')
            bull_dev_ids_list.push_back(switchX_devices[i].id);
    }
}

void Ibis::GetAnafaDevIds(device_id_list_t& dev_ids_list)
{
    for (unsigned int i = 0; i < arrsize(anafa_devices); i++)
        dev_ids_list.push_back(anafa_devices[i].id);
}

void Ibis::GetShaldagDevIds(device_id_list_t& mlnx_dev_ids_list,
                            device_id_list_t& volt_dev_ids_list)
{
    for (unsigned int i = 0; i < arrsize(shaldag_devices); i++) {
        //mellanox device ids, their opn starts with M
        if (shaldag_devices[i].opn.at(0) == 'M')
            mlnx_dev_ids_list.push_back(shaldag_devices[i].id);
        //voltaire device id, including IBM HSSM, that has voltaire vendor id
        else
            volt_dev_ids_list.push_back(shaldag_devices[i].id);
    }
}

//bridge
bool Ibis::IsDevBridgeXIB(u_int16_t dev_id) {
    return (this->IsIBDevice(bridgeX_devices, arrsize(bridgeX_devices), dev_id));
}

void Ibis::GetBridgeXIBDevIds(device_id_list_t& dev_ids_list)
{
    for (unsigned int i = 0; i < arrsize(bridgeX_devices); i++)
        dev_ids_list.push_back(bridgeX_devices[i].id);
}

//hca
bool Ibis::IsDevTavor(u_int16_t dev_id) {
    return (this->IsIBDevice(tavor_devices, arrsize(tavor_devices), dev_id));
}

bool Ibis::IsDevSinai(u_int16_t dev_id) {
    return (this->IsIBDevice(sinai_devices, arrsize(sinai_devices), dev_id));
}

bool Ibis::IsDevArbel(u_int16_t dev_id) {
    return (this->IsIBDevice(arbel_devices, arrsize(arbel_devices), dev_id));
}

bool Ibis::IsDevConnectX_1IB(u_int16_t dev_id) {
    return (this->IsIBDevice(connectX_devices, arrsize(connectX_devices), dev_id));
}
bool Ibis::IsDevConnectX_2IB(u_int16_t dev_id) {
    return (this->IsIBDevice(connectX_2_devices, arrsize(connectX_2_devices), dev_id) ||
            //this->IsIBDevice(connectX_2_ENt_devices, arrsize(connectX_2_ENt_devices), dev_id) || --only ETH
            this->IsIBDevice(connectX_2_Lx_devices, arrsize(connectX_2_Lx_devices), dev_id));
}
bool Ibis::IsDevConnectX_3IB(u_int16_t dev_id) {
    return (this->IsIBDevice(connectX_3_devices, arrsize(connectX_3_devices), dev_id));
}
bool Ibis::IsDevConnectXIB(u_int16_t dev_id) {
    return (this->IsDevConnectX_1IB(dev_id) ||  \
            this->IsDevConnectX_2IB(dev_id) ||  \
            this->IsDevConnectX_3IB(dev_id));
}
bool Ibis::IsDevGolan(u_int16_t dev_id) {
    return (this->IsIBDevice(golan_devices, arrsize(golan_devices), dev_id));
}

void Ibis::GetConnectX_3IBDevIds(device_id_list_t& mlnx_dev_ids_list,
                                 device_id_list_t& bull_dev_ids_list) {
    for (unsigned int i = 0; i < arrsize(connectX_3_devices); i++) {
        if (connectX_3_devices[i].opn.at(0) == 'M')
            mlnx_dev_ids_list.push_back(connectX_3_devices[i].id);
        else if (connectX_3_devices[i].opn.at(0) == 'B')
            bull_dev_ids_list.push_back(connectX_3_devices[i].id);
    }
}

void Ibis::GetGolanDevIds(device_id_list_t& dev_ids_list) {
    for (unsigned int i = 0; i < arrsize(golan_devices); i++)
        dev_ids_list.push_back(golan_devices[i].id);
}

void Ibis::GetTavorDevIds(device_id_list_t& dev_ids_list)
{
    for (unsigned int i = 0; i < arrsize(tavor_devices); i++)
        dev_ids_list.push_back(tavor_devices[i].id);
}

void Ibis::GetSinaiDevIds(device_id_list_t& dev_ids_list)
{
    for (unsigned int i = 0; i < arrsize(sinai_devices); i++)
        dev_ids_list.push_back(sinai_devices[i].id);
}

void Ibis::GetArbelDevIds(device_id_list_t& dev_ids_list)
{
    for (unsigned int i = 0; i < arrsize(arbel_devices); i++)
        dev_ids_list.push_back(arbel_devices[i].id);
}

void Ibis::GetConnectXDevIds(device_id_list_t& dev_ids_list)
{
    for (unsigned int i = 0; i < arrsize(connectX_devices); i++)
        dev_ids_list.push_back(connectX_devices[i].id);
}

void Ibis::GetConnectX_2DevIds(device_id_list_t& dev_ids_list)
{
    for (unsigned int i = 0; i < arrsize(connectX_2_devices); i++)
        dev_ids_list.push_back(connectX_2_devices[i].id);
}

void Ibis::GetConnectX_2_ENtDevIds(device_id_list_t& dev_ids_list)
{
    for (unsigned int i = 0; i < arrsize(connectX_2_ENt_devices); i++)
        dev_ids_list.push_back(connectX_2_ENt_devices[i].id);
}

void Ibis::GetConnectX_2_LxDevIds(device_id_list_t& dev_ids_list)
{
    for (unsigned int i = 0; i < arrsize(connectX_2_Lx_devices); i++)
        dev_ids_list.push_back(connectX_2_Lx_devices[i].id);
}
