/*
 *  Copyright (c) 2004-2013 Mellanox Technologies LTD. All rights reserved
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */


#include <stdio.h>
#include "tool_trace_spinlock.h"


/******************************************************/
int tt_spinlock_init(tt_spinlock_t * const p_spinlock)
{
#ifndef WIN32
    if (pthread_mutex_init(&p_spinlock->mutex, NULL))
        return -1;
#else
    p_spinlock->mutex = CreateMutex(NULL,		/* default security descriptor */
            FALSE,		/* not to acquire on creation */
            NULL);		/* object name: unnamed */
    if (!p_spinlock->mutex)
        return -1;
#endif          /* ndef WIN32 */
    return 0;
}


/******************************************************/
void tt_spinlock_destroy(tt_spinlock_t * const p_spinlock)
{
#ifndef WIN32
    pthread_mutex_destroy(&p_spinlock->mutex);
#else
    CloseHandle(p_spinlock->mutex);
#endif          /* ndef WIN32 */
}


/******************************************************/
int tt_spinlock_acquire(tt_spinlock_t * const p_spinlock)
{
#ifndef WIN32
    if (pthread_mutex_lock(&p_spinlock->mutex))
        return -1;
#else
    if (WaitForSingleObject(p_spinlock->mutex, INFINITE) != WAIT_OBJECT_0)
        return -1;
#endif          /* ndef WIN32 */
    return 0;
}


/******************************************************/
int tt_spinlock_release(tt_spinlock_t * const p_spinlock)
{
#ifndef WIN32
    if (pthread_mutex_unlock(&p_spinlock->mutex))
        return -1;
#else
    if (ReleaseMutex(p_spinlock->mutex) == 0)
        return -1;
#endif          /* ndef WIN32 */
    return 0;
}

